//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*        Name:  coproc.h                                                       *
//*                                                                              *
//*      Author:  Daniel F. Ramirez                                              *
//*                                                                              *
//*        Date:  06/13/2003                                                     *
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*                                                                              *
//*     This is a Master Math Co-Processor application that uses a PIC18F8720 to * 
//* send expressions to the embedded Math Co-Processor using the SPI interface,  *
//* in order to compute floating point arithmetic expressions in RPN format.  It *
//* can be used for Sumo Robots that require floating point for navigation and   *
//* tracking of an oponent with minimal space, power and weight requirements are *
//* prime candidates for using this co-processor, since it is very compact.      *
//* The master can be a PIC or STAMP or any other micro-controller that requires *
//* a math co-processor and has at least 4 spare I/O pins. In addition to the    *
//* basic arithmetic functions, the math co-processor can also evaluate          *
//* expressions that contain trigonometric and scientific functions.             *
//*                                                                              *
//*      A communication interface is provided that allows the host or SPI or    *
//* I2C Master to request individual sensor readings, average sensor readings,   *
//* calibrating a sensor or perform a sensor processing algorithm using command  *
//* message packets.  The commands available are described below.  This header   *
//* file provides the common Master and Slave message formats and descriptors.   *
//*                                                                              *
//*                                                                              *
//* The high priority ISR will invoke the "Z;" reset command via the serial      *
//* interface. Both the serial interface and the SPI interface are supported by  *
//* defining the approprate compiler flag.                                       *
//*                                                                              *
//*                                                                              *
//*     This SPI slave controller can be used with a PC host or any SPI master,  *
//* in order to  give any PC the capability to serve as an SPI controller.       * 
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each SPI device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application will include implementing various    *
//* filters including FIR, FFR, Kalman and Median filters.  Both FFT and IFFT    *
//* functions and a Histogram function are also in the works. Advanced algorithms*
//* such as Vector Force Fields (VFF), Fuzzy Logic and Neural Networks are new   *
//* areas that may be explored using this application.   In addition, this       *
//* application may be able to take advantage of the new DSPic Signal Processors *
//* (DSP's) become available from Microchip.                                     *
//*                                                                              *
//* Usage:                                                                       *
//*	The following Math Co-Processor Controller command messages are available:   *
//*                                                                              * 
//* calculate    - Evalueates the RPN expressions that are stored in the         *
//* message expression buffer.  Multiple expressions can be sent to the          *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.      *
//* Large expressions can span multiple messages to the co-processor.            *
//*                                                                              * 
//* get_answers  - Get the results from the latest arithmetic operations         *
//* carried out during the last cycle. Up to 7 separate single precision floating*
//* point and 3 double precision  values may be returned during a single message *
//* transfer.                                                                    *
//*                                                                              *
//* clear_memory - Erases the contents of the Math co-processor's storage        * 
//* registers. These same registers are used for storing and retrieving temporary*
//* values. It clears floating point registers 0..MAX_FLOAT_REGISTERS.           *
//*                                                                              *
//* store   - Stores a floating point value to a selected floating point         *
//* register. The register ID range is 0..MAX_FLOAT_REGISTERS.                   *                                                            
//*                                                                              *
//* recall 	- Retrieves a floating point value from the selected floating        *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.             *                                                         
//*                                                                              *                                                                     
//*	GetStatus - Get the Math Co-processor status including current state, and    *  
//* any error conditions encountered i.e. division by 0, overflow, underflow.    *
//*                                                                              *
//* ResetCommand - Reset the DC motor Controller (brake the motor).  This        *
//* command can be used to implement an emergency stop function.                 *  
//*                                                                              *
//*                                                                              * 
//*                                                                              *
//********************************************************************************
    
    
#ifndef COPROC_LOADED  /* prevent multiple loading */
#define COPROC_LOADED


#include "flags.h"                                    // Global flags used for MPSIM debugging 
#include "stdio.h"                                    // I/O support functions  
#include "network.h"                                  	// Include support for networking


#define PARALLEL_PORT_INTERFACE							// Use Parallel SLave Port interface
 
//#define SPI_BYTE_INTERFACE   							// SPI Byte I/O
//#define SPI_BUFFER_INTERFACE  							// SPI Buffered I/O (using SPI MSSP interrupt)
//#define SPI_NETWORK_INTERFACE  						// SPI Message Passing I/O (using SPI MSSP interrupt)

//#define I2C_BYTE_INTERFACE   							// I2C Byte I/O
//#define I2C_BUFFER_INTERFACE  						// I2C Buffered I/O (using I2C MSSP interrupt)
//#define I2C_NETWORK_INTERFACE  						// SPI Message Passing I/O (using I2C MSSP interrupt)

//#define USART_BYTE_INTERFACE   						// USART Serial Byte I/O
//#define USART_BUFFER_INTERFACE  						// USART Buffered I/O (using I2C MSSP interrupt)
//#define USART_NETWORK_INTERFACE  						// USART Message Passing I/O (using I2C MSSP interrupt)

// Common Constants

#define INPUT 1											// Pin input 
#define OUTPUT 0										// Pin output

#define ENABLE 0										// Enable the Slave device
#define DISABLE 1										// Disable the Slave device
#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value

#define ON  1                                         	//  read operation
#define OFF 0                                         	// SPI write operation

//#define READ  1                                       // SPI read operation
//#define WRITE 0                                       // SPI write operation

#ifdef SINGLE_PRECISION
#define MAX_FLOAT_REGISTERS 20							// Ten single precision floating point registers
#define MAX_ANSWERS 7									// Seven single precision floating point registers
#else
#define MAX_FLOAT_REGISTERS 10							// Ten double precision floating point registers
#define MAX_ANSWERS 3									// Three single precision floating point registers
#endif
#define MESSAGE_BUFFER_SIZE 29							// Size of message buffer

// Math Co-processor enumeration types

typedef enum   {SINGLE_PRECISION, DOUBLE_PRECISION, DEGREES, RADIANS} CoprocessorModes_T;                   // Co-processor Modes
 

//**********************************************************************************
// Define messages for each type of command that is sent by the SPI Master and 
// recognized by the selected SPI Slave Math Co-processor.
//**********************************************************************************

//**********************************************************************************
//* calculate_type - Evalueates the RPN expressions that are stored in the *
//* message expression buffer.  Multiple expressions can be sent to the            *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.  Large *
//* expressions can span multiple messages to the co-processor.                    *
//**********************************************************************************
typedef struct calculate_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;           // Calculate Command to SPI Slave (see enumeration
                                	// declaration in network.h)

   byte Buffer[MESSAGE_BUFFER_SIZE];					// RPN expression to be evaluated
    
} CALCULATE_TYPE;

//**********************************************************************************
//* get_answers_type  - Get the results from the latest arithmetic operations      *
//* carried out during the last cycle. Up to 7 separate single precision floating  *
//* point and 3 double precision  values may be returned during a single message   *
//* transfer.                                                                      *
//**********************************************************************************
typedef struct get_answers_type
{
  	word MessageID;    				// Message ID range (0000 - FFFF) Hex 

  	COMMAND_TYPE Command;           // Get Answers Command to SPI Slave (see enumeration
                                	// declaration in network.h)
#ifdef SINGLE_PRECISION 
 	float Answers[MAX_ANSWERS];		// Return up to 7 single precision floating point answers
    //byte Filler[1];		 	    // Padding 
#else
 	double Answers[MAX_ANSWERS];	// Return up to 3 double precision floating point answers
    byte Filler[4];		 	        // Padding 
#endif

 	byte NumberOfAnswers; 			// Total number of answers to be obtained   

} GET_ANSWERS_TYPE;

//**********************************************************************************
//* clear_memory_type - Erases the contents of the Math co-processor's storage 
//* registers. These same registers are used for storing and retrieving temporary values.
//* It clears floating point registers 0..MAX_FLOAT_REGISTERS.
//**********************************************************************************
typedef struct clear_memory_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;           // Clear memory Command to SPI Slave (see enumeration
                                	// declaration in network.h)
    byte Filler[29];				// Padding 
    
} CLEAR_MEMORY_TYPE;

//**********************************************************************************
//* store_type - Stores a floating point value to a selected floating point        *
//* register.     The register ID range is 0..MAX_FLOAT_REGISTERS.                 *                                                                             *
//**********************************************************************************
typedef struct store_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;           // Store Command to SPI Slave (see enumeration
                                	// declaration in network.h)
  
 	byte RegisterID; 				// Selected float register ID, range (0..MAX_FLOAT_REGISTERS)  

#ifdef SINGLE_PRECISION 
 	float RegisterValue;			// The single precision floating point register value 
   	byte Filler[24];				// Padding 
#else
 	double RegisterValue;			// The double precision floating point register value 
   	byte Filler[20];				// Padding 
#endif

    
} STORE_TYPE;

//**********************************************************************************
//* recall_type - Retrieves a floating point value from the selected floating      *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.               *                                                               *
//**********************************************************************************
typedef struct recall_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;           // Recall Command to SPI Slave (see enumeration
                                	// declaration in network.h)
  
 	byte RegisterID; 				// Selected float register ID, range (0..MAX_FLOAT_REGISTERS)  
 #ifdef SINGLE_PRECISION 
	float RegisterValue;			// The floating point register value 
  	byte Filler[24];				// Padding 
 #else
 	double RegisterValue;			// The double precision floating point register value 
  	byte Filler[20];				// Padding 
#endif
    
} RECALL_TYPE;

/******************************************************************************
 * Define union of unformatted message and command message structures in order to 
 * overlay the memory. 
 ******************************************************************************/

typedef union message_union_type
{
    CALCULATE_TYPE CalculateCommand;							// Calculate command message
	GET_ANSWERS_TYPE GetAnswersCommand;							// Get answers command message
	CLEAR_MEMORY_TYPE CLearMemoryCommand;						// Clear memory command message
	STORE_TYPE StoreCommand;									// Store command message
	RECALL_TYPE RecallCommand;                          		// Recall command message	
 	GET_STATUS_COMMAND_TYPE GetStatusCommand;					// Get status message
	RESET_COMMAND_TYPE ResetCommand;					 		// Reset command message

	COMMAND_MESSAGE_TYPE SelectedCommand;						// Selected command	
	MESSAGE_TYPE Message;										// Unformatted message
} MESSAGE_UNION_TYPE;

#endif

