//********************************************************************************
//*         Name:  coprocs.c                                                     *               
//*                                                                              *
//*       Author:  Daniel F. Ramirez                                             *
//*                                                                              *
//*          Date:  06/13/2003                                                   *
//*                                                                              *
//*   Description:                                                               *
//*                                                                              *
//*                                                                              *
//*     This is a Math Co-Processor application that utilizes a PIC18F8720 to    * 
//* send expressions to the embedded Math Co-Processor using the 8-Bit Slave     *
//* parallel interface, in order to compute floating point arithmetic expressions*
//* in RPN format.  It can be used for Sumo Robots that require floating point   *
//* for navigation and tracking of an oponent with minimal space, power and      *
//* weight requirements are prime candidates for using this co-processor, since  *
//* it is very compact. The master can be a PIC or STAMP or any other            *
//* micro-controller that requires a math co-processor and has at least 4 spare  *
//* I/O pins. In addition to the basic arithmetic functions, the math            *
//* co-processor can also evaluate expressions that contain trigonometric and    *
//* scientific functions.                                                        *
//*                                                                              *
//*      A communication interface is provided that allows the host to use high  *
//* speed 8-Bit slave parallel port, asynchronous serial link or SPI or I2C      *
//* Master to request floating point equation calculations or to pass it         *
//* individual sensor readings, average sensor readings, calibrating a sensor or *
//* to perform a sensor processing algorithm using command message packets.  The *
//* commands available are described below.  This header file provides the common*
//* Master and Slave message formats and descriptors.                            *                                           *
//*                                                                              *
//*                                                                              *
//* The high priority ISR will invoke the "Z;" reset command via the serial      *
//* interface. Both the serial interface and the SPI interface are supported by  *
//* defining the approprate compiler flag.                                       *
//*                                                                              *
//*                                                                              *
//*     This slave controller can be used to communicate with a PC host or any   *
//* USART, I2C or SPI master, in order to  give the microcontroller the          *
//* capability to serve as an SPI or I2C slave controller in order to provide it * 
//* the floating point coprocessor functions.                                    *  
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each I2C device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application will include implementing various    *
//* filters including FIR, FFR, Kalman and Median filters.  Both FFT and IFFT    *
//* functions and a Histogram function are also in the works. Advanced algorithms*
//* such as Vector Force Fields (VFF), Fuzzy Logic and Neural Networks are new   *
//* areas that may be explored using this application.   In addition, this       *
//* application may be able to take advantage of the new DSPic Signal Processors *
//* (DSP's) become available from Microchip.                                     *
//*                                                                              *
//* Usage:                                                                       *
//*	The following Math Co-Processor Controller command messages are available:   *
//*                                                                              * 
//* calculate    - Evalueates the RPN expressions that are stored in the         *
//* message expression buffer.  Multiple expressions can be sent to the          *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.      *
//* Large expressions can span multiple messages to the co-processor.            *
//*                                                                              * 
//* get_answers  - Get the results from the latest arithmetic operations         *
//* carried out during the last cycle. Up to 7 separate single precision floating*
//* point and 3 double precision  values may be returned during a single message *
//* transfer.                                                                    *
//*                                                                              *
//* clear_memory - Erases the contents of the Math co-processor's storage        * 
//* registers. These same registers are used for storing and retrieving temporary*
//* values. It clears floating point registers 0..MAX_FLOAT_REGISTERS.           *
//*                                                                              *
//* store   - Stores a floating point value to a selected floating point         *
//* register. The register ID range is 0..MAX_FLOAT_REGISTERS.                   *                                                            
//*                                                                              *
//* recall 	- Retrieves a floating point value from the selected floating        *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.             *                                                         
//*                                                                              *                                                                     
//*	GetStatus - Get the Math Co-processor status including current state, and    *  
//* any error conditions encountered i.e. division by 0, overflow, underflow.    *
//*                                                                              *
//* ResetCommand - Reset the DC motor Controller (brake the motor).  This        *
//* command can be used to implement an emergency stop function.                 *  
//*                                                                              *
//*                                                                              * 
//*                                                                              *
//********************************************************************************
    
 
// Initialize Calculator compilation flags here

//#define SINGLE_PRECISION                               // Include Single Precision code

//#define TEST                                           // Test code

// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18F8720
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
 
#include <ctype.h>
                        
#ifdef SERIAL_DRIVER
#include "stdio.h"                                      // Header file for stdio file
#endif

#include "network.h"                                    // Include SPI Network Support
#include "spislave.h"                                   // Include SPI Slave Services
#include "coproc.h"										// Include Common Math Co-processor header
#include "coprocs.h"									// Include Math Co-processor Services header
#include "math.h"
#include "constant.h"									// Jmath constants 
#include "jmath.h"										// Jmath headers

// Constants

#define TRUE 1
#define FALSE 0


// Port A

#define SS			PORTFbits.RF7						// SPI Slave Co-processor Select/Status
#define SS_DIR		TRISFbits.TRISF7					// SPI Slave Co-processor Select Direction/Status 
 
// Port B

#define TRACE0		PORTBbits.RB0						// Trace bit 0 for logic analyzer
#define TRACE0_DIR	TRISBbits.TRISB0

// Port C

#define SPI_SCL_DIR TRISCbits.TRISC3 					// SPI Clock Direction (Slave)
#define SPI_SI_DIR TRISCbits.TRISC4 					// SPI Input Direction (Slave)
#define SPI_SO_DIR TRISCbits.TRISC5 					// SPI Output Direction (Slave)
#define SPI_SCL PORTCbits.RC3 							// SPI Clock Direction (Master)

// PORT D definitions

#define LeftEmitter PORTDbits.RD0							  // Left IR emitter (used for strobe)
#define RightEmitter PORTDbits.RD1							  // Right IR emitter (used for strobe)


#define INPUT 1											 // Port input direction 
#define OUTPUT 0										 // Port output direction 

#ifdef PARALLEL_PORT_INTERFACE

#define BUSY					 PORTBbits.RB0          // Slave BUSY Status Pin (1=Busy, 0=Finished)
#define BUSY_DIR				 TRISBbits.TRISB0       // Slave BUSY Status Pin Direction  

#define RD_BAR					 PORTEbits.RE0          // RD_BAR Control Pin 
#define WR_BAR                   PORTEbits.RE1          // WR_BAR Control Pin   
#define CS_BAR                   PORTEbits.RE2          // CS_BAR Control Pin  

#define RD_BAR_DIR				 TRISEbits.TRISE0       // RD_BAR Control Pin Direction
#define WR_BAR_DIR               TRISEbits.TRISE1       // WR_BAR Control Pin Direction 
#define CS_BAR_DIR               TRISEbits.TRISE2       // CS_BAR Control Pin Direction 


// Bit definitions for flag register
 
typedef struct slave_port_flags
{
        unsigned ERR:1;				// err  	equ	00h		;Error flag bit
        unsigned OUTRDY:1;			// OUTRDY	equ	01h		;Output data ready flag
        unsigned INFULL:1;			// INFULL	equ	02h		;Input data received flag
        unsigned SPARE1:1;
        unsigned SPARE2:1;
        unsigned SPARE3:1;
        unsigned SPARE4:1;
        unsigned SPARE5:1;
 
} SLAVE_PORT_FLAGS ;

// Register definitions

static volatile SLAVE_PORT_FLAGS FlagReg;			// FLAGREG	equ	20h		;Flag bit register
byte OutData;						// OUTDATA	equ	21h		;Output data
byte InData;						// INDATA	equ	22h		;Input data
//int	Count = 0;						// COUNT	equ	23h		;Count of times output register read

#endif

// Variables

int Status = 0;											// Status return code

byte ControllerState; 		// State of selected slave controller, range (0..255) 
											// 		0 => NO_STATES		No States    
											//		1 => INITIALIZED	The selected slave contoller is Initialzed 
											//		2 => DONE 	 	 	The selected slave contoller is done processing current command
											// 		3 => BUSY 	 		The selected slave contoller is busy processing current command 
											// 		4 => ERROR 	 		The selected slave contoller encountered an error processing the current command 

byte ControllerErrorCode; 	// Error code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_ERRORS 		No error codes  
											// 		1 => FAILURE 	 	Failed to complete command
											//  	2 => SUCCESS  	 	Completed command successfully 
 
byte ControllerDiagnosticCode; 	// Diagnostic message code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_DIAGNOSTICS 	No Diagnostic messages
											//  	1 => DIAGNOSTIC 1  	Diagnostic message 2 
											// 		2 => DIAGNOSTIC 2 	Diagnostic message 1
 
// Declare SPI Slave Buffer variables here

byte SPI_Rx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Receive Buffer
byte SPI_Tx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Transmit Buffer
byte SPI_Start_Buffer=0;                       			// Start pointer for ring buffer
byte SPI_End_Buffer = 0;								// SPI End Buffer Index
byte SPI_Rx_Flag = FALSE;                        		// Indicates that data was received into the ring buffer
word SPI_Timeout = 0;                            		// SPI I/O Timeout counter
byte SPI_Timeout_Flag;                           		// SPI Timeout flag
word SPI_Number_Characters = 0;						 	// SPI Number of characters in buffer
int SPI_Communications_Error;                    		// SPI I/O error flag
                                                 		// -1 = SPI Input Bufer overrun error
                                                 		// -2 = SPI Communications timeout error
                                                 		// -3 = SPI buffer empty error

int SPI_Interrupt_Count = 0;                   			// Number of MSSP interrupts
int SPI_Data_In = 0;									// Data byte read from the SPI Master
int SPI_Data_Out = 0;									// Data byte sent to the SPI Master
int SPI_Status;											// SPI Status returned from routines

int Dummy;												// Dummy Data byte read from the SPI bus

static char TheCommand;									// The Command byte read from the SPI bus

byte AddressPointer;                         	 		// SPI Device Address pointer used to simulate serial
                                                 		// EEPROM reads and writes
byte EEPromPointer;                          	 		// Pointer to EEPROM array

static int Index = 0;                            		// SPI Ring Buffer index

static byte CurrentState = 0;                    		// Current SPI state variable


// Declare global variables here

#define BufferSize		20			// Floating point buffer size
#define StackSize		 5			// HP-45 stack (5 levels)
#define RegisterSize	10			// HP-45 Registers (1-9)

#define X_LOCATION 0				// X Location in stack
#define Y_LOCATION 1				// Y Location in stack
#define Z_LOCATION 2				// Z Location in stack
#define T_LOCATION 3				// T Location in stack (top of stack)

//************************************************************************************************

 
// General Math Coprocessor functions

#define GOLD    'a'					// Gold function key    
#define ENTER	'b'                 // Enter key  
#define ROLL    'c'					// Roll key   
#define SWAP    'd'					// Swap X with Y key  
#define STORE   'e'					// Store X into register N, N=1..9   
#define RECALL  'f'					// Recall register N, N=1..9 into X  
#define LASTX   'g'                 // Recall register Last X into X

// Math Operators 

#define PLUS	'+'					// "+" Operator
#define MINUS	'-'					// "-" Operator
#define TIMES	'*'					// "*" Operator
#define DIVIDE	'/'					// "/" Operator

// Math functions


// Trigonometric Functions

#define SIN		'A'					// Sine function
#define ASIN  	'B'					// Tangent function
#define COS		'C'					// Cosine function
#define ACOS  	'D'					// Arc Cosine function
#define TAN		'E'					// Tangent function
#define ATAN	'F'					// Arc Tangent function
#define ATAN2	'G'					// Arc Tangent Two function
 
// Scientific Functions

#define SQRT	'H'					// Square root function
#define SQR		'I'					// Square function
#define EXP		'J'					// Exponential function E**X
#define LOG		'K' 				// Log function
#define log10	'L' 				// Log base 10 function
#define POWER	'M' 				// X**N  function
#define MAX		'N'                 // Maximum of x and y
#define MIN		'O'                 // Minimum of x and y
#define ABS		'P'                 // Absolute value of x
#define SIGN	'Q'                 // Sign of x and y

// Special functions

#define RAD 	'R'					// Convert to radians
#define DEG 	'S'					// Convert to degrees
#define FIX 	'T'					// Rounds x towards 0
#define FLOOR 	'U'					// Rounds x towards - Infinity
#define CEIL 	'V'					// Rounds x towards + Infinity
#define ROUND 	'W'					// Round x to nearest integer
#define PRECISION 	'X'				// Specify maximum number of digits precision up to machine precision used for display purposes
#define MOD		'Y'					// Mod Function identical to C mod function 


//************************************************************************************************

typedef union message_union_type
{
    CALCULATE_TYPE CalculateCommand;							// Calculate command message
	GET_ANSWERS_TYPE GetAnswersCommand;							// Get answers command message
	CLEAR_MEMORY_TYPE CLearMemoryCommand;						// Clear memory command message
	STORE_TYPE StoreCommand;									// Store command message
	RECALL_TYPE RecallCommand;                          		// Recall command message	
 	GET_STATUS_COMMAND_TYPE GetStatusCommand;					// Get status message
	RESET_COMMAND_TYPE ResetCommand;					 		// Reset command message

	COMMAND_MESSAGE_TYPE SelectedCommand;						// Selected command	
	MESSAGE_TYPE Message;										// Unformatted message
} MESSAGE_UNION_TYPE;

// Define the Math Co-processor Commands message
MESSAGE_UNION_TYPE CoProcessorMessage;
  
#ifdef SINGLE_PRECISION
float Registers[MAX_FLOAT_REGISTERS];
float Answers[MAX_ANSWERS];
#else
double Registers[MAX_FLOAT_REGISTERS];
double Answers[MAX_ANSWERS];
#endif

char af1[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
/*
char Period[] = ".";
char Plus[] = "+";
char Minus[] = "-";
char Blank[] = " ";
char E[] = "E";

char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 
//extern char Operators[];                               // Math operators declared in keypad.c

char NumericChars[] = {'.','E',0};                       // Numeric characters
*/
unsigned char DisableLift = FALSE;								 // Disable the lift flag

 
static byte SyncFlag = FALSE;							 //  SPI Master/Slave Sync flag
static byte GetStatusFlag = FALSE;						 //  SPI Master/Slave Get Status flag
static byte AnswerIndex = 0;								// Index into co-processor answers buffer
extern byte ReceivedDataFlag ;
 

/*
// Power of 10 lookup tables used by the pow10 function

#ifdef PIC18F8720
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const rom double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const rom double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E10 };
#else
//     10 **                 0     1      2      3     4       5      6      7      8      9      10
const double PlusPowerOf10[11] = {1.0, 1.0E1, 1.0E2, 1.0E3, 1.0E4, 1.0E5, 1.0E6, 1.0E7, 1.0E8, 1.0E9, 1.0E10 };
const double MinusPowerOf10[11] = {1.0, 1.0E-1, 1.0E-2, 1.0E-3, 1.0E-4, 1.0E-5, 1.0E-6, 1.0E-7, 1.0E-8, 1.0E-9, 1.0E-10 };
#endif
*/
// Declare function headers here

void main(void);
void InitializeMathCoprocessor(void);
void ServiceHighInterrupt(void);
 
// HP-45 Functions

void EvaluateRPN(void);
double Calculator(double Argument1, char Operator, double Argument2);
void Lift(void);
void Drop(void);
void Roll(void);
void Swap(void);
 

// Floating point conversion functions

void Normalize(double *Value, int *Exponent);
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits);

#ifdef SINGLE_PRECISION
void FloatToAscii(float Number, char * s);
void AsciiToFloat(float Number, char * s);
#endif
void DoubleToAscii(double Number, char * s);
void AsciiToDouble(double *Number, char * s);
int Member(char Ch, char * Set);

float GetFloat(void);
double GetDouble(void);
//double log10(double x);
double pow10(int x);
void ConfigureInterrupts(void);
char GetKey(void);

//char TheOperator;        // Current operator function key: "+","-","*","/"
//char TheEqualSign;       // Equal sign, "="

double Argument1;
double Argument2;
double Result;

// Calculator specific variables

extern char Buffer[BufferSize];				// Floating point Buffer
char StackLabel[]={'X','Y','Z','T'};		// Stack register labels
double Stack[StackSize];					// HP-45 Stack 
double Registers[RegisterSize];				// HP-45 Registers 1-9
double X = 0.0;								// X register (Display)
double Y = 0.0;								// Y register
double LastX = 0.0;							// Last X register
byte RegisterNumber = 0;					// Register number 1..9
//int CharacterCount = 0;								// Character count
int BufferCount = 0;								// Buffer count
int MessageBufferIndex = 0;							// Message buffer index
 
byte KeyValue; 
extern char Ch;
double my_x;
byte TheCharacterRead;

 

// Interrupt service routines for coprocs.c

 
 
//******************************************************************************
//* main - Slave Co-processor application accepts expressions and commands from the 
//* master controller, evaluates them and sends back the answers. 
//******************************************************************************
#ifdef SLAVE_COPROCESSOR

 
void main (void)
{

	int i;                                   // Index 
	byte Ch;
	
	
#ifdef STDIO_INTERRUPTS	
	ReceivedDataFlag =  FALSE;
#endif

 	NumberCharacters = 0;							// Initialize the number of characters read from the message buffer
	BufferCount = 0;								// Buffer count
	AnswerIndex = 0;								// points to current answer in Answer[] array				

	// Initialize math test package 

	InitializeMath();

	// Initialize the Math Co-processor 
	
	  InitializeMathCoprocessor();


  // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
   //InitializeUsart();
 
  

 
// Initialize test data messages

#ifdef DEBUG
	SPI_Rx_Flag = TRUE;													// Flag indicating that a message was received
  	CoProcessorMessage.CalculateCommand.MessageID = 1;    				// Message ID (0000 - FFFF) Hex

  	CoProcessorMessage.CalculateCommand.Command = CALCULATE_COMMAND;           // Send Calculate Command to math coprocessor   (see enumeration
      
                              								// declaration in network.h)

	// Copy test buffer 
    for (i=0; i<strlen(InputBuffer); i++)
	{
    	CoProcessorMessage.CalculateCommand.Buffer[i] = InputBuffer[i];					// RPN expression to be evaluated: "12.0E2.0/...";
		//Ch = (char) CoProcessorMessage.CalculateCommand.Buffer[i];
 	}
 	 
#endif

    while(1)
   {
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
//#ifdef SPI_BUFFER_INTERFACE
#ifdef USART_BUFFER_INTERFACE

   		// Read a message, start by looking for sync bytes which indicate the start of a message
	   	 
   	   	//while ((TheCharacterRead = get_byte()) != SYNC_BYTE);
  	   	//while ((TheCharacterRead = ReadHexByte()) != SYNC_BYTE);

		// Read second sync byte

        //if (get_byte() == SYNC_BYTE)
        //{
			//for (i=1; i<SPI_BUFFER_SIZE; i++)

            i = 0;
			while (i<SPI_BUFFER_SIZE)
			{
		
 				// Now read the complete message sent by the Master

    			//if (DataRdy1USART())
    			//if (ReceivedDataFlag)
    			//{
       				// Read a single byte from the SPI Bus (SSPBUF) into the serial 
       				// input ring buffer.
	
					//SPI_Rx_Buffer[i] = get_byte();
	
					TheCharacterRead = ReadHexByte(); 
					if (TheCharacterRead != 0)
					{
						SPI_Rx_Buffer[i++] = TheCharacterRead;
					}	

					//ReceivedDataFlag = FALSE;
			//	}
				
			}

			// Send ACK back to the Master
            // ************************* Missing logic *****************************

			SPI_Rx_Flag = TRUE;													// Indicates that a complete message was read and is valid
/*
		}
		else
		{
			// Flush buffers and start all over...
            // or 
			// Reset the Controller by allowing the Watchdog Timer to timeout.
	
			printf("WATCHDOG is Resetting the Math Coprocessor !!!\r\n");
			while (1);			
		}
*/
#endif

   		if (SPI_Rx_Flag == TRUE)
   		{
   			SPI_Rx_Flag = FALSE;
   			



	//******************************************************************************
	// Process Math Co-processor Commands here
	//******************************************************************************

 
   		
    		// Change controller state to BUSY
			ControllerState = BUSY; 
				  	
       		// Process the selected command
			        	
 			switch (CoProcessorMessage.SelectedCommand.Command)
 			{
				case CALCULATE_COMMAND:
				{
 					// Process the Calculate command message that evaluates RPN expression
 					ProcessCalculateCommand();
 				
					break;
				} 
			
				case GET_ANSWERS_COMMAND:
				{
		
 					// Process Get Answers command message in order to return the floating point answers to the master
  					ProcessGetAnswers();
  					
  					break;
				}
			
				case CLEAR_MEMORY_COMMAND:
				{
  					// Process the Clear Memory Command message that erases the floating point registers
					ProcessClearMemoryCommand();
					
 					break;
				}
			
				case STORE_COMMAND:
				{

 					// Process the Store Command message that saves a floating point value to a selected float register
					ProcessStoreCommand();
					
   					break;
				}
			
				case RECALL_COMMAND:
				{

  					// Process the Recall Command message that  Retrieves a floating point value from a 
  					// selected float register.	
  					ProcessRecallCommand();
   					break;
				}
#ifdef NEW_CODE
				case FILTER_COMMAND:
				{
  					// Process the Filter command message that performs selected filter 
  					// operation (FIR, IFR)

					ProcessFilterCommand();
    				break;
				}
				
				case FFT_COMMAND:
				{
  					// Process the FFT command message that performs the selected 
  					//FFT operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case IFFT_COMMAND:
				{
  					// Process the Inverse Filter command message that performs an inverse
  					// FFT operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case KALMAN_FILTER_COMMAND:
				{
  					// Process the Kalman Filter command message that performs a Kalman
  					// Filter on the specified data

					ProcessKalmanFilterCommand();
    				break;
				}
				
				case MEDIAN_FILTER_COMMAND:
				{
  					// Process the Median Filter command message that performs a Median filter 
  					//operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case HISTOGRAM_COMMAND:
				{
  					// Process the Histogram command messagePerforms a Histogram operation

					ProcessHistogramCommand();
    				break;
				}
				
				case SORT_COMMAND:
				{
  					// Process the Sort command message that performs the selected Sort operation 
  					//on the specified data

					ProcessSortCommand();
    				break;
				}
#endif			
				case GET_STATUS_COMMAND:
				{
  					// Process the Get Status Command message

					ProcessGetStatusCommand();
					
   					break;
				}
				
				case RESET_COMMAND:
				{
  					// Process the Reset controller command message

					ProcessResetCommand1();
					
   					break;
				}

			}
			
	  		// Change controller state to DONE
			ControllerState = DONE;
			 	  
        	// Copy the entire Co-Processor Response to the SPI_Tx_Buffer for transmission to the SPI Master
        	// during the next message sent from the Master.  This 1 cycle delay allows the Math CO-processor 
        	// complete its task and provide the requested results.
          		 			
  			//memcpy(SPI_Tx_Buffer+1, &CoProcessorMessage.Message.Data, DATA_BUFFER_SIZE);  
  			
				
 

#ifdef TEST  
			for (i=0; i<SPI_Number_Characters; i++)
			{
				SPI_Tx_Buffer[i] = DATA_BUFFER_SIZE-i;
			}
			 			
			printf("Slave Co-processor application, SPI_Interrupt_Count = ");
			dec(SPI_Interrupt_Count);
			printf("\r\n");
			
			// Display contents of SPI Send and Receive Buffers 	
			
			for (i=0; i<SPI_BUFFER_SIZE; i++)
			{
				printf("Math Co-processor: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);
			}		

       		printf("Slave: Math Co-processor received the command:.");
#endif       
			
 /*        				
			// Display the current contents of the buffer
			for (i=0; i<SPI_Number_Characters; i++)
			{
				printf("Slave: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);

			}

*/

//*******************************************************************************
//#endif

 
 		}
 		
//#endif 		
 		
  		// Check to see if the SPI Co-Processor Slave Select is asserted (=0).
      	// Disable the SPI interrupts if it is not.
/*
      	if (!SS)
      	{
      		// Enable SPI MSSP interrupts

			EnableInterrupts();
      	}
      	else
      	{
 		 	// Disable SPI MSSP interrupts

			DisableInterrupts();
     	}
*/

		}
    
/*        
   while (1)
   {
   	// Clear the BF and SSPIF flag bits by reading the SSPBUF register

   	Data = SSPBUF;  
    
   	PIR1bits.SSPIF = 0;  
   	
   	// Wait for receive to end
      
      while (SSPSTATbits.BF);
      
      // Check if last byte received was a data byte
      
      if (SSPSTATbits.D_A)
      {
      	Data = SSPBUF;
      }
      printf("Data = ");
      dec(Data);
      printf("\r\n");
          
 		//PORTB = Data;                                // Write value to PORTB LED(s)
 	}
   
 
   while (1)
   {
 		PORTB = CurrentState;                                // Write value to PORTB LED(s)
      
      printf("Slave: SPI_Interrupt_Count = ");
      dec(SPI_Interrupt_Count);
      printf("\r\n");
 		
      printf("Slave: CurrentState = ");
      dec(CurrentState);
      printf("\r\n");
     
   }

 

   for (i=0; i<255; i++)
   {
       printf("This is a test ");
       dec(i);
       printf(" times ! \r\n");
       PORTB = i;
       pause(500);
   }
*/
    
/*   
   printf("PIC18C452 SPI Slave Controller Application... \r\n");
   
   // Test LED(s)
   
   for (i=0; i<256; i++)
   {
    	PORTB = i;
   		pause(200);
   	
      printf("Slave: i = ");
      dec(i);
      printf("\r\n");
      
   }
*/   
  


	// close the spi port

	CloseSPI();

	// Done.

	while(1);

 
 
}



//******************************************************************************
//* EvaluateRPN - Evaluates an RPN Expression in order to handle more complicated
//* arithmetic expressions. It does not currently support trigonometric or scientific 
//* functions.
//******************************************************************************
void EvaluateRPN(void)
{
		char Ch;					// Local character 

 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
				
		// Get a token character (operator or numeric) from the message buffer 
 
   		while ((NumberCharacters <  MESSAGE_BUFFER_SIZE) && (NumberCharacters < strlen(CoProcessorMessage.CalculateCommand.Buffer)))
   		{
 
	 
			Ch = CoProcessorMessage.CalculateCommand.Buffer[NumberCharacters++];					// Get next token from message buffer  
 
			// Check for end of statement semicolon character, ";"

 			if (Ch == ';')
			{
				// Save last answer and continue parsing the buffer until all characters in it have been processed.
                // If answers exceed the maximum number then wrap the index to 0 using the mod function.

				Answers[AnswerIndex++]= LastX;
 				AnswerIndex = AnswerIndex % MAX_ANSWERS;
				
			}

			if (isdigit(Ch) || (Ch == '.'))
			{
				// Save the key in the input buffer

				Buffer[BufferCount++] = Ch;
 			}
			else
			{
												

				if (BufferCount > 0)
				{
					// Automatically lift the stack by one unless lift has been disabled for the current operation
 				
 					Lift();
 					
					// This logic needs to be modified or moved ?????
				
					Buffer[BufferCount++] = 0;			// Terminate the string 
				
					// Convert the value stored in the buffer from ascii to float and save it in register X
 
					AsciiToDouble(&X, Buffer);
				
					// Copy number from the display register X to the Stack register X 
				
 					Stack[X_LOCATION] = X;
 				
					LastX = X;							// Save value of X into LAST_X register
					BufferCount = 0;					// Buffer count
				}	
 										
				// Interpret the HP-45 RPN command here
				
				switch (Ch)
				{
					// Stack operations

					case ENTER:						// "Enter ^" key 
					{
						// Lift stack up one level and disable automatic lift for the next operation

						Lift();
						DisableLift = TRUE;
						break;
					}

					case ROLL:						// "R !" key 
					{
						// Rotate the values in the stack while preserving all values.    

						Roll();
						break;
					}

					case SWAP:						// "X <> Y" key 
					{
						// Swap X with Y

						Swap();
						break;
					}

					case STORE:						// "STO" key 
					{
						// Store X into register N, where N=1..9

						RegisterNumber = GetKey();
						Registers[RegisterNumber] = X;
						break;
					}

					case RECALL:					// "RCL" key 
					{
						// Recall register N, where N=1..9 into X

						RegisterNumber = GetKey();
						X = Registers[RegisterNumber];
						break;
					}

					case LASTX:					// "LAST X" key 
					{
						// Recall Last X register into X

						X = LastX;
 						Stack[X_LOCATION] = X;
 						
						break;
					}

					// Basic math operations

					case PLUS:						// "+" key
					{
						// Compute the "+" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],PLUS, Stack[X_LOCATION]);
			
						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case MINUS:						// "-" key
					{
						// Compute the "-" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], MINUS, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case TIMES:						// "*" key
					{
						// Compute the "*" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], TIMES, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case DIVIDE:					// "/" key
					{
						// Compute the "/" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],DIVIDE, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}					

					// Trigonometric functions

					case SIN:					// sin(x) function  
					{
						// Compute the sin(x) and place result in the X register

						Stack[X_LOCATION] = sin(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 					
					case ASIN:					// asin(x) function  
					{
						// Compute the asin(x) and place result in the X register

						Stack[X_LOCATION] = asin(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case COS:					// cos(x) function  
					{
						// Compute the cos(x) and place result in the X register

						Stack[X_LOCATION] = cos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case ACOS:					// acos(x) function  
					{
						// Compute the acos(x) and place result in the X register

						Stack[X_LOCATION] = acos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case TAN:					// tan(x) function  
					{
						// Compute the cos(x) and place result in the X register

						Stack[X_LOCATION] = cos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case ATAN:					// atan(x) function  
					{
						// Compute the atan(x) and place result in the X register

						Stack[X_LOCATION] = atan(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					// Scientific functions ...

					case SQRT:					// sqrt(x) function  
					{
						// Compute the sqrt(x) and place result in the X register

						Stack[X_LOCATION] = sqrt(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case SQR:					// sqr(x) function  
					{
						// Compute the sqrt(x) and place result in the X register

						Stack[X_LOCATION] = sqr(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
#ifdef NEW_CODE
					case EXP:					// sqr(x) function  
					{
						// Compute the exp(x) and place result in the X register

						Stack[X_LOCATION] = exp(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case LOG:					// log(x) function  
					{
						// Compute the log(x) and place result in the X register

						Stack[X_LOCATION] = log(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}					

					case log10:					// log10(x) function  
					{
						// Compute the log10(x) and place result in the X register

						Stack[X_LOCATION] = log10(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case POWER:					// power(x,n) function  
					{
						// Compute the power(x,n) and place result in the X register

						Stack[X_LOCATION] = power(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					} 					
#endif

					case MAX:					// max(x,y) function  
					{
						// Compute the max(x,y) and place result in the X register

						Stack[X_LOCATION] = max(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case MIN:					// min(x,y) function  
					{
						// Compute the min(x,y) and place result in the X register

						Stack[X_LOCATION] = min(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case ABS:					// abs(x) function  
					{
						// Compute the abs(x) and place result in the X register

						Stack[X_LOCATION] = abs(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case SIGN:					// sign(x,y) function  
					{
						// Compute the sign(x,y) and place result in the X register

						Stack[X_LOCATION] = sign(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case MOD:					// mod(x,y) function  identical to the C mod  function
					{
						// Compute the mod(x,y) and place result in the X register

						Stack[X_LOCATION] = mod(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
					


					// Special math coprocessor functions

 					case RAD:					// rad(x) function  
					{
						// Convert degrees to radians and place result in the X register

						Stack[X_LOCATION] = radians(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case DEG:					// deg(x) function  
					{
						// convert radians to degrees and place result in the X register

						Stack[X_LOCATION] = degrees(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
 
#ifdef NEW_CODE

  					case FIX:					// fix(x) - function rounds x towards 0 
					{
						// Rounds x towards 0 and places result in the X register

						Stack[X_LOCATION] = fix(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

   					case FLOOR:					// floor(x) - Rounds x towards - Infinity 
					{
						// Rounds x towards - Infinity and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

    				case FLOOR:					// floor(x) - Rounds x towards + Infinity 
					{
						// Rounds x towards + Infinity and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
     				case ROUND:					// round(x) - Round x to nearest integer
					{
						// Rounds x to nearest integer and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
     				case PRECISION:					// precision(x) - Specify maximum number of digits precision up to machine precision used for display purposes
					{
						// Specify maximum number of digits precision up to machine precision used for display purposes

						Stack[X_LOCATION] = precison(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
#define PRE 	'X'					// 

#endif
				}
							
							
				// Copy number from the Stack register X to the display register X and to the Last X register  
				
 				X = Stack[X_LOCATION];
				LastX = X;
				
				// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

				DoubleToAscii(X, af1);

			}
		
 
		}
 		

	
 
}

//******************************************************************************
//* Calculator - Simple calculator that performs double precision floating point
//* addition, subtraction, multiplication and division.  It does not currently
//* support levels of nesting parentheses for complicated mathematical expressions.
//******************************************************************************

double Calculator(double Argument1, char Operator, double Argument2)
{
	double TheResult;
 
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	switch(Operator)
	{
		case '+':
		{
			TheResult = Argument1 + Argument2;
			break;
		}

		case '-':
		{
			TheResult = Argument1 - Argument2;
			break;
		}

		case '*':
		{
			TheResult = Argument1 * Argument2;
			break;
		}

		case '/':
		{
			TheResult = Argument1 / Argument2;
			break;
		}

		default:
		{
			TheResult = 0;
			break;
		}		

	}

	return (TheResult);

}



// HP-45 Functions

//******************************************************************************
//* Lift - Pushes the value in register X onto the simulated HP-45 Stack.  The 
//* contents of register T, at the top of the stack is lost, being replaced by
//* the contents register Z. The next number keyed into X overwrites the old X.
//******************************************************************************
void Lift(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
	// Check to make sure that the Lift operation has not been disabled 
	
	if (DisableLift)
	{
		DisableLift = FALSE;								// Enable Lift for the next operation
	}
	else
	{
 		Stack[T_LOCATION] = Stack[Z_LOCATION]; 				// Contents of T are lost
 		Stack[Z_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register Z   
 		Stack[Y_LOCATION] = Stack[X_LOCATION]; 				// Copy contents of register X to register Y  
 		//Stack[Y_LOCATION] = X; 							// Copy contents of register X to register Y
 	}  
}

//******************************************************************************
//* Drop - The rest of the stack is dropped to fill the gap left by the 
//* previous operation.  The value of the Y register is lost. 
//******************************************************************************
void Drop(void)
{
	int i;												// Local loop variable

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	//Stack[X_LOCATION] = Stack[Y_LOCATION]; 			// Contents of X are lost
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Y to register Z   
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register X to register Y  
	//X = Stack[X_LOCATION]; 							// Copy contents of register X to X  

 		
 		
	// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

	DoubleToAscii(Stack[X_LOCATION], af1);
		
	// Send result to Serial Port
	printf(" = ");
	puts(af1);
/*	
	// Display the contents of the Stack
		
	printf("\r\nStack...\r\n");
	for (i=3; i>=0; i--)
	{
 		putch(StackLabel[i]);
 		printf(": ");
		DoubleToAscii(Stack[i], af1);
		string(af1);
	}
	printf("\r\n> ");					// Calculator prompt
*/

#ifdef SERIAL_DRIVER
	// Hit any  key to continue
	Ch = getchar();                   // Serial Input   
#endif
				
#ifdef KEYPAD_DRIVER
	// Hit any  key to continue
	Ch = GetKey();                      // Keypad Input    
#endif
	
 	//lcd_clear();						// clear LCD, home cursor
 	//printf("> ");					// Calculator prompt
	
	
}

//******************************************************************************
//* Roll - 	 Rotate the values in the stack while preserving them.    
//******************************************************************************
void Roll(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Z to register Y
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register T to register Z    
 	Stack[T_LOCATION] = X; 								// Copy contents of X to register T  
    X = Stack[X_LOCATION];								// Copy contents of register X to X 
 
}

//******************************************************************************
//* Swap - Swaps the contents of register X with the contents of register Y.
//******************************************************************************
void Swap(void)
{
	double TempValue;									// Temporary storage for swap
	
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	TempValue = Stack[X_LOCATION];						// Save contents of register X
	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X   
  	Stack[Y_LOCATION] = TempValue; 						// Copy contents of TempValue to register Y     	
	X = Stack[X_LOCATION]; 								// Copy contents of register X to X  
 
}
//********************************************************************************
//* ConfigureInterrupts - Configure interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void ConfigureInterrupts(void)
{
     // Configure USART interrupts (1 = High Priority, 0 = Low Priority).

    IPR1bits.RCIP = 1;

    // Enable USART asynchronous interrupt

    PIE1bits.RCIE = 1; 

  // Enable peripheral interrupts
 
   INTCONbits.PEIE = 1;                                    

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts	   
}




//********************************************************************************
//********************************************************************************
//********************************************************************************
//********************************************************************************
 

//********************************************************************************
//* InitializeMathCoprocessor - Initializes Timers, program variables and peripheral registers
//* for the math co-processor.
//********************************************************************************
void            InitializeMathCoprocessor(void)
{
 	int i; 									// Local loop index
 
  	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
       
    // Initialize SPI device address pointer

    AddressPointer = 0;
    SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts                           

    // Initialize ring buffer to zero  
 
    for (i=0; i< SPI_BUFFER_SIZE; i++)
    {

       SPI_Rx_Buffer[i] = 0;
       SPI_Tx_Buffer[i] = 0;
    } 

     // configure port A for all digital inputs

    OpenADC(ADC_0ANA, ADC_INT_OFF);
    
    // Initialize ports  

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                        // Clear PORTC (Some Port C bits are reserved for USART)
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction for output

 
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
    //TRISC = 0x00;                                        // PORTC all outputs (Some Port C bits are reserved for USART).
    TRISD = 0x00;                                          // PORTD all outputs.
    TRISE = 0x00;                                          // PORTE all outputs.

   	// Setup timers 

	SetupTimers();  

   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous or to 19.2 KB for high speed serial link

   	InitializeUsart();


#ifdef PARALLEL_PORT_INTERFACE

//******************************************************************************
//* InitializeParallelPort - Initializes the 8-Bit Parallel Slave Port.
//******************************************************************************
 
 
	OutData = 0;					// clrf	OUTDATA		;Clear data registers
	InData = 0;						// clrf	INDATA
									// bsf	STATUS,RP0	;Select register page 1
 


	// Set Port_B to all outputs
    PORTB = 0;
 	TRISB = 0x00;					// movwf	TRISB		;Set Port_B to all outputs
 
	BUSY_DIR = OUTPUT;				   						// Set Slave BUSY Status Pin Direction  to output

    BUSY = 1;												// Let the Master know the Slave is busy initializing


	// Configure Port D and an 8 Bit Slave Parallel Port 

	//TRISEbits.PSPMODE = 1;
	PSPCONbits.PSPMODE = 1;

	// Configure PORT E for 8 Bit Slave Parallel Port
/*
	TRISE = 	0x17;				// movlw	b'00010111'	;Set RD, WR, and CS as
 	 								// movwf	TRISE	    ; inputs, Enable Parallel Slave port
 									// movlw	0FFh
*/


 	RD_BAR_DIR	= INPUT;       // RD_BAR Control Pin Direction
 	WR_BAR_DIR 	= INPUT;       // WR_BAR Control Pin Direction 
 	CS_BAR_DIR 	= INPUT;       // CS_BAR Control Pin Direction 

 
  	// Configure the ADCON1 register for digital I/O (Port D and E) using the 
	// PCFG3:PCFG0: A/D Port Configuration Control bits TO 0x0F:

	ADCON1bits.PCFG3 =  1;
	ADCON1bits.PCFG2 =  1;
	ADCON1bits.PCFG1 =  1;
	ADCON1bits.PCFG0 =  1;


	PIE1bits.PSPIE = 1;        		// movlw	b'10000000'	;
									// movwf	PIE1		;Enable Parallel Slave Port interrupt
									// bcf	STATUS,RP0	;Select register page 0

	PORTD = OutData;				// movf	OUTDATA,W	;Set output Data in PORTD
									// movwf	PORTD

	// Initialize Message buffers

 
	// Initialization Logic for SPI Slave Buffer Interface goes here ...
	for (i=0; i<SPI_BUFFER_SIZE; i++)
	{
 		SPI_Rx_Buffer[i] = 0;                  	// Clear SPI Receive Buffer 
 		//SPI_Tx_Buffer[i] = 0;               	// Clear SPI Transmit Buffer   
 		SPI_Tx_Buffer[i] = i;                  	// Initialize SPI Transmit Buffer  to a ramp 0..SPI_BUFFER_SIZE 
	}
	
 
 	
 	
	for (i=0; i<SPI_BUFFER_SIZE; i++)
	{
 		CoProcessorMessage.Message.Data[i] = 0;            			// Clear the message  
 	}
   
 	
	INTCONbits.GIE  = 1;			// movlw	b'11000000'	;Set GIE, PEIE (enable interrupts)
	INTCONbits.PEIE = 1;			// movwf	INTCON
#endif
 

#ifdef SPI_BUFFER_INTERFACE 
  
	// Initialize the SPI Slave Pin directions
	
	SPI_SCL_DIR = INPUT; 									// Set SPI Clock Direction to Input
 	SPI_SI_DIR = INPUT;										// Set SPI Input Direction  to Input
    SPI_SO_DIR = OUTPUT; 									// Set SPI Output Direction to Output

    SS_DIR = INPUT;									  		// Set Slave Select line to Input

    TRACE0_DIR   = OUTPUT;								   // Set TRACE 0 line to output

    // Turn OFF trace 0

    TRACE0 = OFF;
    
    // Configure the PIC SPI module as a slave
        
   //OpenSPI(SLV_SSOFF, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                          // SPI Mode 0,0, and Sample on falling edge.
   OpenSPI(SLV_SSON, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 0,0, and Sample on falling edge.
                                                          // SPI Mode 0,0, and Sample on falling edge.    //OpenSPI(SLV_SSON, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 0,0, and Sample on falling edge.

    //OpenSPI(SLV_SSOFF, MODE_11, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                           // SPI Mode 1,1, and Sample the middle.

    //OpenSPI(SLV_SSON, MODE_11, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 1,1, and Sample the middle.

    PIR1 = 0;

    SSPSTAT = 0;


   // Configure USART interrupts (1 = High Priority, 0 = Low Priority).

   //IPR1bits.RCIP = 1;

	// Configure interrupts (1 = High Priority, 0 = Low Priority).  Interrupt 0 is always
	// set to high priority

	//INTCON3bits.INT1IP = 1;                                // Set interrupt 1 priority to high
	//INTCON3bits.INT2IP = 1;                                // Set interrupt 2 priority to high

	// Configure MSSP interrupts (1 = High Priority, 0 = Low Priority).

   IPR1bits.SSPIP = 1;

   // Clear the BF and SSPIF flag bits by reading the SSPBUF register

   SSPBUF = 0;
    
   PIR1bits.SSPIF = 0;
    
 #endif


/*
   // Clear the BF and SSPIF flag bits by reading the SSPBUF register

    i = SSPBUF;  
    
    PIR1bits.SSPIF = 0;    
*/
 
	SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts
	Index = 0;                               // SPI Ring Buffer index
	SPI_Rx_Flag =  FALSE;
	NumberCharacters = 0;

	CurrentState = 0;                        // Current SPI state variable
 
	// Configure interrupts for serial I/O and other peripherals that need them
	
	//ConfigureInterrupts();
 
      // Wait 5 seconds 
/*      
      printf("5 Second Test ...\r\n");
      pause(5000);
      printf("End of 5 Second Test\n");
*/

	// Initialize Serial LCD Display here
	
	//printf("\r8-N-1");  
	 
	//printf("DIY Calculator \r\n");	
	
	// Test the Ascii to double conversion function
/*	
	af1[0] = '9';
	af1[1] = '8';
	af1[2] = '7';
	af1[3] = '.';
	af1[4] = '6';
	af1[5] = '5';
	af1[6] = '4';
	af1[7] = 0;
	
	AsciiToDouble(&X, af1);
	

   Enter();
*/	
/* 
	Buffer[0] = '0';
	Buffer[1] = '0';
	Buffer[2] = '0';
	Buffer[3] = '0';
	Buffer[4] = '0';
	Buffer[5] = '0';
	Buffer[6] = '0';
	Buffer[7] = 0;

	Buffer[0] = '3';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
	
	Buffer[0] = '4';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
*/	

	// Initialize stack 
	
   Stack[X_LOCATION] = 0.0;
   Stack[Y_LOCATION] = 0.0;
   Stack[Z_LOCATION] = 0.0;
   Stack[T_LOCATION] = 0.0;


	// Test stack operations 
	
   Stack[X_LOCATION] = 1.0;
   Stack[Y_LOCATION] = 2.0;
   Stack[Z_LOCATION] = 3.0;
   Stack[T_LOCATION] = 4.0;
    
	// Make sure we can perform a stack lift initially
	
    DisableLift = FALSE;
    AnswerIndex = 0;
   	MessageBufferIndex = 0;
   	   
	// Initialize Math Co-processor Controller State, Error codes and Diagnostic codes
	
	ControllerState = INITIALIZED; 	  
   	ControllerErrorCode =  NO_ERRORS;	   
  	ControllerDiagnosticCode = NO_DIAGNOSTICS;  
}

//****************************************************************************************
//****************************************************************************************
//****************************************************************************************
//****************************************************************************************


//**********************************************************************************
//* ProcessCalculateCommand - Evalueates the RPN expressions that are stored in    *
//* the message expression buffer.  Multiple expressions can be sent to the        *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.  Large *
//* expressions can span multiple messages to the co-processor.                    *
//**********************************************************************************
byte ProcessCalculateCommand(void)  
{
 	byte CalculateCommandStatus = TRUE;							// Status return
 
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		 	
 	// Parse and Evalueate the RPN expression contained in the co-processor message buffer ...
 	
 	
 	
#ifndef TEST
    printf("slave: Calculate Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.CalculateCommand.MessageID);
   	printf("\r\n");
    printf("Buffer = !");
    string(CoProcessorMessage.CalculateCommand.Buffer);
    printf("!");
   	printf("\r\n\r\n");
#endif

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

 	// Evaluate the RPN  expression and store the answers in the Answer buffer
 	EvaluateRPN();
				
	return CalculateCommandStatus;
}

//**********************************************************************************
//* ProcessGetAnswers - Get the results from the latest arithmetic operations      *
//* carried out during the last cycle. Up to 7 separate single precision floating  *
//* point or 3 double precision  values may be returned during a single message    *
//* transfer.                                                                      *
//**********************************************************************************
byte ProcessGetAnswers(void)
{
 	byte GetAnswersStatus = TRUE;								// Status return
   	byte AnswerID;									 		    // Local Answer Loop Index 
 		
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		    				
  	// Send  latest answers from requested RPN expressions back to the master.
	CoProcessorMessage.GetAnswersCommand.NumberOfAnswers = AnswerIndex;

#ifndef TEST

	// Send test answer responses back to master (Value of PI a digit at a time)
	 
    printf("slave: Get Answers Command: \r\n");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetAnswersCommand.MessageID);
   	
	CoProcessorMessage.GetAnswersCommand.NumberOfAnswers = 7;
	Answers[0] = 3.0;
	Answers[1] = 1.0;
	Answers[2] = 4.0;
	Answers[3] = 1.0;
	Answers[4] = 5.0;
	Answers[5] = 9.0;
	Answers[6] = 2.0;
	
/*	
	CoProcessorMessage.GetAnswersCommand.Answers[0] = 3.0;
	CoProcessorMessage.GetAnswersCommand.Answers[1] = 1.0;
	CoProcessorMessage.GetAnswersCommand.Answers[2] = 4.0;
	CoProcessorMessage.GetAnswersCommand.Answers[3] = 1.0;
	CoProcessorMessage.GetAnswersCommand.Answers[4] = 5.0;
	CoProcessorMessage.GetAnswersCommand.Answers[5] = 9.0;
	CoProcessorMessage.GetAnswersCommand.Answers[6] = 2.0;
*/	
#endif
	
 	//for (AnswerID = 0; AnswerID < CoProcessorMessage.GetAnswersCommand.NumberOfAnswers; AnswerID++)
 	for (AnswerID = 0; AnswerID < 7; AnswerID++)
	{	
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
		// Copy local answers to Answer message
		CoProcessorMessage.GetAnswersCommand.Answers[AnswerID] = Answers[AnswerID];

#ifndef TEST
   		printf("  Answer ID = ");
    	dec(AnswerID);
    	printf(", Answer = ");
    	//hex4(CoProcessorMessage.GetAnswersCommand.Answers[AnswerID]);  
    	hex4(Answers[AnswerID]);  
    	printf("\r\n");
#endif
 
	}
 
	return GetAnswersStatus;
}
			
//**********************************************************************************
//* ProcessClearMemoryCommand - Erases the contents of the Math co-processor's 
//* floating point storage registers. These same registers are used for storing and
//* retrieving temporary values and answers. It does this by setting floating point 
//* registers 0..MAX_FLOAT_REGISTERS to zero.  Registers 0..10 are reserved for answers.
//**********************************************************************************
byte ProcessClearMemoryCommand(void)
{
 	byte ClearMemoryCommandStatus = TRUE;							// Status return
    byte RegisterID;									 		    // Local Loop Index
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	// Read the Pot control duty value and send it to the corresponding DC motor 
 
 	for (RegisterID = 0; RegisterID < MAX_FLOAT_REGISTERS; RegisterID++)
	{
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		Registers[RegisterID] = 0.0;
	}
	
#ifndef TEST
    printf("save: Clear Memory Command Completed.");
   	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   printf("\r\n\r\n");
#endif

	return ClearMemoryCommandStatus;
}

//**********************************************************************************
//* ProcessStoreCommand - Stores a floating point value to a selected floating     *
//* point register.     The register ID range is 0..MAX_FLOAT_REGISTERS.           *
//**********************************************************************************
byte ProcessStoreCommand(void)
{
 	byte StoreCommandStatus = TRUE;										// Status return

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
			
 	// Store the value into the selected register.
  					
 	Registers[CoProcessorMessage.StoreCommand.RegisterID] = CoProcessorMessage.StoreCommand.RegisterValue;
  	 
#ifndef TEST
    printf("slave: Store Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   	printf("  Register ID = ");
   	dec(CoProcessorMessage.StoreCommand.RegisterID);
    printf("Register Value = ");
    hex4(Registers[CoProcessorMessage.StoreCommand.RegisterID]);
    printf("\r\n\r\n");
#endif 

	return StoreCommandStatus;
}
  					
//**********************************************************************************
//* ProcessRecallCommand - Retrieves a floating point value from the selected      *
//* floating point register. The register ID range is 0..MAX_FLOAT_REGISTERS.      *                                                               
//**********************************************************************************
byte ProcessRecallCommand(void)
{
 	byte RecallCommandStatus = TRUE;									// Status return

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
 	
    // Accept number of samples xxxx to be used for calibration and data
   	// collection used for reading the DC Motor Control Pots, where the 
   	// number of samples range 0000..FFFF.
	
  	CoProcessorMessage.RecallCommand.RegisterValue = Registers[CoProcessorMessage.RecallCommand.RegisterID];
  	
#ifndef TEST
    printf("slave: Recall Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.RecallCommand.MessageID);
   	printf("  Register ID = ");
   	dec(CoProcessorMessage.RecallCommand.RegisterID);
    printf("Register Value = ");
    hex4(CoProcessorMessage.RecallCommand.RegisterValue);
    printf("\r\n\r\n");
#endif 

	return RecallCommandStatus;
}


//**********************************************************************************
//* ProcessGetStatusCommand - Process the the SPI Slave Controller Get Status 
//* command and return the latest command execution status. 
//**********************************************************************************
void ProcessGetStatusCommand(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		
    CoProcessorMessage.GetStatusCommand.ControllerState = ControllerState; 		 			// Store current co-processor State
    CoProcessorMessage.GetStatusCommand.ControllerErrorCode = ControllerErrorCode; 			// Store current co-processor Error Code
    CoProcessorMessage.GetStatusCommand.ControllerDiagnosticCode = ControllerDiagnosticCode;  // Store current co-processor Diagnostic Code

 
 #ifndef TEST
    CoProcessorMessage.GetStatusCommand.ControllerState = 3; 		 			// Store current co-processor State
    CoProcessorMessage.GetStatusCommand.ControllerErrorCode = 2; 			// Store current co-processor Error Code
    CoProcessorMessage.GetStatusCommand.ControllerDiagnosticCode = 1;  // Store current co-processor Diagnostic Code
     
    printf("Slave: Received Get Status Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   	printf("  ControllerState = ");
   	dec(ControllerState);
   	printf("  ControllerErrorCode = ");
   	dec(ControllerErrorCode);
   	printf("  ControllerDiagnosticCode = ");
   	dec(ControllerDiagnosticCode);
    printf("\r\n");
#endif

}
 			

//**********************************************************************************
//* ProcessResetCommand - Process the Reset the SPI Slave Math Co-processor.  This command 
//* resets the selected controller.  It is used to force a hard reset of the math
//* co-procesor when a fatal error is encountered, using the PIC's watchdog timer or 
//* via the "Z" command over the serial interface.
//**********************************************************************************
void ProcessResetCommand1(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
#ifndef TEST
    printf("Reset Command: ");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.ResetCommand.MessageID);
    printf("\r\n\r\n");
#endif

	// Reset the Controller by allowing the Watchdog Timer to timeout.
	
	while (1);
}
 			
