//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*        Name:  jmath.c                                                        *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  08/06/2002                                                     *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements floating point math package that includes the* 
//* trigonmetric and scientific functions using IEEE floating point format for   *
//* the Microchip PIC18F452 micro-controller.  Included is support for a version *
//* of printf that formats and displays floating point numbers to a display or   *
//* PC or laptop connected to the PIC serial port or I2C port.  Both single and  *
//* double precision floating point formats are supported using the PIC18 C      *
//* floating point math.  Results are written to a log file in Matlab format so  *
//* that they may be validated by comparing the results from the JMATH library   *
//* to those from Matlab                                                         *
//*                                                                              *
//*                                                                              *
//*      Future upgrades to this application include: Using lookup tables for    *
//* trigonometric functions, and signal processing routines such as FFT(s), FIR  *
//* FFR digital filters.  In addition the possibility for supporting a PIC       *
//* math co-processor will be explored.  In addition an NJU ??? real-time clock  *
//* will be incorporated to obtain benchmark results without overhead from timer * 
//* interrupts.                                                                  * 
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
  
// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18F452     
#include <p18f452.h>					// Register definitions for PIC18F452
#endif
  
 
#ifdef PIC18F6620     
#include <p18f6620.h>
#endif
 
#ifdef PIC18F8720     
#include <p18f8720.h>					// Register definitions for PIC18F8720
#else
#include <stdio.h>
#include <stdlib.h>
#endif

#include <ctype.h>
#include <string.h>

#ifdef PIC18F8720
#include <i2c.h>                                        // I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
//#include "portutil.h"                                 // Port utility functions
#include "myprintf.h"                                   // My version of printf I/O
#include "timeutil.h"                                   // Include time utilities
#include "math.h"                                       // Include floating point math functions
#include "stdio.h"                                      // Serial I/O support functions
#else
#include "constant.h"
#include "jmath.h"
#endif
 #include "math.h"
#include "constant.h"
#include "jmath.h"

#undef max
#undef min
#undef abs
#undef sign

#define max(x,y)    (((x) > (y)) ? (x) : (y))
#define min(x,y)    (((x) < (y)) ? (x) : (y))
#define abs(x) 	  (((x) < 0  ) ? -(x): (x))
#define sign(x,y)   (((y) < 0  ) ? (-(abs(x))): (abs(x)))

#define TRUE 1
#define FALSE 0
 
double s1, s2, s3, s4;
  
 //double tan(double x);

/* Here's a structure that allows us to hack into
 * the floating point word.
*/


typedef union{
	double fp;
	struct{
		unsigned long lo;
		unsigned long hi;
	};
} hack_structure;
 
//********************************************************************************
// Constant Definitions
//********************************************************************************

                                          					// Flag used for debug I/O
#define	PI	3.14159265358979
#define	TWO_PI	6.28318530717958
#define	HALF_PI	1.570796326794895
 
 
#define TABLE_SIZE  360                                 // Size of tables (0..359 degrees)
#define SYSTEM_FREQUENCY 20                             // System frequency, Fs = 20 MHz

// Compute timer clock frequency which depends on the system frequency.  This constant is 
// used by DelayUs function. 

#define CLOCK_FREQUENCY   (SYSTEM_FREQUENCY / 4)        //  Fc = Fs / 4 
 
#define TIME_SCALE_FACTOR 100                           // 100 micro-second granularity for delays

 
#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value

#define READ  1                                         // I2C read operation
#define WRITE 0                                         // I2C write operation

 
//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

#define INPUT    1                                     // Input pin direction
#define OUTPUT   0                                     // Output pin direction

//*******************************************************************************
// Define port bit definitions here
//*******************************************************************************

// Port C Bits

   
// PORT D Bits

#define LeftEmitter PORTDbits.RD0                                         // Left IR emitter 
#define RightEmitter PORTDbits.RD1                                        // Right IR emitter

//********************************************************************************
// Type definitions
//********************************************************************************
  
 
/*
//  File variable declarations

FILE *f1;												// Log file #1
FILE *f2;												// Log file #2
FILE *f3;												// Log file #3
FILE *f4;												// Log file #4
FILE *f5;												// Log file #5
FILE *f6;												// Log file #6
FILE *f7;												// Log file #7
FILE *f8;												// Log file #8
FILE *f9;												// Log file #9
FILE *f10;												// Log file #10
FILE *f11;												// Log file #11
FILE *f12;												// Log file #12
*/

//********************************************************************************
// Variable declarations
//********************************************************************************
 
double ConvertToRadians;													// Convert from degrees to radians  
double ConvertToDegrees; 													// Convert from radians to degrees  

byte TheCommand = 0;                                            			// The I2C Command read from host

double x, y, z, w;					// Local values 
float z1,z2,z3;

char ax[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
char ay[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
char az[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
char aw[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
		
// Pointers and arrays

//byte SinTable[TABLE_SIZE];                                           		// Store a 0..TABLE_SIZE lookup table  

 
 
//********************************************************************************
// Function Prototypes
//********************************************************************************

  
// Math Test Functions  

void main(void);                          	// Main application 
 
void InitializeMath(void);
double Factorial(double n);
double my_sqrt(double a, int n); 

#ifdef JMATH_TEST
 
//********************************************************************************
//* main - Driver to test all the floating point math functions including the 
//* trigonmetric and scientific functions along with testing the floating point I/O.
//********************************************************************************
   
void main(void)
{
	long int i,j;						// Loop index
	double my_x, my_y,my_z,angle;
	char c;
 
/*
#ifdef PIC18F8720


   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
   InitializeUsart();
#else
   // Open log file to write test results to Matlab

   f1 = fopen("sin.log","w");   
   f2 = fopen("cos.log","w");
   f3 = fopen("tan.log","w");
   f4 = fopen("asin.log","w");   
   f5 = fopen("acos.log","w");
   f6 = fopen("atan2.log","w");

   f7 = fopen("sqrt.log","w");   
   f8 = fopen("exp.log","w");
   f9 = fopen("power.log","w");
   f10 = fopen("mod.log","w");   
   f11 = fopen("log.log","w");
   f12 = fopen("log10.log","w");

#endif
*/

	// Initialize math test package 

	InitializeMath();
	
     // Setup timers 

      SetupTimers();  

    i = sizeof(int);
    j = sizeof(long int);
    x = sizeof(float);
    y = sizeof(double);
    z = sizeof(long);
    
	// Test the scientific functions

    // log - Natural log 

	my_x = log(1.0);
	my_x = log(2.0);
   	my_x = log(2.78182818);
 	my_x = log(PI);

    // log10  - Log base 10

	my_x = log10(1.0);
	my_x = log10(2.0);
	my_x = log10(10.0);
	my_x = log10(100.0);
	my_x = log10(1000.0);
	my_x = log10(PI);

    // The my_sqrt funtion works !!!
    
 	my_x = my_sqrt(4.0, 15);
	my_y = my_sqrt(16.0, 15);
	my_z = sqrt(144.0);

	
    my_x = my_sqrt(25.0, 15);
    my_y = my_sqrt(1024, 15);
    z = my_sqrt(144.0, 15);
    
	//x = square_root((double)25.0);
	my_y = sqrt((double)25.0);
	//x = square_root(-4.0);
	w = sqrt(-4.0);
	
	// Test J. Crenshaw's sqrt function
	
	my_x = sqrt(2.0);
	my_y = sqrt(4.0);
	z = sqrt(16.0);
	w = sqrt(144.0);
	
	// Test Trig functions

    // Sin and Asin test

	my_x =   sin(radians(30.0));
    my_y =   degrees(asin(my_x));

	my_x =   sin(radians(45.0));
    my_y =   degrees(asin(my_x));

	my_x =   sin(radians(60.0));
    my_y =   degrees(asin(my_x));

    // Cos and Acos test

	my_x =   cos(radians(30.0));
    my_y =   degrees(acos(my_x));

	my_x =   cos(radians(45.0));
    my_y =   degrees(acos(my_x));

	my_x =   cos(radians(60.0));
    my_y =   degrees(acos(my_x));

    // Tan and Atan test

	my_x =   tan(radians(30.0));
    my_y =   degrees(atan(my_x));

	my_x =   tan(radians(45.0));
    my_y =   degrees(atan(my_x));

	my_x =   tan(radians(60.0));
    my_y =   degrees(atan(my_x));

    // Atan2(y,x) test

	my_x = degrees(atan2(0.0, 1.0));  	// 0.0 Degrees
	my_x = degrees(atan2(1.0, 0.0));  	// 90.0 Degrees
	my_x = degrees(atan2(0.0, -1.0));	// 180.0 Degrees
	my_x = degrees(atan2(-1.0, 0.0));  	// 270.0 Degrees
	my_x = degrees(atan2(0.5, 0.5));	// 45.0 Degrees
	my_x = degrees(atan2(0.5, -0.5));	// 135.0 Degrees
	my_x = degrees(atan2(-0.5, -0.5));	// 225.0 Degrees
	my_x = degrees(atan2(-0.5, 0.5));	// 315.0 Degrees

	
#ifdef NEW_CODE	

	// Test J. Crenshaw's sin function.  The sin function works !!!
	
	x = sin(0.0*ConvertToRadians);
	y = sin(90.0*ConvertToRadians);
	z = sin(180.0*ConvertToRadians);
	w = sin(270.0*ConvertToRadians);
	
	x = sin(5.0*ConvertToRadians);
	y = sin(94.0*ConvertToRadians);
	z = sin(177.0*ConvertToRadians);
	w = sin(272.0*ConvertToRadians);
	
	angle = 0.0*ConvertToRadians;
	x = sin(angle);
	
	angle = 90.0*ConvertToRadians;
	y = sin(angle);
	
	angle = 180.0*ConvertToRadians;
	z = sin(angle);
	
	angle = 270.0*ConvertToRadians;
	w = sin(angle);
	
	angle = 5.0*ConvertToRadians;
	x = sin(angle);
	
	angle = 94.0*ConvertToRadians;
	y = sin(angle);
	
	angle = 177.0*ConvertToRadians;
	z = sin(angle);
	
	angle = 272.0*ConvertToRadians;
	w = sin(angle);

	// Test J. Crenshaw's cos function.  The cos function works !!!
	
	x = cos(0.0*ConvertToRadians);
	y = cos(90.0*ConvertToRadians);
	z = cos(180.0*ConvertToRadians);
	w = cos(270.0*ConvertToRadians);
	
	x = cos(5.0*ConvertToRadians);
	y = cos(94.0*ConvertToRadians);
	z = cos(177.0*ConvertToRadians);
	w = cos(272.0*ConvertToRadians);
#endif

/*
	printf("PIC18F8720 Math Test Application... \r\n");

	printf("Delay 5 seconds ...\r\n");
#ifdef PIC18F8720
	pause(5000);
#endif
	printf("End of 5 seconds.\r\n");
*/
	
	// Test the N ! function from 0! to 100! and see how big the PIC18 C Stack is.
	
	//for (i=0; i<100; i++)
	//for (i=0; i<12; i++)
	for (i=0; i<9; i++)
	{
		my_y = Factorial((double) i);

/*		dec(i);
		printf("! = ");
		
 		// Convert the value in  x, and y from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(my_y, ay);
		puts(ay);
		printf("\r\n");
		//pause(1000);
*/

	}
	
   // Generate table of sines from 0..360 degrees and write the results to a log file (sin.log)

	//printf("Generate a Sine table from 0 to 360 degrees  ... \r\n");
	
	for (i=0; i<360; i++)   
	{
		my_x =  (double)i*ConvertToRadians;
		my_y = sin(my_x);
			
		// Send result to Serial Port
/* 		
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		
 		// Convert the value in  x, and y from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(my_x, ax);
		string(ax);
		printf(" sin(my_x) = ");
		DoubleToAscii(my_y, ay);
		string(ay);
		printf("\r\n");	
		//pause(500);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  my_x = %15.8le, sin = %15.8le  \n", i, my_x, my_y);
		//fprintf(f1,"  i = %d,  my_x = %15.8le, sin = %15.8le  \n", i, my_x, my_y);
		fprintf(f1,"%15.8le \n", my_y);
#endif
*/ 
	}
	
	// Generate table of Cosines from 0..360 degrees and write the results to a log file (cos.log)
 
	//printf("Generate a Cosine table from 0 to 360 degrees ... \r\n");

 
	for (i=0; i<360; i++)   
	{
		my_x =  (double)i*ConvertToRadians;
		my_y =  cos(my_x);
 
/*
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		
 		// Convert the value in  x, and y from float to ascii for display to the monitor, LED or LCD.

		DoubleToAscii(my_x, ax);
		string(ax);
		printf(" cos(my_x) = ");
		DoubleToAscii(my_y, ay);
		string(ay);
		printf("\r\n");	
		//pause(500);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  my_x = %15.8le, sin = %15.8le  \n", i, my_x, my_y);
		//fprintf(f1,"  i = %d,  my_x = %15.8le, sin = %15.8le  \n", i, my_x, my_y);
		fprintf(f1,"%15.8le \n", my_y);
#endif
*/
	}

	// Generate table of Tangents from 0..359 degrees and write the results to a log file (tan.log)
 
	//printf("Generate a Tangent table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		my_x =  (double)i*ConvertToRadians;
		my_y = tan(my_x);
/*
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		dec(my_x);
		printf(" my_y = ");
		dec(my_y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  my_x = %15.8le, tan = %15.8le  \n", i, my_x, my_y);
		fprintf(f3, "  i = %d,  my_x = %15.8le, tan = %15.8le  \n", i, my_x, my_y);
		//fprintf(f3,"%15.8le \n", my_y);
#endif
*/ 
	}
	
	// Generate table of arcsines from 0..359 degrees and write the results to a log file (asin.log)
 
	//printf("Generate an arcsine table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		my_x =  (double)i*ConvertToRadians;
		my_y =  sin(my_x);
		my_z =  asin(my_y);
/*
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		dec(my_x);
		printf(" my_y = ");
		dec(my_y);
		printf(" my_z = ");
		dec(my_z);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  my_x = %15.8le, sin = %15.8le, asin = %15.8le  \n", i, my_x, my_y, my_z);
		fprintf(f4, "  i = %d,  my_x = %15.8le, sin = %15.8le, asin = %15.8le  \n", i, my_x, my_y, my_z);
		//fprintf(f4,"%15.8le \n", my_z);
#endif
*/ 
	}
	






	// Generate table of arccosines from 0..359 degrees and write the results to a log file (acos.log)
 
	//printf("Generate an arccosine table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		my_x =  (double)i*ConvertToRadians;
		my_y =   cos(my_x);
		my_z =  acos(my_y);
/*
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		dec(my_x);
		printf(" my_y = ");
		dec(my_y);
		printf(" my_z = ");
		dec(my_z);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  my_x = %15.8le, cos = %15.8le, acos = %15.8le  \n", i, my_x, my_y, my_z);
		fprintf(f5, "  i = %d,  my_x = %15.8le, cos = %15.8le, acos = %15.8le  \n", i, my_x, my_y, my_z);
		//fprintf(f5,"%15.8le \n", my_z);
#endif
*/ 
	}
		
	// Generate table of arctangents from 0..359 degrees and write the results to a log file (atan.log)
 
	//printf("Generate an arctangent table from 0 to 360 ... \r\n");

	for (i=0; i<360; i++)   
	{
		my_x =  (double)i*ConvertToRadians;
		my_y =   tan(x);
		my_z =  atan(y)*ConvertToDegrees;
/*
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		dec(my_x);
		printf(" my_y = ");
		dec(my_y);
		printf(" my_z = ");
		dec(my_z);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d, my_x = %15.8le, tan = %15.8le, atan = %15.8le  \n", i, my_x, my_y, my_z);
		fprintf(f6, "  i = %d, my_x = %15.8le, tan = %15.8le, atan = %f  \n", i, my_x, my_y, my_z);
		//fprintf(f6,"%15.8le \n", my_z);
#endif
*/ 
	}

 
	// Generate table of square roots from 0..1024 degrees and write the results to a log file (sqrt.log)
 
	//printf("Generate an square root table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		my_x = (double) i;
		my_y = sqrt(my_x);
/* 
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_x = ");
		dec(x);
		printf(" my_y = ");
		dec(y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  x = %f,  sqrt = %15.8le  \n", i, my_x, my_y);
		fprintf(f7,"  i = %d,  my_x = %f, sqrt = %15.8le  \n", i, my_x, my_y);
		//fprintf(f7,"%15.8le \n", my_y);
#endif
*/
	}

#ifdef NEW_CODE

	// Generate table of exp values from 0..1024 and write the results to a log file (exp.log)
 
	//printf("Generate an exponential table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		//my_x = (double) i;
		my_y = exp((double) i);
/* 
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_y = ");
		dec(my_y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  exp = %15.8le \n", i,my_y);
		fprintf(f8,"%15.8le \n", my_y);
#endif
*/ 
	}

#endif

	// Generate table of power values from 0..TABLE_SIZE and write the results to a log file (power.log)
 
	//printf("Generate an power of 2 table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		//my_x = (double) i;
		my_y = power(2.0, i);
/* 
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_y = ");
		dec(my_y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  power = %15.8le \n", i,my_y);
		fprintf(f9,"%15.8le \n", my_y);
#endif
*/ 
	}

	// Generate table of power values from 0..1024 and write the results to a log file (mod.log)
 
	//printf("Generate an mod (x,360) table from x: 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		my_x = (double) i;
		my_y = mod(x, 360.0);
/* 
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_y = ");
		dec(my_y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  mod = %15.8le \n", i,my_y);
		fprintf(f10,"%15.8le \n", my_y);
#endif
*/ 
	}

	// Generate table of log values from 0..1024 and write the results to a log file (log.log)
 
	//printf("Generate an log table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		my_x = (double) i;
		my_y = log(my_x); 
/* 
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_y = ");
		dec(my_y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  log = %15.8le \n", i,my_y);
		fprintf(f11,"%15.8le \n", my_y);
#endif
*/
 
	}
#ifdef NEW_CODE
	// Generate table of log10 values from 0..1024 and write the results to a log file (log10.log)
 
	//printf("Generate an log10 table from 0 to 1024 ... \r\n");

	for (i=0; i<1025; i++)   
	{
		my_x = (double) i;
		my_y = log10(my_x); 
/* 
#ifdef PIC18F8720
		printf("  i = ");
		dec(i);
		printf(" my_y = ");
		dec(my_y);
		printf("\r\n");	
		//pause(1000);		// Wait to allow viewing of the results
#else
		printf("  i = %d,  log10 = %15.8le \n", i,my_y);
		fprintf(f12,"%15.8le \n", my_y);
#endif
 */

	}
#endif
/*
#ifdef PIC18F8720
   //Close1USART();
#else

   // Close log files

   fclose(f1);
   fclose(f2);
   fclose(f3);
   fclose(f4);
   fclose(f5);
   fclose(f6);
   fclose(f7);
   fclose(f8);
   fclose(f9);
   fclose(f10);
   fclose(f11);
   fclose(f12);
 
#endif
*/

}

#endif
 
//******************************************************************************
//******************************************************************************
//******************************************************************************
//******************************************************************************

//******************************************************************************
//* InitializeMath - Initialize math test package variables.
//******************************************************************************

void            InitializeMath(void)
{
 
	// Initialize the math constants used by the jmath package

	InitializeConstants();

	// Compute the conversion from degrees to radians variable

	ConvertToRadians = PI/180.0;
	ConvertToDegrees = 180.0/PI;
/*
#ifdef PIC18F8720
    // Initialize ports  

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                          // Clear PORTC (Some Port C bits are reserved for USART)
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction for output

 
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
    //TRISC = 0x00;                                        // PORTC all outputs (Some Port C bits are reserved for USART).
    TRISD = 0x00;                                          // PORTD all outputs.
    TRISE = 0x00;                                          // PORTE all outputs.

#endif   
*/ 
}

 

//******************************************************************************
// I2CReadByte - Reads a byte from the selected I2C slave device.
//******************************************************************************
  
byte I2CReadByte(byte I2CReadAddress)
{
 	byte TheData;
 
   return(TheData); 
}
//******************************************************************************
// I2CWriteByte - Writes a byte to the selected I2C slave device
//******************************************************************************

void I2CWriteByte(byte I2CWriteAddress, byte TheData)
{
  

   //pause(10);                      // Wait 10 milliseconds (Is this statement needed ????)
}

//******************************************************************************
// Factorial - Finds the value of N!.   
//******************************************************************************

double Factorial(double n)
{
 	
 	// Check for recursion exit condition
 	
	if (n == 0.0)
	{
		return 1.0;
	}
	else
	{
		// Recusion of N!
		return  (n*Factorial(n-1.0));
	}
	
} 

//******************************************************************************
// my_sqrt - Finds the square root by iteration (from Crenshaw's book).  Does not 
// determine the best guess at the initial root.  
//******************************************************************************

double my_sqrt(double a, int n)
{
	double x;
	int i;
	x = a;
	
	if (x < 0.0)
		return x;
		
	for (i=0; i<n; i++)
	{
		x = 0.5 * (a/x + x);
	}
	return x;
} 
/* File jmath.cpp
 *
 * Defines commonly used primitive macros and functions.  Some macros are
 * redefined to make sure we get them right when porting to different
 * platforms.
 *
 * Some of the functions are "safe" functions, intended to be used in
 * embedded systems where error halts are undesirable.  Instead of halting,
 * the functions return reasonable values and keep going.  Function cmod
 * fixes a problem with the library modulo functions, which give incorrect
 * values if one or both arguments are negative.
 *
*/

double sqrt(double a)
{
	return my_sqrt(a, 15);
}

/*
// Model for fast square root with optimal first guess

double sqrt(double a)
{
	short expo;
	//double factor = root_2/2;
	double factor = 0.707106781;
	const double A =  0.417319242;
	const double B = 0.590178532;
	hack_structure x;
	double root;

	// check for negative or zero
	if(a <= 0)
		return 0;
	x.fp = a;

	// grab the exponent
	expo = (short)(x.hi >> 20);
	expo -= 0x3fe;

	// normalize the number
	x.hi &= 0x000fffff;
	x.hi += 0x3fe00000;

	// get square root of normalized number
	// generate first guess
	root = A + B * x.fp;

	// iterate three times (probably overkill)
	// delete one iteration for single precision

	root = 0.5 * (x.fp/root + root);
	root = 0.5 * (x.fp/root + root);
	root = 0.5 * (x.fp/root + root);

	// now rebuild the result
	x.fp = root;

	// force the exponent to be even
	if(expo & 1){
		x.fp *= factor;
		++expo;
	}

	// halve the exponent
	if(expo < 0)
		expo = expo/2;
	else
		expo = (expo + 1)/2;

	// put it back
	expo += 0x3fe;
	x.hi &= 0x000fffff;
	x.hi += ((unsigned long)expo << 20);

	return x.fp;
}

*/

/* Sine and cosine functions.  See Listings
 * 5.2 and 5.4 of text.
 */

 // Find the Sine of an Angle <= 45

double _sine(double xx)
{	
  double si1, si2, si3, si4;
  double zz;

	si1 = 1.0/(2.0*3.0);
	si2 = 1.0/(4.0*5.0);
	si3 = 1.0/(6.0*7.0);
	si4 = 1.0/(8.0*9.0);
	zz = xx * xx;
 
	return ((((si4*zz-1.0)*si3*zz+1.0)*si2*zz-1.0)*si1*zz+1.0)*xx;
 
}



double sine(double x)
{
	long n;

	n = (long)(x/halfpi + 0.5);
	x -= n * halfpi;
	n = mod(n, (long)4);
	switch(n){
		case 0:
      	return _sine(x);
		case 1:
      	return _cosine(x);
		case 2:
      	return - _sine(x);
		case 3:
      	return  - _cosine(x);
		default:
		return 0.0;
	}
}

double sin(double x)
{
	long n;  
    double TheSin;

	n = (long)(x/halfpi + 0.5);
	x -= n * halfpi;
	n = mod(n, (long)4);
	switch(n){
		case 0:
      	 TheSin = _sine(x);
      	 break;

		case 1:
      	  TheSin = _cosine(x);
      	  break;

		case 2:
      	  TheSin = - _sine(x);
      	  break;

		case 3:
      	  TheSin = - _cosine(x);
      	  break;

		default:
		TheSin = 0.0;
		break;
	}
    return TheSin;

}



// Find the Cosine of an Angle <= 45

double _cosine(double x)
{
	double c1, c2, c3, c4, z;

	c1 = 1.0/(1.0*2.0);
	c2 = 1.0/(3.0*4.0);
	c3 = 1.0/(5.0*6.0);
	c4 = 1.0/(7.0*8.0);
	z = x * x;
   return (((c4*z-1.0)*c3*z+1.0)*c2*z-1.0)*c1*z+1.0;
}

// cos function

double cosine(double x)
{
	if(x > pi)
		return sine(x - (pi+halfpi));

	return sine(x + halfpi);
}


double cos(double x)
{
	if(x > pi)
		return sine(x - (pi+halfpi));

 	return sine(x + halfpi);
}
 
#ifdef NEW_CODE

/* The following function computes both sine and cosine in one
 * call.  It should therefore be faster than making two calls.
 * You might choose to incorporate it in jmath.cpp, and
 * use it as a general-purpose function in your own math
 * library.  The coefficients as shown are good enough for
 * single precision.  Use functions _sine and _cosine above
 * for double precision accuracy.
*/

// Find both the sine and cosine of an angle

void sincos(double x, double *s, double *c)
{
	double s1, c1;
	long n;
 	double s1, c1, z;

	n = (long)(x/pi_over_six + 0.5);
	x -= (double)n * pi_over_six;
	n = n % 12;

   if(n < 0)
		n += 12;

 	z = x*x;
	s1 = ((z/20.0-1)*z/6.0+1.0)*x;
	c1 = ((z/30.0 +1.0)*z/12.0-1.0)*z/2.0+1.0;

	switch(n){
	case 0:
		*s = s1;
		*c = c1;
		break;
	case 1:
		*s =  cos_30 * s1 + sin_30 * c1;
		*c = -sin_30 * s1 + cos_30 * c1;
		break;
	case 2:
		*s =  sin_30 * s1 + cos_30 * c1;
		*c = -cos_30 * s1 + sin_30 * c1;
		break;
	case 3:
		*s =  c1;
		*c = -s1;
		break;
	case 4:
		*s = -sin_30 * s1 + cos_30 * c1;
		*c = -cos_30 * s1 - sin_30 * c1;
		break;
	case 5:
		*s = -cos_30 * s1 + sin_30 * c1;
		*c = -sin_30 * s1 - cos_30 * c1;
		break;
	case 6:
		*s = -s1;
		*c = -c1;
		break;
	case 7:
		*s = -cos_30 * s1 - sin_30 * c1;
		*c =  sin_30 * s1 - cos_30 * c1;
		break;
	case 8:
		*s = -sin_30 * s1 - cos_30 * c1;
		*c =  cos_30 * s1 - sin_30 * c1;
		break;
	case 9:
		*s = -c1;
		*c =  s1;
		break;
	case 10:
		*s =  sin_30 * s1 - cos_30 * c1;
		*c =  cos_30 * s1 + sin_30 * c1;
		break;
	case 11:
		*s =  cos_30 * s1 - sin_30 * c1;
		*c =  sin_30 * s1 + cos_30 * c1;
		break;
	}
}

#endif


// tan function

double tan(double x)
{
	return(sine(x)/cosine(x));
}

// safe tangent
double tangent(double x){
	x = ang_180(x);
	if(abs(x) == halfpi)
		return BIG_FLOAT;
	return tan(x);
}
 


/* Prototype for arctangent function using continued
 * fractions.  This is Listing 6.7 of text.
 */

// atan function

 /* Rational approximation for atan(x)
  * This polynomial uses a minimaxed 5th-order
  * ratio of polynomials.  Input angles are restricted
  * to values no greater than 15 degrees.  Accuracy is
  * better than 4e-8 over the full range of inputs.
  */

double atan(double x)
{
  // constants for segmentation

  static double b;
  static double k;
  static double b0;
  static double k0;

  // constants for rational polynomial

  static const double A = 0.999999020228907;
  static const double B = 0.257977658811405;
  static const double C = 0.59120450521312;
  double ang;
  double x2;
  int comp = FALSE;
  int hi_seg = FALSE;
  int Sign;

  b = pi/6;
  k = tangent(b);
  b0 = pi/12;
  k0 = tangent(b0);

  // make argument positive
  Sign = (x < 0);
  x = abs(x);

  // limit argument to 0..1
  if(x > 1)
  {
    comp = TRUE;
    x = 1/x;
  }

  // determine segmentation
  if(x > k0){
    hi_seg = TRUE;
    x = (x - k)/(1 + k*x);
  }

  /* argument is now < tan(15 degrees)
   * approximate the function
   */
  x2 = x * x;
  ang = x*(A + B*x2)/(1 + C*x2);

  // now restore offset if needed
  if(hi_seg)
    ang += b;

  // restore complement if needed
  if(comp)
    ang = halfpi - ang;

  // restore Sign if needed
  if(Sign)
    return -ang;
  else
    return ang;
}

/*
double tangent(double x)
{
	return(sine(x)/cosine(x));
}

*/


double asin(double x){
	if(abs(x) >= 1.0)               
		return sign(halfpi, x); 
	return atan(x/sqrt(1.0 - x*x));
}

double acos(double x){
	return(halfpi - asin(x));
}

// Inverse sine

// Safe arcsine
double arcsin(double x){
	if(x>1.0)
		return halfpi;
	if(x<-1.0)
		return -halfpi;
	return asin(x);
}
 
// Inverse cosine

double arccos(double x)
{       
	if(x>1.0)
		return 0;
	if(x<-1.0)
		return pi;
	return acos(x);

}

// Four-quadrant arctan

double atan2(double y, double x)
{  

	double retval;  
	
	if(x == 0.0)              
		retval = halfpi;        
	else            
		retval = atan(abs(y/x));        
	
	if(x < 0.0)               
	retval = pi - retval;   
	
	if(y < 0.0)               
		retval = -retval;       
	return retval;
}


// safe, four-quadrant arctan

double arctan2(double y, double x){
	if(x == 0.0)
		return sign(halfpi, y);
	return atan2(y, x);
}

 
// convert angle from degrees to radians
double radians(double x){
	return x * radians_per_degree;
}


// convert angle from radians to degrees
double degrees(double x){
	return x * degrees_per_radian;
}

// Reduce an Angle (in radians) to the range 0.. 360 degrees
double ang_360(double x){
	return mod(x, twopi);
}

// Reduce an angle (in radians) to the Range -180..180 degrees
double ang_180(double x){
	double retval;
	retval = mod(x + pi, twopi) - pi;
	if (retval > -pi)
		return retval;
	else
		return retval + twopi;
}



 /* Find the natural log for a floating point
 * number in the range 0..1
 *
 * Method: Limit the range to z = 0.069
 * Use minimaxed 5th-order rational
 * polynomial to calculate the log.
 */

double __ln(double x)
{
	const double a = 1;
	const double b = -0.2672435;
	const double c = -0.600576;
	double z, z2;
 
	z = (x-1)/(x+1);
	z2 = z*z;

	return 2*z*(a + b* z2)/(1 + c*z2);
}

double _ln(double x)
{
	const double limit1 = 0.879559563; 	// 0.5^(1/5)
	const double limit2 = 0.659753955; 	// 0.5^(3/5)
	const double k1 = 0.757858283; 		// 0.5^(2/5)
	const double k2 = 0.574349177; 		// 0.5^(4/5)
	const double ln_k = -0.138629436;  	// ln(0.5^(1/5)

	if(x >= limit1)
		return(__ln(x));
	else if(x >= limit2)
		return(__ln(x/k1) + 2*ln_k);
	else
		return(__ln(x/k2) + 4*ln_k);
}

//  log - Natural log function

/* Main part of log function
 * Hacks the f.p. argument
 * Separates integer and fractional parts
 * Recombines after generating
 * kernel function. 
*/

double log(double X)
{
   double ln2;
	short expo;
	double retval;
	hack_structure x;

	ln2 = log10(2.0);
	if(X <= 0)
		return -BIG_FLOAT;
	x.fp = X;

	// grab the exponent
	expo = (short)(x.hi >> 20);
	expo -= 0x3fe;

	// force the exponent to zero
	x.hi &= 0x000fffff;
	x.hi += 0x3fe00000;

	// compute the log of the mantissa only
	retval = _ln(x.fp);

	// rebuild the result
	retval = ln2*(expo + retval/ln2);
	return retval;
}
/*
// Log base 10 of x

double log10(double x)
{
	return (log(x)/log(10));
}
*/

/* Generate the exponential
 *
 * Function is optimized for the range
 * 0..1/4
 */
 
double exp(double x)
{
	//double root_e;
	const double root_e = 1.284025417;
	const double a= 1.000000034750736;
	const double b= 0.499997989964957;
	const double c= 0.166704077809886;
	const double d= 0.041365419657829;
	const double e= 9.419273457583088E-3;
	double retval;
	double intpart;
	int n;

	//root_e = sqrt(sqrt(exp(1)));
 
	x *= 4;
	n = (int)x;
	x -= n;
	x /= 4;
	retval = ((((x*e+d)*x+c)*x+b)*x+a)*x+1;
	intpart = power(root_e,(double)n);

	return intpart*retval;
}

 
 // safe exponential function
double exponential(double x){
	if(x > 46)
		return BIG_FLOAT;
	return exp(x);
}

/* An efficient pow function for integer powers
 *Use only for positive integers
 */
double power(double x, int n){
	double retval;
	double factor;
	retval = 1;
	factor = x;
	while(n != 0){
		if((n&1) != 0)
			retval *= factor;
		factor *= factor;
		n >>= 1;
	}
	return retval;
}
 
 
/* Return a number modulo the second argument
 * Unlike most library routines, this function works
 * properly for all values of both arguments.
 */

double mod(double x, double y)
{
	int i;

	if(y == 0) return x;
	i = (int)(x/y);
	if(x*y < 0)--i;
	x = x-((double)i)*y;
	if(x==y)x -= y;
	return x;
}




// Safe square root
double square_root(double x)
{
	if(x <= 0) return 0;
	return sqrt(x);
}
 
 // Square function, sqr(x)
double sqr(double x)
{	 
	return x*x;
}



#ifdef NEW_CODE

/* The function simpson provides a general-purpose implementation of
 * numerical quadrature using Simpson's rule.  This algorithm
 *	generates the integral under the curve:
 *
 *						y = f(x)
 *
 * Between the limits x1 and x2, dividing this range up into N
 * intervals.
 *
 * The nature of Simpson's rule requires that N be even.
 */

double simpson(double (*f)(double), double x1, double x2, int n)
{
	double sum = 0;
	double h; 
	double x;
	int i;

	if((n & 1) != 0)
	{
		//printf("Simpson: N must be an even integer\n");
		return 0;
	}

	h = (x2-x1)/n;
	x = x1;
	for(i=0; i<n/2; i++){
		sum += f(x)+2*f(x+h);
		x += 2*h;
	}
	sum = 2*sum-f(x1)+f(x2);
	return h*sum/3;
}

#endif


