//********************************************************************************                                                                                      
//*        Name:  sensor.h                                                       *
//*      Author:  Daniel F. Ramirez                                              *  
//*        Date:  02/20/2003                                                     *     
//*                                                                              *
//*                                                                              *  
//* Description:                                                                 *  
//*     This application gathers and processes sensor data collected from up to  *
//* 8 A/D channels and sends the data back to the Host, I2C or SPI Master        *
//* Controller using the Microchip PIC18C452 microcontroller.  It can be used    *
//* to collect and display data from any light sensor (phototransistor or CDS    *
//*  cell), IR proximity sensor or temperature sensor for robotic applications.  *
//* It also provides access to the digital I/O ports (A-E), which is used for    *
//* reading switches, status words and binary data.  Port B interrupts may be    *
//* utilized for interfacing external devices such as A/D's, keypads and         *
//* switches.  The device can get the PIC's attention using a Level or edge      *
//* triggered interrupt.                                                         *
//*                                                                              *
//*      A communication interface is provided that allows the host or SPI or    *
//* I2C Master to request individual sensor readings, average sensor readings,   *
//* calibrating a sensor or perform a sensor processing algorithm using command  *
//* message packets.  The commands available are described below.  This header   *
//* file provides the common Master and Slave message formats and descriptors.   *                                                            
//*                                                                              *
//*      Possible sensor algorithms include: Fuzzy Logic, Sensor Fusion, AI      *  
//* algorithms, Neural Networks, Mean, RMS, STD, Variance, Median, Filters,      *
//* Least Squares Fit, and Gaussian Polynomial Curve Fitting.                    * 
//*                                                                              *
//*      Possible filter algorithms include: IIR, FIR, digital filters,          *
//* averaging filters, median filters and Kalman Filters.                        *         
//*                                                                              *
//*                                                                              *
//*		The On-Chip calibration process includes finding the minimum, maximum    *
//* and average light sensor readings so that the sensor data can be             *
//* characterized and scaled.  These values are later stored to EEPROM or        *
//* scratch pad RAM available on the PIC, so that they may be accessed by other  *
//* applications which need them.   This calibration process may also be         *
//* performed from a laptop or PC Workstation in order to save EEPROM/FLASH      *
//* program space on the PIC.                                                    *
//*                                                                              *
//* 		The Controller allows Each port (A-E) on the PIC processor to be     *
//* be read (digitial I/O) and count Port B interrupts which indicate that a     *
//* digital switch has fired off will also be reported.  New sensor history      *
//* functions that compute sensor statistics  using previously stored data in a  *
//* ring buffer will also be used for data logging and data analysis.            *
//* The switch states for Port B are considered to be a pseudo sensor with       *
//* ID = 0 or 8 ???.  No calibration or scaling is required for this sensor.     *
//*                                                                              *
//*     The sensor calibration data is collected by HYPERTERMINAL, PCPLUS or VB  *
//* or VC++ application on the host, and then may be plotted using Matlab, Excel *
//* or the Stamp Plot Lite program which also runs on the host PC. Plot commands *
//* are sent from the PIC to the host using specific debug commands (printf).    *
//*                                                                              *
//*                                                                              *
//* Usage:                                                                       *
//*        The following Sensor Controller commands are available using the      *
//* serial interface:                                                            *               
//*                                                                              *
//*        "Axx,SamplePeriod;" - Record sensor data  by entering the sampling    *
//*           period (ms) in 4 digit hex format to record an animation for the   *
//*           selected joystick xx, where the Joystick ID range (01..02) is in a *
//*           2 digit hex byte format.                                           *                                                           *
//*                                                                              *
//*        "Bxx,SamplePeriod;" - Play back sensor data by entering the           *
//*           sampling period (ms) in 4 digit hex format to play an animation for*
//*           the selected joystick xx, where the Joystick ID range (01..02) is  *
//*           in a 2 digit hex byte format.                                      *  
//*                                                                              *
//*        "Cxx,SamplePeriod;" - Calibrate the selected Sensor xx, by computing  *
//*           its Gain and Offset calibation values, where the Sensor ID range   *
//*           (00..FF) is in a 2 digit hex byte format.  The sampling period (ms)*
//*           is entered in 4 digit hex and stored to EEPROM for later use by    *
//*           the Record and Play functions.                                     *     
//*                                                                              *
//*        "Dxx;"       - Send  interrupt counts for Port B bit xx, where the bit*
//*                       ID range (04..07) is in a 2 digit hex byte format.     *
//*                                                                              *
//*        "Exx;"       - Execute sensor algorithm xx, where the algorithm ID    *
//*                       range (00..FF) is in a 2 digit hex byte format.        * 
//*                                                                              *
//*        "Fxx;"       - Select filter xx, where the filter ID                  *
//*                       range (00..FF) is in a 2 digit hex byte format,        * 
//*                       where 00 => No Filter, 01 => Average Filter,           *
//*                       02 => Kalman Filter, 03 => Filter 3,...                *                                  
//*                                                                              *
//*        "Ixx;"       - Read a port (00=>A..04 =>E) Send value read for Port   *
//*                       xx, where the Port ID range (00..04) is in a 2 digit   * 
//*                       hex byte format.                                       *
//*                                                                              *
//*        "Sxx;"       - Send calibration data from Sensor xx, where the        *
//*                       Sensor ID range (00..FF) is in a 2 digit hex byte      * 
//*                       format.                                                *  
//*                                                                              *
//*        "Wxx,WindowMin,WindowMax;" -                                          * 
//*                       Accept window data used to scale the                   * 
//*                       Sensor commands, where the Sensor ID range             *
//*                       (00..FF) is in a 2 digit hex byte format and the two   *
//*                       16-Bit hex words that follow represent integer values  * 
//*                       for the minimum and maximum window coordinates.        *  
//*                                                                              *                             
//*         "Rxx;"      - Send raw sensor reading and switch status data         * 
//*                       from sensor xx, where the sensor ID range (00..FF) is  *
//*                       in a 2 digit hex byte format using selected filter.    * 
//*                                                                              *                             
//*         "Nxxxx;"     - Accept total number of samples, XXXX to be read from  *
//*                       selected sensor, where xxxx is in 4 digit hex byte     * 
//*                       format  range (00..FFFF)                               * 
//*                                                                              *
//*         "Pxx;"     - Send scaled sensor reading and switch status data       *
//*                       from sensor xx, where the sensor ID range (00..FF)     * 
//*                       is in a 2 digit hex byte format using selected filter  *   
//*                                                                              *
//*         "Txx;"      - Test the selected sensor xx, using selected filter,    *
//*                       where the sensor ID range (00..FF) is in a 2 digit hex *
//*                       byte format.                                           *
//*                                                                              *
//*         "Z;"        - Reset the Sensor Controller.                           *
//*                                                                              *
//* Example:                                                                     *
//*    Send the following string to the Sensor controller to read a raw sensor   *
//* reading from Sensor #1 using the string:                                     *
//*       R01;"                                                                  *                                     
//* Where:                                                                       *
//*      R - Is the calibration command                                          *
//*     01 - Is the Sensor ID                                                    *
//*                                                                              *
//*      Future upgrades to this application include: Reading other kinds        *
//* of sensors such as temperature sensors, sound (MIC) transducers,             *
//* Potentiometers (Sensor) encoders, IR rangers and Sonar rangers.              *
//* Collecting data from the Digital I/O Ports (A-E) is another option.  Other   *
//* upgrades include interfacing to VC++ applications collect.cpp, sensor.cpp,   *
//* control.cpp and actuator.cpp. The USART, I2C and SPI bus are used to debug   *
//* the application.  The I2C and SPI bus are also used as a high speed          *
//* communication bus to communicate with the PC host, or with other I2C and SPI *
//* devices include other SPI masters such as  PIC processors for use in advanced*
//* robot designs that take advantage of multi-processing or networked           *
//* processors.                                                                  *
//*                                                                              *
//* Note:                                                                        *
//*		Each kind of sensor uses the same RAM location as the raw sensor         *
//* reading location.                                                            *
//*                                                                              *
//********************************************************************************   
      
#ifndef SENSOR_LOADED
#define SENSOR_LOADED

// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
//#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18F452     
//#include <p18c452.h>					// Register definitions for PIC18F8720
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
 
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <spi.h>													  // I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <adc.h>                                        // ADC library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#include "network.h"                                    // Include SPI Network Support
#include "timeutil.h"                                   // Timer utility functions
#include "stdio.h"                                      // I/O support functions
#include "portutil.h"                                   // Port Utilities
#include "sensors.h"								    // Include Sensor Controller utilities


//**********************************************************************************
// Define messages for each type of command that is sent by the SPI Master and 
// recognized by the selected SPI Slave.
//**********************************************************************************

//**********************************************************************************
//* Cxx,SamplePeriod   - Calibrate the selected Sensor xx, by computing            *
//* its Gain and Offset calibation values, where the Sensor ID range               *
//* (00..FF) is in a 2 digit hex byte format.  The sampling period (ms)            *
//* is entered in 4 digit hex and stored to EEPROM for later use by                *
//* the Record and Play functions.                                                 *     
//**********************************************************************************
typedef struct calibrate_sensor_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex

  	COMMAND_TYPE Command;         	// Calibrate Command to SPI Slave (see enumeration
                                	// declaration above.)  
 	byte SensorID; 					// Selected Sensor range (0..255)  
    word SamplePeriod;				// The sampling period (ms)    
 
    byte Filler[26];				// Padding 
    
} CALIBRATE_SENSOR_TYPE;

//**********************************************************************************
//* Dxx - Get latest interrupt counts from Port B bit xx, where the bit ID range   *
//* (04..07) is in a 2 digit hex byte format.                                      *
//**********************************************************************************
typedef struct get_interrupt_counts_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Get latest interrupt counts Command from SPI Slave  
 
 	byte SensorID; 					// Selected sensor range (0..255)  
  	word PortBCount4;				// latest interrupt counts from Port B bit 4
 	word PortBCount5;				// latest interrupt counts from Port B bit 5
 	word PortBCount6;				// latest interrupt counts from Port B bit 6
 	word PortBCount7; 				// latest interrupt counts from Port B bit 7           	
    byte Filler[20];				// Padding 

} GET_INTERRUPT_COUNTS_TYPE;

//**********************************************************************************
//* Exx - Execute sensor algorithm xx, where the algorithm ID  range (00..FF) is   *
//* in a 2 digit hex byte format.                                                  * 
//**********************************************************************************
typedef struct select_algorithm_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Get latest interrupt counts Command from SPI Slave  
 
 	byte SensorID; 					// Selected sensor range (0..255)  
	byte AlgorithmID;				// Select algorithm xx, where the algorithm ID range (00..FF)
    byte Filler[27];				// Padding 
    
} SELECT_ALGORITHM_TYPE;

//**********************************************************************************
//* Fxx - Select filter xx, where the filter ID range (00..FF) is in a 2 digit hex *
//* byte format,  where 00 => No Filter,                                           *
//*                     01 => Average Filter,                                      *
//*                     02 => Kalman Filter,                                       *
//*                     03 => Filter 3,...                                         *                                  
//**********************************************************************************
typedef struct select_filter_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Get latest interrupt counts Command from SPI Slave  
 
 	byte SensorID; 					// Selected sensor range (0..255)  
	byte FilterID;					// Select filter xx, where the filter ID range (00..FF)
    byte Filler[27];				// Padding 
   
} SELECT_FILTER_TYPE;

//**********************************************************************************
//* Ixx - Read a port (00=>A..04 =>E) Send value read for Port xx, where the Port  *
//* ID range (00..04) is in a 2 digit hex byte format.   This function is used for *
//* digital I/O.                                                                   *
//**********************************************************************************
typedef struct read_port_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Read selected port from SPI Slave  
 	byte PortID; 					// Selected Port ID range (0..4)  
    byte Port[MAXPORTS]; 			// Selected port value range (0..255)  
    byte Filler[23];				// Padding 
   
} READ_PORT_TYPE;

//**********************************************************************************
//* Oxx - Write a value to the selected Port (00=>A..04 =>E) , where the Port      *
//* ID range (00..04) is in a 2 digit hex byte format.   This function is used for *
//* digital I/O.                                                                   *
//**********************************************************************************
typedef struct write_port_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Read selected port from SPI Slave  
 
 	byte PortID; 					// Selected Port ID range (0..4)  
  	byte Port[MAXPORTS]; 			// Selected port value range (0..255)  
    byte Filler[23];				// Padding 
   
} WRITE_PORT_TYPE;

//**********************************************************************************
//* Sxx - Get calibration data from Sensor xx, where the Sensor ID range (00..FF)  *
//* is in a 2 digit hex byte format.                                               *  
//**********************************************************************************
typedef struct get_sensor_calibration_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex

  	COMMAND_TYPE Command;         	// Send Calibration Data Command from SPI Slave.
 	byte SensorID; 					// Selected Sensor range (0..255)  
    float MinSensorValue; 			// Minimum sensor value window    
    float MaxSensorValue; 			// Maximum sensor value window    
    float SensorGain; 				// Sensor Gain scale factor value 
    float SensorOffset; 			// Sensor Offset value 
    byte Filler[12];   				// Padding 

} GET_SENSOR_CALIBRATION_TYPE;

//**********************************************************************************
//* Wxx,WindowMin,WindowMax - Set window data used to scale the Sensor commands,   *
//* where the Sensor ID range (00..FF) is in a 2 digit hex byte format and the two *
//* 16-Bit hex words that follow represent integer values for the minimum and      *
//* maximum window coordinates.                                                    *  
//**********************************************************************************
typedef struct set_sensor_window_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex

  	COMMAND_TYPE Command;         	// Send Calibration Data Command from SPI Slave.
 	byte SensorID; 					// Selected Sensor range (0..255)  
	float WindowMin;				// Minimum window coordinate used to scale sensor.   
	float WindowMax;				// Maximum window coordinate used to scale sensor.   
    byte Filler[20];   				// Padding 
			
} SET_SENSOR_WINDOW_TYPE;  

//**********************************************************************************
//* Rxx - Get raw sensor reading and switch status data from sensor xx, where the  *
//* sensor ID range (00..FF) is in a 2 digit hex byte format using selected filter.* 
//**********************************************************************************
typedef struct get_raw_sensor_data_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex

  	COMMAND_TYPE Command;         	// Send Calibration Data Command from SPI Slave.
 	byte SensorID; 					// Selected Sensor range (0..255)  
	byte DebouncedSwitchState;		// Switch state (Digital I/O) range (0..255)
	word RawSensorValue;			// Raw Sensor Value range (0000 .. FFFF) Hex
    byte Filler[25];   				// Padding 
} GET_RAW_SENSOR_DATA_TYPE;   

//**********************************************************************************
//* Pxx - Get scaled sensor reading and switch status data from sensor xx, where   *
//* the sensor ID range (00..FF) is in a 2 digit hex byte format using selected    *
//* filter.                                                                        *   
//**********************************************************************************
typedef struct get_scaled_sensor_data_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex

  	COMMAND_TYPE Command;         	// Send Calibration Data Command from SPI Slave.
 	byte SensorID; 					// Selected Sensor range (0..255)  
	byte DebouncedSwitchState;		// Switch state (Digital I/O) range (0..255)
	float CurrentSensorValue;		// Current scaled Sensor Value    
    byte Filler[23];   				// Padding 
} GET_SCALED_SENSOR_DATA_TYPE;				 

//**********************************************************************************
//* Axx,SamplePeriod - Record sensor data  by entering the sampling period (ms)    *
//* in 4 digit hex format to record an animation for the selected joystick xx,     *
//* where the Joystick ID range (01..02) is in a 2 digit hex byte format.          *                                                                                                  *
//*                                                                                *
//**********************************************************************************

typedef struct record_command_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Record Sensor Data  
 	byte SensorID; 					// Selected sensor ID range (0..255)  
 	word SamplePeriod;				// Sampling Period range (0000 .. FFFF) Hex 
    byte Filler[26];				// Padding 
   
} RECORD_COMMAND_TYPE;

//**********************************************************************************
//* Bxx,SamplePeriod - Play back sensor data by entering the sampling period (ms)  *
//* in 4 digit hex format to play an animation for the selected joystick xx, where *
//* the Joystick ID range (01..02) is in a 2 digit hex byte format.                *                                 
//**********************************************************************************
 
typedef struct play_command_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Play back Sensor data  
 
 	byte SensorID; 					// Selected sensor ID range (0..255)  
 	word SamplePeriod;				// Sampling Period range (0000 .. FFFF) Hex 
    byte Filler[26];				// Padding 
   
} PLAY_COMMAND_TYPE; 

//**********************************************************************************
//* Txx - Test the selected sensor xx, using selected filter, where the sensor ID  *
//* range (00..FF) is in a 2 digit hex byte format.                                *
//**********************************************************************************
typedef struct test_command_type
{
  	word MessageID;    				// Message ID range (0000 .. FFFF) Hex 

  	COMMAND_TYPE Command;         	// Play back Sensor data  
 
 	byte SensorID; 					// Sensor ID range (0..255)  
    byte Filler[28];				// Padding 
   
} TEST_COMMAND_TYPE;

//**********************************************************************************
//* Define union of unformatted message and command message structures in order to 
//* overlay the memory. 
//**********************************************************************************
typedef union sensor_message_union_type
{
    CALIBRATE_SENSOR_TYPE CalibrateSensorCommand;				// Calibrate sensor command message
	   					 
	GET_SENSOR_CALIBRATION_TYPE GetSensorCalibrationCommand;	// Get sensor calibration data from the Sensor Controller
	SET_SENSOR_WINDOW_TYPE  SetSensorWindowCommand;				// Set the sensor window used to scale data for the Sensor
	GET_RAW_SENSOR_DATA_TYPE GetRawSensorDataCommand;  			// Get raw sensor data from the Sensor Controller
	GET_SCALED_SENSOR_DATA_TYPE GetScaledSensorDataCommand; 	// Get scaled sensor data from the Sensor Controller
	SELECT_FILTER_TYPE SelectFilterCommand;                     // Select the filter to be applied to collected data
	SELECT_ALGORITHM_TYPE SelectAlgorithmCommand;				// Select the algorithm to be used for processing the sensors

 	RECORD_COMMAND_TYPE RecordCommand;                         	// Record command message	
	PLAY_COMMAND_TYPE PlayCommand;                          	// Play command message	
	SAMPLE_COMMAND_TYPE SampleCommand;                          // Sample command message	
	PERIOD_COMMAND_TYPE PeriodCommand;					 		// Period command message
	GET_STATUS_COMMAND_TYPE GetStatusCommand;					// Get status message
	GET_INTERRUPT_COUNTS_TYPE GetInterruptCountsCommand;        // Get latest interrupt count
	READ_PORT_TYPE ReadPortCommand;                             // Read from  the selected port
	WRITE_PORT_TYPE WritePortCommand;                           // Write to the selected ports
	TEST_COMMAND_TYPE TestCommand;                              // Test Command
	RESET_COMMAND_TYPE ResetCommand;					 		// Reset command message

	COMMAND_MESSAGE_TYPE SelectedCommand;						// Selected command	
	MESSAGE_TYPE Message;										// Unformatted message
} SENSOR_MESSAGE_UNION_TYPE;

#endif


