//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  spimsg.h                                                          *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  01/21/2003                                                        *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements an SPI master and slave message formats used *
//* by spimast.c, spicli.c, spislave.c, and spiserv.c applications.  This file is*
//* included by each of these applications so that they may communicate over the *
//* SPI bus using common message formats.                                        *                                                        *
//*                                                                              *
//*     This SPI slave controller can be used with a PC host or any SPI master,  *
//* in order to  give any PC the capability to serve as an SPI controller.       * 
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each SPI device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large          *
//* CRC and Checksum capabilities along with large message definitions for       *
//* various applications.                                                        *
//*                                                                              *
//* Usage:                                                                       *
//*	The following DC motor Controller command messages are available:            *
//*                                                                              * 
//* 	Cmm       - Calibrate motor speed for motor mm range (00..01). This      *
//*                 command is used to vary the motor speed and direction using  *
//*                 two potentiometers with the blades connected to AN0 and AN1. *
//*                 The true motor speed (RPM) is determined from the counts     *
//*                 received from the optical encoders (if used) and displayed.  *
//*                                                                              * 
//* 	Dmm,xxxx  - Send PWM duty command range xxxx (0000..FFFF) to motor       *
//*                 mm range (00..01). This command is used to vary              *
//*                 the motor speed and direction.                               *
//*                                                                              *
//* 	Emm       - Read DC motor encoders for position, speed and               *
//*                 acceleration and display the information for motor           *
//*                 mm range (00..01).                                           *
//*                                                                              *
//*     Fxx       - Select filter xx, where the filter ID                        *
//*                 range (00..FF) is in a 2 digit hex byte format,              *
//*                 where 00 => No Filter, 01 => Average Filter,                 *
//*                 02 => Kalman Filter2, 03 => Filter3,...                      *
//*                                                                              *
//* 	Gmm       - Get new motor commands from the current POT readings and     *
//*                 display them for the specific motor mm range (00..01).  This *
//*                 mode will remain in effect until the Hold command is issued. *
//*                                                                              *
//* 	Hmm       - Hold current POT reading and use it for the next PWM duty    *
//*                 command range (0000..FFFF) to motor                          *
//*                 mm range (00..01). This command is used to vary              * 
//*                 the motor speed and direction.                               *
//*                                                                              *
//*     Nxxxx     - Accept total number of samples, XXXX to be read from         *
//*                 selected Potentiometer, where xxxx is in 4 digit             *
//*                 hex byte format  range (00..FFFF)                            *
//*                                                                              *
//* 	Pmm,xx    - Send PWM period command range xx (00..FF) to motor           *
//*                 mm range (00..01). This command is used to vary the          *
//*                 PWM update frequency.                                        *
//*                                                                              *
//* 	Smm       - Read DC motor status including temperature, direction,       *
//*                 speed (RPM read from optical encoders and power state        * 
//*                 (ON/OFF) from motor mm range (00..01).                       *
//*                                                                              *
//* 	Tmm,xx    - Send trajectory profile xx range (00..FF) to motor mm        *
//*                 range (00..01).                                              *
//*                                                                              *
//* 	Vmm,xxxx  - Send velocity command range xxxx (0000..FFFF) to motor       *
//*                 mm range (00..01). This command is used to vary              * 
//*                 the motor speed and direction.                               *
//*                                                                              *
//* 	Zmm;       - Reset the DC motor Controller (brake the motor).  This      *
//*                 command can be used to implement an emergency stop           *
//*                 function.                                                    *
//*                                                                              *
//*                                                                              * 
//*   In order to use PWM2, the CCP2/RB3 pin fuse must be selected when          *
//*   programming the PIC.                                                       *                                                                                                       
//*                                                                              *
//* 	Example:                                                                 *
//*   		Send the following command string to the DC motor controller         *
//* 	to set PWM period to 0xFF using the string:                              *
//*       "P01,FF;"                                                              *
//* 	Where:                                                                   *
//*      	P - Is the motor command to change the PWM period.                   *
//*     	01 - Is the motor ID.                                                *
//*     	FF - Is the 2 character hex PWM period command.                      *
//*                                                                              *
//********************************************************************************
   
#ifndef SPIMSG_LOADED  /* prevent multiple loading */
#define SPIMSG_LOADED


#include "flags.h"                                    // Global flags used for MPSIM debugging 
#include "stdio.h"                                    // I/O support functions  
#include "network.h"                                  	// Include support for networking
 
//**********************************************************************************
// Define messages for each type of command that is sent by the SPI Master and 
// recognized by the selected SPI Slave.
//**********************************************************************************

//**********************************************************************************
//* 	Cmm       - Calibrate motor speed message for motor mm range (00..01). This*
//*                 command is used to vary the motor speed and direction using    *
//*                 two potentiometers with the blades connected to AN0 and AN1.   *
//*                 The true motor speed (RPM) is determined from the counts       *
//*                 received from the optical encoders (if used) and displayed.    *
//**********************************************************************************
typedef struct calibrate_motor_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;         	// Calibrate Command to SPI Slave (see enumeration
                                	// declaration above.)  
 
 	byte MotorID; 					// Selected motor range (0..1)  
    byte Filler[28];				// Padding 
    
} CALIBRATE_MOTOR_TYPE;

//**********************************************************************************
//* 	Dmm,xxxx  - Send PWM duty command message range xxxx (0000..FFFF) to motor *
//*                 mm range (00..01). This command is used to vary                *
//*                 the motor speed and direction.                                 *
//**********************************************************************************
typedef struct send_duty_type
{
  	word MessageID;    				// Message ID range (0000 - FFFF) Hex 

  	COMMAND_TYPE Command;         	// Duty Command to SPI Slave (see enumeration
                                	// declaration above.)  
 
 	byte MotorID; 					// Selected motor range (0..1) Hex
 	word Duty; 						// Duty Cycle range (0000 - FFFF) Hex              	
    byte Filler[26];				// Padding 

} SEND_DUTY_TYPE;

//**********************************************************************************
//* 	Gmm       - Get new motor commands message.  Used to get the current POT   *
//*                 readings and display them for the specific motor mm range      *
//*                 (00..01).  This mode will remain in effect until the Hold      *
//*                 command is issued.                                             *
//**********************************************************************************
typedef struct get_pot_command_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;         	// Get new motor commands from the current POT readings
  	                                // Command (see enumeration declaration above.)  
 
 	byte MotorID; 					// Selected motor range (0..1)  
    byte Filler[28];				// Padding 
    
} GET_POT_COMMAND_TYPE;

//**********************************************************************************
//*                                                                                *
//* 	Hmm       - Hold current POT reading message. It is used for the next PWM  *
//*                 duty command range (0000..FFFF) to motor                       *
//*                 mm range (00..01). This command is used to vary                * 
//*                 the motor speed and direction.                                 *
//**********************************************************************************
typedef struct hold_pot_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;         	// Get new motor commands from the current POT readings
  	                                // Command (see enumeration declaration above.)  
 
 	byte MotorID; 					// Selected motor range (0..1)  
    byte Filler[28];				// Padding 
    
} HOLD_POT_TYPE;

  
#endif

