//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*        Name:  spislave.h                                                     *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  02/12/2002                                                     *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements an SPI Slave Co-Processor controller that    *
//* emulates Microchip's 24LCXX SPI Memory chip interface and acts as a smart    *
//* peripheral, by using the SPI hardware module contained in the PIC18C452      *  
//* micro-controller.  In addition it can act as an SPI network server to an     *
//* SPI Client or function as an SPI based RPN Floating Point co-processor.  In  *
//* the simplest mode of operation, it simulates writing to the EEPROM and       *
//* reading it back under control from the Master SPI controller.  Included is a *
//* diagnostic function that will write a ramp to the EEPROM using SPI and read  *
//* it back to the Host PC via the USART when used with the spimast.c            *
//* application. A more sophisticated SPI Networking mode is also available to   *
//* applications that require message passing with CRC or checksum.  The common  *
//* message format between the SPI Master and SPI Slave is defined in the        *
//* file spimsg.h.  SPI transmit and receive buffers are available to process    *
//* messages to/from the Master controller.                                      *                         
//*                                                                              *
//*      A communication interface is provided that allows the host or SPI or    *
//* I2C Master to request individual sensor readings, average sensor readings,   *
//* calibrating a sensor or perform a sensor processing algorithm using command  *
//* message packets.  The commands available are described below.  This header   *
//* file provides the common Master and Slave message formats and descriptors.   *                                                             *   
//*                                                                              *
//*     This SPI slave controller can be used with a PC host or any SPI master,  *
//* in order to  give any PC the capability to serve as an SPI controller.       * 
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each SPI device has its own unique slave select enable line   *
//* (SS_BAR) and only listens to the SPI Master commands when SS_BAR is cleared. *  
//*                                                                              *
//*    This application emulates a Microchip 24LCXX EEPROM that has 8 registers  *
//* accessed as sub-addresses 1:8 modulo 7.  Sub-address 0 is used as a data     *
//* channel.  All the read and write data modes available for this device        *
//* including: Byte Write, Page Write, Current Address Read, Random Read and     *
//* Sequential Read are emulated by this application.  As far as the Master SPI  * 
//* controller knows, it is talking to a real EEPROM instead of another PIC.     *
//*                                                                              *
//*                                                                              * 
//*     The SPI message format used by this application is for the master to     *
//* send commands to the Digital Pot SPI slave device.  This translates to the   *
//* following sequence of operations for the Master:                             *                    
//*                                                                              *
//*  1) The SPI Co-Processors Command Bytes: X|X|C1|C0|X|X|P1|P0                 *
//*                                                                              *
//*                                                                              *
//*  2) Data is latched in on the rising edge of the SCK.                        *
//*                                                                              *
//*  3) Data is clocked out the SO pin after the falling edge of SCK.            *
//*                                                                              *
//*  4) The Clock Iddle is high.                                                 *
//*                                                                              *
//*  5) The SPI Clock speed is FOSC/16                                           *
//*                                                                              *
//*  6) The Reset Pin, RS_BAR will automatically set all potentiometer data      *
//*     latches to mid scale (Code 80h).                                         *
//*                                                                              *
//*  7) The Shutdown Pin, SHDN_BAR when held low will (provided that the pin is  *
//*     held low for at least 150 ns and CS_BAR is high, will cause the          *
//*     application circuit to into a power saver mode.                          *
//*                                                                              *
//*  8) The device when powered up for the first time, will automatically set    *
//*     reset all potentiometer data latches to mid scale (Code 80h).  A power   *
//*     on will insure that the device powers up in this known state             *
//*                                                                              *
//*  9) This device can operate in SPI Mode 0,0, in which the clock idles in the *
//*     low state.                                                               *
//*                                                                              *
//*  10) The following SPI data transfers are available:                         *
//*                                                                              *
//*    		Master sends data byte  -  Slave sends dummy data byte               *                                                                        
//*    		Master sends data byte  -  Slave sends data byte                     *                                                                  
//*    		Master sends dummy data byte  -  Slave sends data byte               *                                                                         
//*                                                                              *
//*                                                                              *
//* Note:                                                                        *
//*		The SPI networking is handled using a simple data packet messaging       *
//* protocol.  Each message includes an SPI source address field (Master), an SPI*
//* destination address field, the actual data and a CRC or checksum field.  Data*
//* packets that are determined to be corrupted will be retransmitted up to a    *
//* specified number of times.                                                   *
//*                                                                              * 
//*                                                                              *
//*      Future upgrades to this application include: the ability to communicate * 
//* with other SPI slave devices such as Microchip MPC3002 A/D converter, the    *
//* Maxim MAX7219 LED Driver, the Microchip SPI Serial EEPROMS such as the       *
//* 93XXXX Series and the ISD 33270 Voice Recorder IC.  It may also be used to   *
//* communicate as a master or a slave for applications requiring that dual      *
//* capability.  Other upgrades include using SPI to implement the C printf      *
//* function to be used for a "cheap" embedded debugger similar to JTAG used by  *
//* high end micro-processors. Explore using communication protocols and error   *
//* detection/correction algorithms such as parity, CRC and various other codes  *
//* for robust operation and fault tolerance. TMR and N-Module redundancy could  *
//* also be investigated for fault tolerant systems that work in hazardous       *
//* environments. The same SPI networking functions may also be used with the    *
//* asynchronous serial RS-232 interface and the I2C interface.                  *                         
//*                                                                              *
//*                                                                              *
//********************************************************************************

#ifndef SPISLAVE_LOADED  /* prevent multiple loading */
#define SPISLAVE_LOADED
  
   
// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
//#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18F8720     
//#include <p18c452.h>					// Register definitions for PIC18F8720
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
 
#include <ctype.h>

//#include <stdlib.h>
//#include <conio.h>
#include <math.h>
#include <string.h>
#include "stdio.h"
// Variables

extern int Status;												// Status return code

// Pointers and arrays

// Declare SPI Slave Buffer variables here

extern byte SPI_Rx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Receive Buffer
extern byte SPI_Tx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Transmit Buffer
extern byte SPI_Start_Buffer;                       			// Start pointer for ring buffer
extern byte SPI_End_Buffer;										// SPI End Buffer Index
extern byte SPI_Rx_Flag;                        				// Indicates that data was received into the ring buffer
extern word SPI_Timeout;                            			// SPI I/O Timeout counter
extern byte SPI_Timeout_Flag;                           		// SPI Timeout flag
extern word SPI_Number_Characters;						 		// SPI Number of characters in buffer
extern int SPI_Communications_Error;                    		// SPI I/O error flag
                                                 				// -1 = SPI Input Bufer overrun error
                                                 				// -2 = SPI Communications timeout error
                                                 				// -3 = SPI buffer empty error

extern   int SPI_Interrupt_Count;                   			// Number of MSSP interrupts
extern   int SPI_Data_In;										// Data byte read from the SPI Master
extern   int SPI_Data_Out;										// Data byte sent to the SPI Master
extern   int SPI_Status;										// SPI Status returned from routines

extern   int Dummy;												// Dummy Data byte read from the SPI bus
 
extern byte AddressPointer;                         	 		// SPI Device Address pointer used to simulate serial
                                                 				// EEPROM reads and writes
extern byte EEPromPointer;                          	 		// Pointer to EEPROM array

// SPI Services

 
//******************************************************************************
//* InitializeSlaveSPI - Initialize variables and hardware used by the SPI 
//* slave device.
//******************************************************************************

void InitializeSPISlave(void);
 

/********************************************************************
*     Function Name:    ReadSPIByte                                 *
*     Return Value:     contents of SSPBUF register                 *
*     Parameters:       void                                        *
*     Description:      Read single byte from SPI bus (Slave).      *
********************************************************************/
unsigned char ReadSPIByte( void );

/********************************************************************
*     Function Name:    WriteSPIByte                                *
*     Return Value:     Status byte for WCOL detection.             *
*     Parameters:       Single data byte for SPI bus.               *
*     Description:      This routine writes a single byte to the    * 
*                       SPI bus (Slave).                            *
********************************************************************/
unsigned char WriteSPIByte( unsigned char data_out );

/********************************************************************
*     Function Name:    ReadSPIString                               *
*     Return Value:     void                                        *
*     Parameters:       address of read string storage location and *
*                       length of string bytes to read              *
*     Description:      This routine reads a string from the SPI    *
*                       bus (Slave).  The number of bytes to read   *
*                       is determined by parameter 'length'.        *
********************************************************************/
void ReadSPIString( unsigned char *rdptr, unsigned char length );


/********************************************************************
*     Function Name:    WriteSPIString                              *
*     Return Value:     void                                        *
*     Parameters:       address of write string storage location    *
*     Description:      This routine writes a string to the SPI bus.*  
********************************************************************/
void WriteSPIString( unsigned char *wrptr );

//********************************************************************************
//* EnableSPIInterrupts - Enable SPI interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void EnableSPIInterrupts(void);
 

// SPI input with interrupts and SPI buffer

//******************************************************************************
//* SPI_Get_Byte - Get next character from the SPI Rx Buffer.  The function
//* returns a -1 if the routine times out. 
//******************************************************************************
 
byte	SPI_Get_Byte(void);

//********************************************************************************
//* DisableSPIInterrupts - Disable SPI interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void DisableSPIInterrupts(void);
 
