//********************************************************************************
//*         Name:  coprocm_serial.c                                              *               
//*                                                                              *
//*       Author:  Daniel F. Ramirez                                             *
//*                                                                              *
//*          Date:  06/13/2003                                                   *
//*                                                                              *
//*   Description:                                                               *
//*                                                                              *
//*                                                                              *
//*     This is a Master Math Co-Processor application that uses a PIC18F8720 to * 
//* send expressions to the embedded Math Co-Processor using the USART interface,*
//* in order to compute floating point arithmetic expressions in RPN format.  It *
//* can be used for Sumo Robots that require floating point for navigation and   *
//* tracking of an oponent with minimal space, power and weight requirements are *
//* prime candidates for using this co-processor, since it is very compact.      *
//* The master can be a PIC or STAMP or any other micro-controller that requires *
//* a math co-processor and has at least 4 spare I/O pins. In addition to the    *
//* basic arithmetic functions, the math co-processor can also evaluate          *
//* expressions that contain trigonometric and scientific functions.             *
//*                                                                              *
//*      A communication interface is provided that allows the host to use high  *
//* speed asynchronous serial link or SPI or I2C Master to request floating point*
//* equation calculations or to pass it individual sensor readings, average      *
//* sensor readings, calibrating a sensor or to perform a sensor processing      *
//* algorithm using command message packets.  The commands available are         *
//* described below.  This header file provides the common Master and Slave      *
//* message formats and descriptors.                                             *
//*/*                                                                            *
//* The high priority ISR will invoke the "Z;" reset command via the serial      *
//* interface. Both the serial interface and the SPI interface are supported by  *
//* defining the approprate compiler flag.                                       *
//*                                                                              *
//*                                                                              *
//*     This master controller can be used to communicate with a PC host or any  *
//* USART, I2C or SPI slave, in order to  give the microcontroller the capability*
//* to serve as an SPI or I2C slave controller in order to provide it the        *
//* floating point coprocessor functions.                                        *  
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each I2C device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application will include implementing various    *
//* filters including FIR, FFR, Kalman and Median filters.  Both FFT and IFFT    *
//* functions and a Histogram function are also in the works. Advanced algorithms*
//* such as Vector Force Fields (VFF), Fuzzy Logic and Neural Networks are new   *
//* areas that may be explored using this application.   In addition, this       *
//* application may be able to take advantage of the new DSPic Signal Processors *
//* (DSP's) become available from Microchip.                                     *
//*                                                                              *
//* Usage:                                                                       *
//*	The following Math Co-Processor Controller command messages are available:   *
//*                                                                              * 
//* calculate    - Evalueates the RPN expressions that are stored in the         *
//* message expression buffer.  Multiple expressions can be sent to the          *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.      *
//* Large expressions can span multiple messages to the co-processor.            *
//*                                                                              * 
//* get_answers  - Get the results from the latest arithmetic operations         *
//* carried out during the last cycle. Up to 7 separate single precision floating*
//* point and 3 double precision  values may be returned during a single message *
//* transfer.                                                                    *
//*                                                                              *
//* clear_memory - Erases the contents of the Math co-processor's storage        * 
//* registers. These same registers are used for storing and retrieving temporary*
//* values. It clears floating point registers 0..MAX_FLOAT_REGISTERS.           *
//*                                                                              *
//* store   - Stores a floating point value to a selected floating point         *
//* register. The register ID range is 0..MAX_FLOAT_REGISTERS.                   *                                                            
//*                                                                              *
//* recall 	- Retrieves a floating point value from the selected floating        *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.             *                                                         
//*                                                                              *                                                                     
//*	GetStatus - Get the Math Co-processor status including current state, and    *  
//* any error conditions encountered i.e. division by 0, overflow, underflow.    *
//*                                                                              *
//* ResetCommand - Reset the DC motor Controller (brake the motor).  This        *
//* command can be used to implement an emergency stop function.                 *  
//*                                                                              *
//*                                                                              * 
//*                                                                              *
//********************************************************************************
    
 
// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
//#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18F8720
#endif

#ifdef PIC18F452     
#include <p18f452.h>					// Register definitions for PIC18F8720
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
#include <p18f6620.h>
#endif
 
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <spi.h>                                        // SPI library functions
#include <pwm.h>                                        // PWM library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library function
#include <delays.h>                                     // High resolution delay functions
#include "UARTIntC.h"									// High SPeed Serial Interface
#include "delays.h"

//#include "portutil.h"                                 // Port utility functions
#include "timeutil.h"
#include "stdio.h"                                      // Serial I/O support functions
#include "network.h"                                   	// SPI Networking header
//#include "dcmotor.h"                                   	// SPI DC Motor Controller common message header
#include "spimast.h"                                   	// SPI Master services 
#include "coproc.h"										// Include Common Math Co-processor header
#include "coprocm.h"									// Include Math Co-processor Services header
//#include "dcmotorm.h"                                   // DC Motor Controller functions
 #include "UARTIntC.h"
#include "delays.h"

 
//********************************************************************************
// Constant Definitions
//********************************************************************************
#define CR 13


unsigned char RxBuffer[34];
unsigned char TxBuffer[34];

void low_isr(void);

// serial interrupt taken as low priority interrupt
#pragma code uart_int_service = 0x08
void uart_int_service(void)
{
	_asm	goto low_isr	_endasm
	
}
#pragma code

#pragma	interruptlow low_isr save=section(".tmpdata")
void low_isr(void)
{	
	// call of library module function, MUST
	UARTIntISR();
}

void main(void)
{
	// Test data array
	unsigned char cArray[10]
	   = { 'A','B', 'C', 'D', 'E', 'F', 'G','H','I' , 'J'};
	
	
	// temporary local array	   
	unsigned char writtenArray[10];
	
	// local variables for all test cases and their initialisation
	unsigned char chData;
	unsigned char *pReadArray;
	unsigned char *pWriteArray;
	unsigned char j = 0;
	unsigned char i  = 0;
	unsigned char l = 0;
	unsigned char k = 0;			
	pReadArray = pWriteArray = writtenArray;	


	// call of library module , MUST
	UARTIntInit();	

 	// Initialize Rx and Tx buffers

 	for (i=0; i<34; i++)
	{
		RxBuffer[i] = 0;
		TxBuffer[i] = i;
	}
	
// TEST 1
// Sending Test array data, 100 times, from micro-controller to PC
// and waiting infinitely at the end.
/*
   for(j = 0; j < 100; j++)
	{
		k = 0;
		while(k < 10)
		{
			if(UARTIntPutChar(cArray[k]))
				 k++;			
		}		   
	}	
	while(1);
*/

 
// TEST 2
// Similar to TEST1
/*	
	for(j = 0; j < 100; j++)
	{
		k = 0;
		while(k < 10)
		{  
			if(!vUARTIntStatus.UARTIntTxBufferFull)
			 UARTIntPutChar(cArray[k++]);
		}		   
	}	
	while(1);
*/	
 
 
// TEST 3
//	Sending Test array data, continuously, from micro-controller to PC
//  Usage of status flag can be seen in below example
	
/*	
	while(1)  
	{
		i = 0;
		while(i<10)
		{   //status flag
			if(!vUARTIntStatus.UARTIntTxBufferFull)
			{
				 UARTIntPutChar(cArray[i]);		
				 i++;
			 }
		 }		
	}

*/	

 	
//	TEST 4
//	Sending Test array data, continuously, from micro-controller to PC
// 	Usage of UARTIntGetTxBufferEmptySpace() function can be seen below

 /*	
	while(1)
	{
		i = 0;
		while(i < 10) //works  change to pointer
		{   
			l = 0;
			l = UARTIntGetTxBufferEmptySpace();
			for(k = 0; k < l; k++)
			{   if((i+k) < 10)
				UARTIntPutChar(cArray[i+k]);			
			}
			i = i+ k;						
		}					
	}	

*/	

 
//	TEST 5
//  Checking transmission and reception simultaneously.
//  Sending Test array data, 100 times ,from micro-controller to PC.
//  During this transmission, a single character 'A' or 'B' or 'C' 
//  can be sent from PC to micro-controller. This can be seen as LED
//  glows on PICDEM2 board. At the end of transmission micro-controller
//  waits infinitely at the end.
/*   
  	TRISB = 0;
	PORTB = 0;
    k = 0;

	while(k < 100)
	{
		for(j = 0; j < 100; j++)
		{
			i = 0;
			do{
				if(vUARTIntStatus.UARTIntTxBufferEmpty)
					UARTIntPutChar(cArray[i++]);
			}while(i < 10);
		}
		
		if( !(vUARTIntStatus.UARTIntRxError) && 
		    !(vUARTIntStatus.UARTIntRxOverFlow) &&
		    !(vUARTIntStatus.UARTIntRxBufferEmpty))
		{    
			if(UARTIntGetChar(&chData))
			PORTB = chData;
		}
		
		k++;
	}
	
	while(1);			


*/
 
//	TEST 6
//  Chunk of 30 characters can be sent from PC to micro-controller.
//  Each received character at micro-controller is read and immediately 
//	sent back to PC.
/*	
	while(1)
	{
		while(!vUARTIntStatus.UARTIntRxBufferEmpty)
		{
			UARTIntGetChar(&chData);			
			if(!vUARTIntStatus.UARTIntTxBufferFull)
				UARTIntPutChar(chData);
		};
	};
*/


 
//	TEST 7
// 	Chunk of data is sent from PC to micro. Data received at micro is
//	read in to temporary buffer. The data is immediately sent back to 
//  PC. Usage of pointers is displayed in below example
 /*	
	while(1)   
	{		   
		l = 0;
		l = UARTIntGetRxBufferDataSize();	
		while(l > 0)
		{
			if(UARTIntGetChar(pWriteArray))
			{				
				if(!vUARTIntStatus.UARTIntTxBufferFull)
				    UARTIntPutChar(*pWriteArray);
				l--; pWriteArray++;
				if(pWriteArray == (writtenArray + 10))
					pWriteArray = writtenArray;
			}
		}
	}
		

*/
 
//	TEST 8
// 	Chunk of data is sent from PC to micro. Data received at micro is
//	read in to temporary buffer. The data is sent back to PC. Here 
//  reception and transmission is done independently.
 	
/*	
	while(1)   
	{					   
		l = 0;
		l = UARTIntGetRxBufferDataSize();	
		while(l > 0)
		{
			if(UARTIntGetChar(pReadArray))
			{				
				l--; pReadArray++;
				if(pReadArray == (writtenArray + 10))
					pReadArray = writtenArray;
			}
		}		
		if((pWriteArray < pReadArray) || ((pWriteArray - pReadArray) == 9 ))
		{  
			if(!vUARTIntStatus.UARTIntTxBufferFull)
			{
				UARTIntPutChar(*pWriteArray);
	    		i--; pWriteArray++;
 			    if(pWriteArray == (writtenArray + 10))				  			
					pWriteArray = writtenArray;		
			}
		}									
	}
	
*/

//	TEST 9 
// 	a Tx message containing a ramp 0..33 is sent from the Master and a response in the form of a Rx message is
//  transmitted back from the Slave as a response.  This goes on indefinitely 
  
	while(1)  
	{
		// Flush RxBuffer
 		for (i=0; i<34; i++)
		{
			RxBuffer[i] = 0;
		}
		

		// Send data

	 	i=0;
		while (i<34)
		{   //status flag
			if(!vUARTIntStatus.UARTIntTxBufferFull)
			{
				 	UARTIntPutChar(TxBuffer[i]);		
				 	//UARTIntPutChar(cArray[i]);		
 			 		i++;
			 }
		 }

		// Receive data 
	 	i=0;
 		while (i<34)
		{
			while(!vUARTIntStatus.UARTIntRxBufferEmpty)
			{
				UARTIntGetChar(&chData);
				RxBuffer[i] = chData;			
 				i++;
			}

		}


		i = 0;
		
	}


	
}
