//********************************************************************************
//*         Name:  coprocs_parallel.c                                            *               
//*                                                                              *
//*       Author:  Daniel F. Ramirez                                             *
//*                                                                              *
//*          Date:  06/13/2003                                                   *
//*                                                                              *
//*   Description:                                                               *
//*                                                                              *
//*                                                                              *
//*     This is a Math Co-Processor application that utilizes a PIC18F8720 to    * 
//* send expressions to the embedded Math Co-Processor using the PSP interface,*
//* in order to compute floating point arithmetic expressions in RPN format.  It *
//* can be used for Sumo Robots that require floating point for navigation and   *
//* tracking of an oponent with minimal space, power and weight requirements are *
//* prime candidates for using this co-processor, since it is very compact.      *
//* The master can be a PIC or STAMP or any other micro-controller that requires *
//* a math co-processor and has at least 4 spare I/O pins. In addition to the    *
//* basic arithmetic functions, the math co-processor can also evaluate          *
//* expressions that contain trigonometric and scientific functions.             *
//*                                                                              *
//*      A communication interface is provided that allows the host to a use high*
//* speed Parallel Slave Port PSP, asynchronous serial link or SPI or I2C Master * 
//* to request floating pointequation calculations or to pass it individual      *
//* sensor readings, average sensor readings, calibrating a sensor or to perform *
//* a sensor processing algorithm using command message packets.  The commands   *
//* available are described below.  This header file provides the common Master  *
//* and Slave message formats and descriptors.                                   *
//*                                                                              *
//*                                                                              *
//* The high priority ISR will invoke the "Z;" reset command via the serial      *
//* interface. Both the serial interface and the SPI interface are supported by  *
//* defining the approprate compiler flag.                                       *
//*                                                                              *
//*                                                                              *
//*     This slave controller can be used to communicate with a PC host or any   *
//* PSP, I2C or SPI master, in order to  give the microcontroller the          *
//* capability to serve as an SPI or I2C slave controller in order to provide it * 
//* the floating point coprocessor functions.                                    *  
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each I2C device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application will include implementing various    *
//* filters including FIR, FFR, Kalman and Median filters.  Both FFT and IFFT    *
//* functions and a Histogram function are also in the works. Advanced algorithms*
//* such as Vector Force Fields (VFF), Fuzzy Logic and Neural Networks are new   *
//* areas that may be explored using this application.   In addition, this       *
//* application may be able to take advantage of the new DUSARTc Signal Processors *
//* (DSP's) become available from Microchip.                                     *
//*                                                                              *
//* Usage:                                                                       *
//*	The following Math Co-Processor Controller command messages are available:   *
//*                                                                              * 
//* calculate    - Evalueates the RPN expressions that are stored in the         *
//* message expression buffer.  Multiple expressions can be sent to the          *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.      *
//* Large expressions can span multiple messages to the co-processor.            *
//*                                                                              * 
//* get_answers  - Get the results from the latest arithmetic operations         *
//* carried out during the last cycle. Up to 7 separate single precision floating*
//* point and 3 double precision  values may be returned during a single message *
//* transfer.                                                                    *
//*                                                                              *
//* clear_memory - Erases the contents of the Math co-processor's storage        * 
//* registers. These same registers are used for storing and retrieving temporary*
//* values. It clears floating point registers 0..MAX_FLOAT_REGISTERS.           *
//*                                                                              *
//* store   - Stores a floating point value to a selected floating point         *
//* register. The register ID range is 0..MAX_FLOAT_REGISTERS.                   *                                                            
//*                                                                              *
//* recall 	- Retrieves a floating point value from the selected floating        *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.             *                                                         
//*                                                                              *                                                                     
//*	GetStatus - Get the Math Co-processor status including current state, and    *  
//* any error conditions encountered i.e. division by 0, overflow, underflow.    *
//*                                                                              *
//* ResetCommand - Reset the DC motor Controller (brake the motor).  This        *
//* command can be used to implement an emergency stop function.                 *  
//*                                                                              *
//*                                                                              * 
//*                                                                              *
//********************************************************************************
    
 
// Initialize Calculator compilation flags here

//#define SINGLE_PRECISION                               // Include Single Precision code

//#define TEST                                           // Test code

// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
//#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
//#include <p18c452.h>					// Register definitions for PIC18F8720
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
 
#include <ctype.h>

                        
#ifdef SERIAL_DRIVER
#include "stdio.h"                                      // Header file for stdio file
#endif

#include "UARTIntC.h"
#include "delays.h"
#include "network.h"                                    // Include USART Network Support
//#include "spislave.h"                                   // Include USART Slave Services
#include "coproc.h"										// Include Common Math Co-processor header
#include "coprocs.h"									// Include Math Co-processor Services header
#include "math.h"
#include "constant.h"									// Jmath constants 
#include "jmath.h"										// Jmath headers

// Constants

#define TRUE 1
#define FALSE 0


// Port A

#define SS			PORTFbits.RF7						// SPI Slave Co-processor Select/Status
#define SS_DIR		TRISFbits.TRISF7					// SPI Slave Co-processor Select Direction/Status 
 
// Port B

#define TRACE0		PORTBbits.RB0						// Trace bit 0 for logic analyzer
#define TRACE0_DIR	TRISBbits.TRISB0

// Port C

 
// PORT D definitions

#define LeftEmitter PORTDbits.RD0							  // Left IR emitter (used for strobe)
#define RightEmitter PORTDbits.RD1							  // Right IR emitter (used for strobe)
 
// Declare global variables here

#define BufferSize		20			// Floating point buffer size
#define StackSize		 5			// HP-45 stack (5 levels)
#define RegisterSize	10			// HP-45 Registers (1-9)

#define X_LOCATION 0				// X Location in stack
#define Y_LOCATION 1				// Y Location in stack
#define Z_LOCATION 2				// Z Location in stack
#define T_LOCATION 3				// T Location in stack (top of stack)

 
#define BUSY					 PORTBbits.RB0          // Slave BUSY Status Pin (1=Busy, 0=Finished)
#define BUSY_DIR				 TRISBbits.TRISB0       // Slave BUSY Status Pin Direction  

#define RD_BAR					 PORTEbits.RE0          // RD_BAR Control Pin 
#define WR_BAR                   PORTEbits.RE1          // WR_BAR Control Pin   
#define CS_BAR                   PORTEbits.RE2          // CS_BAR Control Pin  

#define RD_BAR_DIR				 TRISEbits.TRISE0       // RD_BAR Control Pin Direction
#define WR_BAR_DIR               TRISEbits.TRISE1       // WR_BAR Control Pin Direction 
#define CS_BAR_DIR               TRISEbits.TRISE2       // CS_BAR Control Pin Direction 


// Bit definitions for flag register
 
typedef struct slave_port_flags
{
        unsigned ERR:1;				// err  	equ	00h		;Error flag bit
        unsigned OUTRDY:1;			// OUTRDY	equ	01h		;Output data ready flag
        unsigned INFULL:1;			// INFULL	equ	02h		;Input data received flag
        unsigned SPARE1:1;
        unsigned SPARE2:1;
        unsigned SPARE3:1;
        unsigned SPARE4:1;
        unsigned SPARE5:1;
 
} SLAVE_PORT_FLAGS ;

// Register definitions

static volatile SLAVE_PORT_FLAGS FlagReg;			// FLAGREG	equ	20h		;Flag bit register

// Register definitions

static volatile SLAVE_PORT_FLAGS FlagReg;			// FLAGREG	equ	20h		;Flag bit register
byte OutData;						// OUTDATA	equ	21h		;Output data
byte InData;						// INDATA	equ	22h		;Input data
//int	Count = 0;						// COUNT	equ	23h		;Count of times output register read

  
// General Math Coprocessor functions

#define GOLD    'a'					// Gold function key    
#define ENTER	'^'                 // Enter key  
#define ROLL    '|'					// Roll key   
#define SWAP    '~'					// Swap X with Y key  
#define STORE   '>'					// Store X into register N, N=1..9   
#define RECALL  '<'					// Recall register N, N=1..9 into X  
#define LASTX   '='                 // Recall register Last X into X

// Math Operators 

#define PLUS	'+'					// "+" Operator
#define MINUS	'-'					// "-" Operator
#define TIMES	'*'					// "*" Operator
#define DIVIDE	'/'					// "/" Operator

#define PERIOD	'.'					// "." Symbol
#define SEMICOLON	';'				// ";" Symbol

// Math functions


// Trigonometric Functions

#define SIN		'A'					// Sine function
#define ASIN  	'B'					// Tangent function
#define COS		'C'					// Cosine function
#define ACOS  	'D'					// Arc Cosine function
#define TAN		'E'					// Tangent function
#define ATAN	'F'					// Arc Tangent function
#define ATAN2	'G'					// Arc Tangent Two function
 
// Scientific Functions

#define SQRT	'H'					// Square root function
#define SQR		'I'					// Square function
#define EXP		'J'					// Exponential function E**X
#define LOG		'K' 				// Log function
#define Log10	'L' 				// Log base 10 function
#define POWER	'M' 				// X**N  function
#define MAX		'N'                 // Maximum of x and y
#define MIN		'O'                 // Minimum of x and y
#define ABS		'P'                 // Absolute value of x
#define SIGN	'Q'                 // Sign of x and y

// Special functions

#define RAD 	'R'					// Convert to radians
#define DEG 	'S'					// Convert to degrees
#define FIX 	'T'					// Rounds x towards 0
#define FLOOR 	'U'					// Rounds x towards - Infinity
#define CEIL 	'V'					// Rounds x towards + Infinity
#define ROUND 	'W'					// Round x to nearest integer
#define PRECISION 	'X'				// Specify maximum number of digits precision up to machine precision used for display purposes
#define MOD		'Y'					// Mod Function identical to C mod function 


//************************************************************************************************

typedef union message_union_type
{
    CALCULATE_TYPE CalculateCommand;							// Calculate command message
	GET_ANSWERS_TYPE GetAnswersCommand;							// Get answers command message
	CLEAR_MEMORY_TYPE CLearMemoryCommand;						// Clear memory command message
	STORE_TYPE StoreCommand;									// Store command message
	RECALL_TYPE RecallCommand;                          		// Recall command message	
 	GET_STATUS_COMMAND_TYPE GetStatusCommand;					// Get status message
	RESET_COMMAND_TYPE ResetCommand;					 		// Reset command message

	COMMAND_MESSAGE_TYPE SelectedCommand;						// Selected command	
	MESSAGE_TYPE Message;										// Unformatted message
} MESSAGE_UNION_TYPE;

// Define the Math Co-processor Commands message
MESSAGE_UNION_TYPE CoProcessorMessage;
  
#ifdef SINGLE_PRECISION
float Registers[MAX_FLOAT_REGISTERS];
float Answers[MAX_ANSWERS];
#else
double Registers[MAX_FLOAT_REGISTERS];
double Answers[MAX_ANSWERS];
#endif

byte PSP_Rx_Buffer[PSP_BUFFER_SIZE];                  	// PSP Receive Buffer
byte PSP_Tx_Buffer[PSP_BUFFER_SIZE];                  	// PSP Transmit Buffer
byte PSP_Start_Buffer=0;                         		// Start pointer for ring buffer
byte PSP_End_Buffer = 0;								// PSP End Buffer Index
int PSP_Interrupt_Count = 0;							// Number of PSP interrupts
byte PSP_Rx_Flag = FALSE;                        		// Indicates that data was received into the ring buffer
byte PSP_Status;
 

 
char af1[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
/*
char Period[] = ".";
char Plus[] = "+";
char Minus[] = "-";
char Blank[] = " ";
char E[] = "E";

//char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 
extern char Operators[];                               // Math operators declared in keypad.c

char NumericChars[] = {'.','E',0};                       // Numeric characters
*/
unsigned char DisableLift = FALSE;								 // Disable the lift flag

static int Index = 0;                            		// USART Ring Buffer index
static byte CurrentState = 0;                    		// Current USART state variable

static byte SyncFlag = FALSE;							 //  PSP Master/Slave Sync flag
static byte GetStatusFlag = FALSE;						 //  PSP Master/Slave Get Status flag
static byte AnswerIndex = 0;								// Index into co-processor answers buffer
extern byte ReceivedDataFlag ;
int NumChars;

extern byte Ch;
 
// Declare function headers here

void main(void);
void InitializeMathCoprocessor(void);
void InitializeParallelPort(void);

 

// HP-45 Functions

void EvaluateRPN(void);
double Calculator(double Argument1, char Operator, double Argument2);
void Lift(void);
void Drop(void);
void Roll(void);
void Swap(void);
 

// Floating point conversion functions

void Normalize(double *Value, int *Exponent);
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits);

#ifdef SINGLE_PRECISION
void FloatToAscii(float Number, char * s);
void AsciiToFloat(float Number, char * s);
#endif
void DoubleToAscii(double Number, char * s);
void AsciiToDouble(double *Number, char * s);
int Member(char Ch, char * Set);

float GetFloat(void);
double GetDouble(void);
//double log10(double x);
double pow10(int x);
void ConfigureInterrupts(void);
char GetKey(void);

//char TheOperator;        // Current operator function key: "+","-","*","/"
//char TheEqualSign;       // Equal sign, "="

double Argument1;
double Argument2;
double Result;

// Calculator specific variables

extern char Buffer[BufferSize];				// Floating point Buffer
char StackLabel[]={'X','Y','Z','T'};		// Stack register labels
double Stack[StackSize];					// HP-45 Stack 
double Registers[RegisterSize];				// HP-45 Registers 1-9
double X = 0.0;								// X register (Display)
double Y = 0.0;								// Y register
double LastX = 0.0;							// Last X register
byte RegisterNumber = 0;					// Register number 1..9
//int CharacterCount = 0;								// Character count
int BufferCount = 0;								// Buffer count
int MessageBufferIndex = 0;							// Message buffer index
 
byte KeyValue; 
 
double my_x;
byte TheCharacterRead;


byte ControllerState; 		// State of selected slave controller, range (0..255) 
											// 		0 => NO_STATES		No States    
											//		1 => INITIALIZED	The selected slave contoller is Initialzed 
											//		2 => DONE 	 	 	The selected slave contoller is done processing current command
											// 		3 => BUSY 	 		The selected slave contoller is busy processing current command 
											// 		4 => ERROR 	 		The selected slave contoller encountered an error processing the current command 

byte ControllerErrorCode; 	// Error code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_ERRORS 		No error codes  
											// 		1 => FAILURE 	 	Failed to complete command
											//  	2 => SUCCESS  	 	Completed command successfully 
 
byte ControllerDiagnosticCode; 	// Diagnostic message code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_DIAGNOSTICS 	No Diagnostic messages
											//  	1 => DIAGNOSTIC 1  	Diagnostic message 2 
											// 		2 => DIAGNOSTIC 2 	Diagnostic message 1
 
 


 
   
 

// Interrupt service routines for coprocs_parallel.c

//********************************************************************************
// Interrupt Code
//********************************************************************************

// Designate ServiceInterrupt as an interrupt function and save key registers

void Service_Int(void);

// serial interrupt taken as low priority interrupt
#pragma code psp_int_service = 0x08
void psp_int_service(void)
{
	_asm	goto Service_Int	_endasm
	
}
 
#pragma code

//#pragma	interruptlow Service_Int save=section(".tmpdata")
#pragma	interruptlow Service_Int save=section(".tmpdata"),PROD
 
//  Interrupt Handler

//********************************************************************************
//* Interrupt Service Routine
//*	Inputs:	FLAGREG - Flag register to/from the main routine:
//*				Bit 1: OUTRDY -	To Service_Int, indicates data
//*						ready in output queue
//*		OUTDATA - Output data queue
//*		PIR1	- Interrupt flag register
//*		TRISE	- Parallel slave port flag register
//*		PORTD	- Input data from slave port
//*
//*	Outputs:
//*		PORTD	- Output data to slave port
//*		INDATA	- Input data queue
//*	       	FLAGREG - Flag register to/from the main routine:
//*				Bit 0: ERROR -  From Service_Int, indicates input 
//*						buffer overflow
//*				Bit 2: INFULL -	From Service_Int, indicates data
//*						received and in INDATA
//*********************************************************************************
 
void Service_Int(void)
{
 
	int i,j;						// Local loop variables

	if (PIR1bits.PSPIF)				// btfss	PIR1,PSPIF	;Test for Peripheral interrupt
	{
		PIR1bits.PSPIF = 0;			// bcf	PIR1,PSPIF	;Clear Peripheral interrupt
									// bsf	STATUS,RP0	;Select Page 1
		if (PSPCONbits.IBF)			// btfss	TRISE,IBF	;Check if input data ready
		{
									// bcf	STATUS,RP0	;Input ready, select Page 0
			FlagReg.INFULL = 1;		// bsf	FLAGREG,INFULL	;Set flag for main routine
 	
			PSP_Rx_Buffer[PSP_End_Buffer] = PORTD;
 
			// Check for buffer full indicating the end of message

       		//if (PSP_End_Buffer ==  (PSP_BUFFER_SIZE - 2))
       		if (PSP_End_Buffer ==  PSP_BUFFER_SIZE )
        	{
   				BUSY = 1;										// Let the Master know the Slave is Busy processing the message
        		PSP_Rx_Flag = TRUE;								// Set the PSP message Received Data flag
        	
       			//PSP_Number_Characters = 0;

       			PSP_End_Buffer = 0;

         		// Copy the entire DC Motor Controller message sent from the PSP Master
   				// to local storage for processing.
   			
  				//memcpy(&MotorMessage.Message.Data, PSP_Rx_Buffer, sizeof(PSP_Rx_Buffer));  
  			  		
   				for (i=0; i<PSP_BUFFER_SIZE; i++)
				{
  			
					CoProcessorMessage.Message.Data[i] = PSP_Rx_Buffer[i];
				}

 	         	BUSY = 0;										// Let the Master know the Slave is done processing the message
   				   			
        	}
        
			// Increment buffer index 
 			
			PSP_End_Buffer++;										 			// Increment the PSP Ring Buffer pointer
        	//PSP_End_Buffer = PSP_End_Buffer % PSP_BUFFER_SIZE;        	 	// Wrap the EndBuffer pointer if necessary
        

		}
	}
	else
	{
		goto	Intout;				// goto	Intout		;Not a Peripheral interrupt, exit
	}

Notinput:
	if (!PSPCONbits.OBF)				// btfsc	TRISE,OBF	;Check if output data read
	{
									// bcf	STATUS,RP0	;Select Page 0
		if (FlagReg.OUTRDY)			// btfss	FLAGREG,OUTRDY	;Check if data in output queue
		{
    		//if (PSP_End_Buffer != PSP_Start_Buffer)
	 		//{

    			// Disable general interrupts

    			//INTCONbits.GIE = 0;                                    // Disable all interrupts

     			//  Get the oldest byte received from the ring buffer.  
   
 
				PORTD = PSP_Tx_Buffer[PSP_Start_Buffer];		// movf	OUTDATA,W	;Get data from queue
											                    // movf	PORTD, F	;Put data in output buffer
				

				FlagReg.OUTRDY = 0;		// bcf	FLAGREG,OUTRDY	;Clear flag for main routine	
    			PSP_Start_Buffer++;
				//PSP_Start_Buffer = PSP_Start_Buffer % (PSP_BUFFER_SIZE - 1);    // Wrap the StartBuffer pointer if necessary
				PSP_Start_Buffer = PSP_Start_Buffer % PSP_BUFFER_SIZE;    // Wrap the StartBuffer pointer if necessary

  
    			// Enable general interrupts

    			//INTCONbits.GIE = 1;                                    // Enable all interrupts
 

		}
	}
  
Intout:
									// bsf	STATUS,RP0	;Select Page 1
	if (PSPCONbits.IBOV)				// btfsc	TRISE,IBOV	;Check input buffer overflow flag
	{								// bcf	STATUS,RP0	;Select Page 0
		FlagReg.ERR = 1;			// bsf	FLAGREG,err	;Set error flag for main routine
	}
									// retfie			;Re-enable GIE and return
									// bcf	STATUS,RP0	;Select Page 0
		 							// retfie		;Re-enable GIE and return

      	// Increment PSP Interrupt counter
      	PSP_Interrupt_Count++;
	
}

//******************************************************************************
//* main - Slave Co-processor application accepts expressions and commands from the 
//* master controller, evaluates them and sends back the answers. 
//******************************************************************************
#ifdef SLAVE_COPROCESSOR

 
void main (void)
{

	// local variables for all test cases and their initialisation
	unsigned char RxBuffer[34];
	unsigned char TxBuffer[34];
	unsigned char chData;
	int i;                                   // Index 
	byte Ch;

	
	
#ifdef STDIO_INTERRUPTS	
	ReceivedDataFlag =  FALSE;
#endif

 	NumChars = 0;							// Initialize the number of characters read from the message buffer
	BufferCount = 0;								// Buffer count
	AnswerIndex = 0;								// points to current answer in Answer[] array				



 	// Setup timers 

	SetupTimers();  

   // Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous or to 19.2 KB for high speed serial link

   	InitializeUsart();
 
	// Initialize math test package 

	InitializeMath();

	// Initialize the Math Co-processor 
	
	InitializeMathCoprocessor();

	// Initialize the 8-Bit Parallel Slave Port

	InitializeParallelPort();

    printf("PIC18C452 Slave Parallel Port Controller Application ... \r\n");
     	
    BUSY = 0;												// Let the Master know the Slave is ready to receive data


//****************************************************************************************************
//****************************************************************************************************  
#ifdef NEW_CODE
     
  	// Test the scientific functions

/*
    // log - Natural log test

	my_x = log(1.0);
	my_x = log(2.0);
   	my_x = log(2.78182818);
 	my_x = log(3.14159263);


    // log10  - Log base 10 test

	my_x = log10(1.0);
	my_x = log10(2.0);
	my_x = log10(10.0);
	my_x = log10(100.0);
	my_x = log10(1000.0);
	my_x = log10(3.14159263);    

	// exp - exponential function test

*/
  
    // The my_sqrt funtion works !!!
    
 	my_x = sqrt(4.0);
	my_x = sqrt(16.0);
	my_x = sqrt(144.0);

	
    my_x = sqrt(25.0);
    my_x = sqrt(1024);
    my_x = sqrt(144.0);
    
 
	my_x = sqrt((double)25.0);
 	
	// Test J. Crenshaw's sqrt function
	
	my_x = sqrt(2.0);
	my_x = sqrt(4.0);
	my_x = sqrt(16.0);
	my_x = sqrt(144.0);
	
	// Test Trig functions

    // Sin and Asin test

	my_x =   sin(radians(30.0));
    my_x =   degrees(asin(my_x));

	my_x =   sin(radians(45.0));
    my_x =   degrees(asin(my_x));

	my_x =   sin(radians(60.0));
    my_x =   degrees(asin(my_x));

    // Cos and Acos test

	my_x =   cos(radians(30.0));
    my_x =   degrees(acos(my_x));

	my_x =   cos(radians(45.0));
    my_x =   degrees(acos(my_x));

	my_x =   cos(radians(60.0));
    my_x =   degrees(acos(my_x));

    // Tan and Atan test

	my_x =   tan(radians(30.0));
    my_x =   degrees(atan(my_x));

	my_x =   tan(radians(45.0));
    my_x =   degrees(atan(my_x));

	my_x =   tan(radians(60.0));
    my_x =   degrees(atan(my_x));

    // Atan2(y,x) test

	my_x = degrees(atan2(0.0, 1.0));  	// 0.0 Degrees
	my_x = degrees(atan2(1.0, 0.0));  	// 90.0 Degrees
	my_x = degrees(atan2(0.0, -1.0));	// 180.0 Degrees
	my_x = degrees(atan2(-1.0, 0.0));  	// 270.0 Degrees
	my_x = degrees(atan2(0.5, 0.5));	// 45.0 Degrees
	my_x = degrees(atan2(0.5, -0.5));	// 135.0 Degrees
	my_x = degrees(atan2(-0.5, -0.5));	// 225.0 Degrees
	my_x = degrees(atan2(-0.5, 0.5));	// 315.0 Degrees
#endif


//****************************************************************************************************
//****************************************************************************************************



 
// Initialize test data messages

#ifdef DEBUG
	USART_Rx_Flag = TRUE;													// Flag indicating that a message was received
  	CoProcessorMessage.CalculateCommand.MessageID = 1;    				// Message ID (0000 - FFFF) Hex

  	CoProcessorMessage.CalculateCommand.Command = CALCULATE_COMMAND;           // Send Calculate Command to math coprocessor   (see enumeration
      
                              								// declaration in network.h)

	// Copy test buffer which is initialized in mpsim.inc with Coprocessor test data

    for (i=0; i<strlen(InputBuffer); i++)
	{
    	CoProcessorMessage.CalculateCommand.Buffer[i] = InputBuffer[i];					// RPN expression to be evaluated: "12.0E2.0/...";
		//Ch = (char) CoProcessorMessage.CalculateCommand.Buffer[i];
 	}
 	 
#endif


#ifdef TEST_1
	//***********************************************************************************************
    // Test code - Send a ramp 0..PSP_BUFFER_SIZE to the Coprocessor and receive a ramp back from the
    // Coprocessor unlimited number of times.
    
 	for (i=0; i<PSP_BUFFER_SIZE; i++)
	{
 		USART_Rx_Buffer[i] = 0;                  		// Clear USART Receive Buffer 
 		//USART_Tx_Buffer[i] = i;                  	// Initialize USART Transmit Buffer to a ramp 
 		USART_Tx_Buffer[i] = PSP_BUFFER_SIZE-i;      	// Initialize USART Transmit Buffer to a reverse ramp 
	}
 
	while(1)
	{
		// Wait for the Sync byte from Master first

 		Ch = 0;
   	   	while (Ch != SYNC_BYTE)
		{
			UARTIntGetChar(&Ch);
		}

   	   	//while (ReadHexByte() != SYNC_BYTE);

		// Send ACK byte to Master indicating it received the message

		UARTIntPutChar(ACK_BYTE);
		//hex2(ACK_BYTE);

 		//for (i=0; i<PSP_BUFFER_SIZE; i++)
 		for (i=0; i<10; i++)
		{
			UARTIntGetChar(&USART_Rx_Buffer[i]);
			  
			//USART_Rx_Buffer[i] = ReadHexByte();
		}

		// Wait for the Sync byte from Master first

 		Ch = 0;
   	   	while (Ch != SYNC_BYTE)
		{
			UARTIntGetChar(&Ch);
		}

   	   	//while (ReadHexByte() != SYNC_BYTE);

		// Send ACK byte to Master indicating it received the message

		UARTIntPutChar(ACK_BYTE);
	 
		//hex2(ACK_BYTE);

 		//for (i=0; i<PSP_BUFFER_SIZE; i++)
 		for (i=0; i<10; i++)
		{
			UARTIntPutChar(USART_Tx_Buffer[i]);
			//hex2(USART_Tx_Buffer[i]);
		}


   	}
 	//***********************************************************************************************

#endif  
 


   while(1)
   {
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
 
#ifdef USART_BUFFER_INTERFACE

		// Flush the Rx, Tx and message buffers

    	for (i=0; i< PSP_BUFFER_SIZE; i++)
    	{

			USART_Rx_Buffer[i] = 0;
			USART_Tx_Buffer[i] = 0;
 			CoProcessorMessage.Message.Data[i] = 0;
   		} 

		// Synchronize with Master 
		//SyncWithMaster();	   	 

 		// Now read the complete message sent by the Master

        i = 0;
		while (i<PSP_BUFFER_SIZE)
		{
       		// Read a single byte from the serial port (USART) into the serial 
       		// input buffer.
	
			while(!vUARTIntStatus.UARTIntRxBufferEmpty)
			{
				UARTIntGetChar(&TheCharacterRead);
				USART_Rx_Buffer[i] = TheCharacterRead;	
 				i++;
			}
						
		}
		
		// Copy Rx buffer to local message buffer for processing
	
		for (i=0; i<PSP_BUFFER_SIZE; i++)
		{
			CoProcessorMessage.Message.Data[i] = USART_Rx_Buffer[i];
		}

		USART_Rx_Flag = TRUE;								// Indicates that a complete message was read and is valid
 
#endif

   		if (PSP_Rx_Flag == TRUE)
   		{
   			PSP_Rx_Flag = FALSE;
   			
#ifdef TEST   			
			printf("Slave: Mainline, PSP_Interrupt_Count = ");
			dec(PSP_Interrupt_Count);
			printf("\r\n");
			
			// Display contents of PSP Buffers 	
			
			for (i=0; i<PSP_BUFFER_SIZE; i++)
			{
				printf("Slave: Receive: i = ");
				dec(i);			
				printf("  PSP_Rx_Buffer = ");
				dec(PSP_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);
			}
			
 			if (!FlagReg.OUTRDY)				// btfsc	FLAGREG,OUTRDY	;Check if data output already
			{									// goto	Loop		;Not output yet, loop
				// Now send message

#ifdef TEST   			

				for (i=0; i<PSP_BUFFER_SIZE; i++)
				{
					PSP_Tx_Buffer[i] = i;

					printf("Slave: Transmit i = ");
					dec(i);
					printf("  PSP_Tx_Buffer = ");
					dec(PSP_Tx_Buffer[i]);
 			
					printf("\r\n");
			
					//pause(1000);
				}
#endif  

				FlagReg.OUTRDY = 1;				// bsf	FLAGREG,OUTRDY	;Set flag for interrupt routine
			}


#endif  


	//******************************************************************************
	// Process Math Co-processor Commands here
	//******************************************************************************

 
   		
    		// Change controller state to BUSY
			ControllerState = BUSY; 
				  	
       		// Process the selected command
			        	
 			switch (CoProcessorMessage.SelectedCommand.Command)
 			{
				case CALCULATE_COMMAND:
				{
 					// Process the Calculate command message that evaluates RPN expression
 					ProcessCalculateCommand();
 				
					break;
				} 
			
				case GET_ANSWERS_COMMAND:
				{
		
 					// Process Get Answers command message in order to return the floating point answers to the master
  					ProcessGetAnswers();
  					
  					break;
				}
			
				case CLEAR_MEMORY_COMMAND:
				{
  					// Process the Clear Memory Command message that erases the floating point registers
					ProcessClearMemoryCommand();
					
 					break;
				}
			
				case STORE_COMMAND:
				{

 					// Process the Store Command message that saves a floating point value to a selected float register
					ProcessStoreCommand();
					
   					break;
				}
			
				case RECALL_COMMAND:
				{

  					// Process the Recall Command message that  Retrieves a floating point value from a 
  					// selected float register.	
  					ProcessRecallCommand();
   					break;
				}
#ifdef NEW_CODE
				case FILTER_COMMAND:
				{
  					// Process the Filter command message that performs selected filter 
  					// operation (FIR, IFR)

					ProcessFilterCommand();
    				break;
				}
				
				case FFT_COMMAND:
				{
  					// Process the FFT command message that performs the selected 
  					//FFT operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case IFFT_COMMAND:
				{
  					// Process the Inverse Filter command message that performs an inverse
  					// FFT operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case KALMAN_FILTER_COMMAND:
				{
  					// Process the Kalman Filter command message that performs a Kalman
  					// Filter on the specified data

					ProcessKalmanFilterCommand();
    				break;
				}
				
				case MEDIAN_FILTER_COMMAND:
				{
  					// Process the Median Filter command message that performs a Median filter 
  					//operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case HISTOGRAM_COMMAND:
				{
  					// Process the Histogram command messagePerforms a Histogram operation

					ProcessHistogramCommand();
    				break;
				}
				
				case SORT_COMMAND:
				{
  					// Process the Sort command message that performs the selected Sort operation 
  					//on the specified data

					ProcessSortCommand();
    				break;
				}
#endif			
				case GET_STATUS_COMMAND:
				{
  					// Process the Get Status Command message

					ProcessGetStatusCommand();
					
   					break;
				}
				
				case RESET_COMMAND:
				{
  					// Process the Reset controller command message

					ProcessResetCommand1();
					
   					break;
				}

			}
			
	  		// Change controller state to DONE
			ControllerState = DONE;

		 	  
        	// Copy the entire Co-Processor Response to the PSP_Tx_Buffer for transmission to the PSP Master
        	// during the next message sent from the Master.  This 1 cycle delay allows the Math CO-processor 
        	// complete its task and provide the requested results.
          		 			
  			//memcpy(PSP_Tx_Buffer+1, &CoProcessorMessage.Message.Data, DATA_BUFFER_SIZE);  
  			
			// Copy local message buffer back to Tx buffer to send results back to Master
	
			for (i=0; i<PSP_BUFFER_SIZE; i++)
			{
				PSP_Tx_Buffer[i] = CoProcessorMessage.Message.Data[i];
			}
		
 			// Synchronize with Master 
			//SyncWithMaster();	   	 

			// Send data back to Master

	 		i=0;
			while (i<PSP_BUFFER_SIZE)
			{   
				//Check PSP status flag
				if(!vUARTIntStatus.UARTIntTxBufferFull)
				{
				 	UARTIntPutChar(PSP_Tx_Buffer[i]);				
 			 		i++;
			 	}
		 	}

#ifdef TEST  
			for (i=0; i< PSP_BUFFER_SIZE; i++)
   			{
				PSP_Tx_Buffer[i] = DATA_BUFFER_SIZE-i;
			}
			 			
			printf("Slave Co-processor application, PSP_Interrupt_Count = ");
			dec(PSP_Interrupt_Count);
			printf("\r\n");
			
			// Display contents of PSP Send and Receive Buffers 	
			
			for (i=0; i<PSP_BUFFER_SIZE; i++)
			{
				printf("Math Co-processor: i = ");
				dec(i);
				printf("  PSP_Tx_Buffer = ");
				dec(PSP_Tx_Buffer[i]);
			
				printf("  PSP_Rx_Buffer = ");
				dec(PSP_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);
			}		

       		printf("Slave: Math Co-processor received the command:.");
#endif       
			
 /*        				
			// Display the current contents of the buffer
			for (i=0; i<PSP_Number_Characters; i++)
			{
				printf("Slave: i = ");
				dec(i);
				printf("  PSP_Tx_Buffer = ");
				dec(PSP_Tx_Buffer[i]);
			
				printf("  PSP_Rx_Buffer = ");
				dec(PSP_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);

			}

*/

//*******************************************************************************
//#endif

 
 		}
 		
//#endif 		
 		
  		// Check to see if the PSP Co-Processor Slave Select is asserted (=0).
      	// Disable the PSP interrupts if it is not.
/*
      	if (!SS)
      	{
      		// Enable PSP MSSP interrupts

			EnableInterrupts();
      	}
      	else
      	{
 		 	// Disable PSP MSSP interrupts

			DisableInterrupts();
     	}
*/

		}
    
/*        
   while (1)
   {
   	// Clear the BF and SPSPF flag bits by reading the SSPBUF register

   	Data = SSPBUF;  
    
   	PIR1bits.SPSPF = 0;  
   	
   	// Wait for receive to end
      
      while (SSPSTATbits.BF);
      
      // Check if last byte received was a data byte
      
      if (SSPSTATbits.D_A)
      {
      	Data = SSPBUF;
      }
      printf("Data = ");
      dec(Data);
      printf("\r\n");
          
 		//PORTB = Data;                                // Write value to PORTB LED(s)
 	}
   
 
   while (1)
   {
 		PORTB = CurrentState;                                // Write value to PORTB LED(s)
      
      printf("Slave: PSP_Interrupt_Count = ");
      dec(PSP_Interrupt_Count);
      printf("\r\n");
 		
      printf("Slave: CurrentState = ");
      dec(CurrentState);
      printf("\r\n");
     
   }

 

   for (i=0; i<255; i++)
   {
       printf("This is a test ");
       dec(i);
       printf(" times ! \r\n");
       PORTB = i;
       pause(500);
   }
*/
    
/*   
   printf("PIC18C452 PSP Slave Controller Application... \r\n");
   
   // Test LED(s)
   
   for (i=0; i<256; i++)
   {
    	PORTB = i;
   		pause(200);
   	
      printf("Slave: i = ");
      dec(i);
      printf("\r\n");
      
   }
*/   
  
	// Done.

	while(1);

 
 
}
//******************************************************************************
//* EvaluateRPN - Evaluates an RPN Expression in order to handle more complicated
//* arithmetic expressions. It does not currently support trigonometric or scientific 
//* functions.
//******************************************************************************
void EvaluateRPN(void)
{
		char Ch;					// Local character 

 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
				
		// Get a token character (operator or numeric) from the message buffer 
 
   		while ((NumChars <  MESSAGE_BUFFER_SIZE) && (NumChars < strlen(CoProcessorMessage.CalculateCommand.Buffer)))
   		{
 
	 
			Ch = CoProcessorMessage.CalculateCommand.Buffer[NumChars++];					// Get next token from message buffer  
 
			// Check for end of statement semicolon character, ";"

 			if (Ch == ';')
			{
				// Save last answer and continue parsing the buffer until all characters in it have been processed.
                // If answers exceed the maximum number then wrap the index to 0 using the mod function.

				Answers[AnswerIndex++]= LastX;
 				AnswerIndex = AnswerIndex % MAX_ANSWERS;
				
			}

			if (isdigit(Ch) || (Ch == '.'))
			{
				// Save the key in the input buffer

				Buffer[BufferCount++] = Ch;
 			}
			else
			{
												

				if (BufferCount > 0)
				{
					// Automatically lift the stack by one unless lift has been disabled for the current operation
 				
 					Lift();
 					
					// This logic needs to be modified or moved ?????
				
					Buffer[BufferCount++] = 0;			// Terminate the string 
				
					// Convert the value stored in the buffer from ascii to float and save it in register X
 
					AsciiToDouble(&X, Buffer);
				
					// Copy number from the display register X to the Stack register X 
				
 					Stack[X_LOCATION] = X;
 				
					LastX = X;							// Save value of X into LAST_X register
					BufferCount = 0;					// Buffer count
				}	
 										
				// Interpret the HP-45 RPN command here
				
				switch (Ch)
				{
					// Stack operations

					case ENTER:						// "Enter ^" key 
					{
						// Lift stack up one level and disable automatic lift for the next operation

						Lift();
						DisableLift = TRUE;
						break;
					}

					case ROLL:						// "R !" key 
					{
						// Rotate the values in the stack while preserving all values.    

						Roll();
						break;
					}

					case SWAP:						// "X <> Y" key 
					{
						// Swap X with Y

						Swap();
						break;
					}

					case STORE:						// "STO" key 
					{
						// Store X into register N, where N=1..9

						RegisterNumber = CoProcessorMessage.CalculateCommand.Buffer[NumChars++];
						Registers[RegisterNumber] = X;
						break;
					}

					case RECALL:					// "RCL" key 
					{
						// Recall register N, where N=1..9 into X

						RegisterNumber = CoProcessorMessage.CalculateCommand.Buffer[NumChars++];
						X = Registers[RegisterNumber];
						break;
					}

					case LASTX:					// "LAST X" key 
					{
						// Recall Last X register into X

						X = LastX;
 						Stack[X_LOCATION] = X;
 						
						break;
					}

					// Basic math operations

					case PLUS:						// "+" key
					{
						// Compute the "+" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],PLUS, Stack[X_LOCATION]);
			
						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case MINUS:						// "-" key
					{
						// Compute the "-" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], MINUS, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case TIMES:						// "*" key
					{
						// Compute the "*" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], TIMES, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case DIVIDE:					// "/" key
					{
						// Compute the "/" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],DIVIDE, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}					

					// Trigonometric functions

					case SIN:					// sin(x) function  
					{
						// Compute the sin(x) and place result in the X register

						Stack[X_LOCATION] = sin(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 					
					case ASIN:					// asin(x) function  
					{
						// Compute the asin(x) and place result in the X register

						Stack[X_LOCATION] = asin(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case COS:					// cos(x) function  
					{
						// Compute the cos(x) and place result in the X register

						Stack[X_LOCATION] = cos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case ACOS:					// acos(x) function  
					{
						// Compute the acos(x) and place result in the X register

						Stack[X_LOCATION] = acos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case TAN:					// tan(x) function  
					{
						// Compute the cos(x) and place result in the X register

						Stack[X_LOCATION] = cos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case ATAN:					// atan(x) function  
					{
						// Compute the atan(x) and place result in the X register

						Stack[X_LOCATION] = atan(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					// Scientific functions ...

					case SQRT:					// sqrt(x) function  
					{
						// Compute the sqrt(x) and place result in the X register

						Stack[X_LOCATION] = sqrt(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case SQR:					// sqr(x) function  
					{
						// Compute the sqrt(x) and place result in the X register

						Stack[X_LOCATION] = sqr(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
#ifdef NEW_CODE
					case EXP:					// sqr(x) function  
					{
						// Compute the exp(x) and place result in the X register

						Stack[X_LOCATION] = exp(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case LOG:					// log(x) function  
					{
						// Compute the log(x) and place result in the X register

						Stack[X_LOCATION] = log(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}					

					case log10:					// log10(x) function  
					{
						// Compute the log10(x) and place result in the X register

						Stack[X_LOCATION] = log10(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case POWER:					// power(x,n) function  
					{
						// Compute the power(x,n) and place result in the X register

						Stack[X_LOCATION] = power(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					} 					
#endif

					case MAX:					// max(x,y) function  
					{
						// Compute the max(x,y) and place result in the X register

						Stack[X_LOCATION] = max(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case MIN:					// min(x,y) function  
					{
						// Compute the min(x,y) and place result in the X register

						Stack[X_LOCATION] = min(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case ABS:					// abs(x) function  
					{
						// Compute the abs(x) and place result in the X register

						Stack[X_LOCATION] = abs(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case SIGN:					// sign(x,y) function  
					{
						// Compute the sign(x,y) and place result in the X register

						Stack[X_LOCATION] = sign(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case MOD:					// mod(x,y) function  identical to the C mod  function
					{
						// Compute the mod(x,y) and place result in the X register

						Stack[X_LOCATION] = mod(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
					


					// Special math coprocessor functions

 					case RAD:					// rad(x) function  
					{
						// Convert degrees to radians and place result in the X register

						Stack[X_LOCATION] = radians(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case DEG:					// deg(x) function  
					{
						// convert radians to degrees and place result in the X register

						Stack[X_LOCATION] = degrees(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
 
#ifdef NEW_CODE

  					case FIX:					// fix(x) - function rounds x towards 0 
					{
						// Rounds x towards 0 and places result in the X register

						Stack[X_LOCATION] = fix(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

   					case FLOOR:					// floor(x) - Rounds x towards - Infinity 
					{
						// Rounds x towards - Infinity and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

    				case FLOOR:					// floor(x) - Rounds x towards + Infinity 
					{
						// Rounds x towards + Infinity and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
     				case ROUND:					// round(x) - Round x to nearest integer
					{
						// Rounds x to nearest integer and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
     				case PRECISION:					// precision(x) - Specify maximum number of digits precision up to machine precision used for display purposes
					{
						// Specify maximum number of digits precision up to machine precision used for display purposes

						Stack[X_LOCATION] = precison(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

#define PRE 	'X'					// 

#endif

				}
							
							
				// Copy number from the Stack register X to the display register X and to the Last X register  
				
 				X = Stack[X_LOCATION];
				LastX = X;
				
				// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

				DoubleToAscii(X, af1);

			}
		
 
		}
 		

	
 
}

//******************************************************************************
//* Calculator - Simple calculator that performs double precision floating point
//* addition, subtraction, multiplication and division.  It does not currently
//* support levels of nesting parentheses for complicated mathematical expressions.
//******************************************************************************

double Calculator(double Argument1, char Operator, double Argument2)
{
	double TheResult;
 
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	switch(Operator)
	{
		case '+':
		{
			TheResult = Argument1 + Argument2;
			break;
		}

		case '-':
		{
			TheResult = Argument1 - Argument2;
			break;
		}

		case '*':
		{
			TheResult = Argument1 * Argument2;
			break;
		}

		case '/':
		{
			TheResult = Argument1 / Argument2;
			break;
		}

		default:
		{
			TheResult = 0;
			break;
		}		

	}

	return (TheResult);

}



// HP-45 Functions

//******************************************************************************
//* Lift - Pushes the value in register X onto the simulated HP-45 Stack.  The 
//* contents of register T, at the top of the stack is lost, being replaced by
//* the contents register Z. The next number keyed into X overwrites the old X.
//******************************************************************************
void Lift(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
	// Check to make sure that the Lift operation has not been disabled 
	
	if (DisableLift)
	{
		DisableLift = FALSE;								// Enable Lift for the next operation
	}
	else
	{
 		Stack[T_LOCATION] = Stack[Z_LOCATION]; 				// Contents of T are lost
 		Stack[Z_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register Z   
 		Stack[Y_LOCATION] = Stack[X_LOCATION]; 				// Copy contents of register X to register Y  
 		//Stack[Y_LOCATION] = X; 							// Copy contents of register X to register Y
 	}  
}

//******************************************************************************
//* Drop - The rest of the stack is dropped to fill the gap left by the 
//* previous operation.  The value of the Y register is lost. 
//******************************************************************************
void Drop(void)
{
	int i;												// Local loop variable

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	//Stack[X_LOCATION] = Stack[Y_LOCATION]; 			// Contents of X are lost
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Y to register Z   
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register X to register Y  
	//X = Stack[X_LOCATION]; 							// Copy contents of register X to X  

 		
 		
	// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

	DoubleToAscii(Stack[X_LOCATION], af1);
		
	// Send result to Serial Port
	//printf(" = ");
	//puts(af1);

/*	
	// Display the contents of the Stack
		
	printf("\r\nStack...\r\n");
	for (i=3; i>=0; i--)
	{
 		putch(StackLabel[i]);
 		printf(": ");
		DoubleToAscii(Stack[i], af1);
		string(af1);
	}
	printf("\r\n> ");					// Calculator prompt
*/

#ifdef SERIAL_DRIVER
	// Hit any  key to continue
	//Ch = getchar();                   // Serial Input  
	UARTIntGetChar(&Ch);
 
#endif
				
#ifdef KEYPAD_DRIVER
	// Hit any  key to continue
	Ch = GetKey();                      // Keypad Input    
#endif
	
 	//lcd_clear();						// clear LCD, home cursor
 	//printf("> ");					// Calculator prompt
	
	
}

//******************************************************************************
//* Roll - 	 Rotate the values in the stack while preserving them.    
//******************************************************************************
void Roll(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Z to register Y
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register T to register Z    
 	Stack[T_LOCATION] = X; 								// Copy contents of X to register T  
    X = Stack[X_LOCATION];								// Copy contents of register X to X 
 
}

//******************************************************************************
//* Swap - Swaps the contents of register X with the contents of register Y.
//******************************************************************************
void Swap(void)
{
	double TempValue;									// Temporary storage for swap
	
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	TempValue = Stack[X_LOCATION];						// Save contents of register X
	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X   
  	Stack[Y_LOCATION] = TempValue; 						// Copy contents of TempValue to register Y     	
	X = Stack[X_LOCATION]; 								// Copy contents of register X to X  
 
}
//********************************************************************************
//* ConfigureInterrupts - Configure interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void ConfigureInterrupts(void)
{
     // Configure PSP interrupts (1 = High Priority, 0 = Low Priority).

    IPR1bits.RCIP = 1;

    // Enable PSP asynchronous interrupt

    PIE1bits.RCIE = 1; 

  // Enable peripheral interrupts
 
   INTCONbits.PEIE = 1;                                    

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts	   
}




//********************************************************************************
//********************************************************************************
//********************************************************************************
//********************************************************************************
 

//********************************************************************************
//* InitializeMathCoprocessor - Initializes Timers, program variables and peripheral registers
//* for the math co-processor.
//********************************************************************************
void            InitializeMathCoprocessor(void)
{
 	int i; 									// Local loop index
 
  	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		
     
    // Initialize PSP device address pointer

    //AddressPointer = 0;
    PSP_Interrupt_Count = 0;                      // Number of MSSP interrupts                           

    // Initialize Rx, Tx and local message buffer to zero  
 
    for (i=0; i< PSP_BUFFER_SIZE; i++)
    {

		PSP_Rx_Buffer[i] = 0;
		PSP_Tx_Buffer[i] = 0;
 		CoProcessorMessage.Message.Data[i] = 0;
   } 

     // configure port A for all digital inputs

    OpenADC(ADC_0ANA, ADC_INT_OFF);
    
    // Initialize ports  

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                        // Clear PORTC (Some Port C bits are reserved for PSP)
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction for output

 
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
    //TRISC = 0x00;                                        // PORTC all outputs (Some Port C bits are reserved for PSP).
    TRISD = 0x00;                                          // PORTD all outputs.
    TRISE = 0x00;                                          // PORTE all outputs.

  


 

/*
   // Clear the BF and SPSPF flag bits by reading the SSPBUF register

    i = SSPBUF;  
    
    PIR1bits.SUSARTF = 0;    
*/
 
	PSP_Interrupt_Count = 0;                      // Number of MSSP interrupts
	Index = 0;                               // PSP Ring Buffer index
	PSP_Rx_Flag =  FALSE;
	NumChars = 0;

	CurrentState = 0;                        // Current PSP state variable
 
	// Configure interrupts for serial I/O and other peripherals that need them
	
	ConfigureInterrupts();
 
      // Wait 5 seconds 
/*      
      printf("5 Second Test ...\r\n");
      pause(5000);
      printf("End of 5 Second Test\n");
*/

	// Initialize Serial LCD Display here
	
	//printf("\r8-N-1");  
	 
	//printf("DIY Calculator \r\n");	
	
	// Test the Ascii to double conversion function
/*	
	af1[0] = '9';
	af1[1] = '8';
	af1[2] = '7';
	af1[3] = '.';
	af1[4] = '6';
	af1[5] = '5';
	af1[6] = '4';
	af1[7] = 0;
	
	AsciiToDouble(&X, af1);
	

   Enter();
*/	
/* 
	Buffer[0] = '0';
	Buffer[1] = '0';
	Buffer[2] = '0';
	Buffer[3] = '0';
	Buffer[4] = '0';
	Buffer[5] = '0';
	Buffer[6] = '0';
	Buffer[7] = 0;

	Buffer[0] = '3';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
	
	Buffer[0] = '4';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
*/	

	// Initialize stack 
	
   Stack[X_LOCATION] = 0.0;
   Stack[Y_LOCATION] = 0.0;
   Stack[Z_LOCATION] = 0.0;
   Stack[T_LOCATION] = 0.0;


	// Test stack operations 
	
   Stack[X_LOCATION] = 1.0;
   Stack[Y_LOCATION] = 2.0;
   Stack[Z_LOCATION] = 3.0;
   Stack[T_LOCATION] = 4.0;
    
	// Make sure we can perform a stack lift initially
	
    DisableLift = FALSE;
    AnswerIndex = 0;
   	MessageBufferIndex = 0;
   	   
	// Initialize Math Co-processor Controller State, Error codes and Diagnostic codes
	
	ControllerState = INITIALIZED; 	  
   	ControllerErrorCode =  NO_ERRORS;	   
  	ControllerDiagnosticCode = NO_DIAGNOSTICS;  
}

//****************************************************************************************
//****************************************************************************************
//****************************************************************************************
//****************************************************************************************


//**********************************************************************************
//* ProcessCalculateCommand - Evalueates the RPN expressions that are stored in    *
//* the message expression buffer.  Multiple expressions can be sent to the        *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.  Large *
//* expressions can span multiple messages to the co-processor.                    *
//**********************************************************************************
byte ProcessCalculateCommand(void)  
{
 	byte CalculateCommandStatus = TRUE;							// Status return
 
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		 	
 	// Parse and Evalueate the RPN expression contained in the co-processor message buffer ...
 	
 	
 	
#ifdef TEST
    printf("slave: Calculate Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.CalculateCommand.MessageID);
   	printf("\r\n");
    printf("Buffer = !");
    string(CoProcessorMessage.CalculateCommand.Buffer);
    printf("!");
   	printf("\r\n\r\n");
#endif

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

 	// Evaluate the RPN  expression and store the answers in the Answer buffer
 	EvaluateRPN();
				
	return CalculateCommandStatus;
}

//**********************************************************************************
//* ProcessGetAnswers - Get the results from the latest arithmetic operations      *
//* carried out during the last cycle. Up to 7 separate single precision floating  *
//* point or 3 double precision  values may be returned during a single message    *
//* transfer.                                                                      *
//**********************************************************************************
byte ProcessGetAnswers(void)
{
 	byte GetAnswersStatus = TRUE;								// Status return
   	byte AnswerID;									 		    // Local Answer Loop Index 
 		
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		    				
  	// Send  latest answers from requested RPN expressions back to the master.
	CoProcessorMessage.GetAnswersCommand.NumberOfAnswers = AnswerIndex;

#ifdef TEST

	// Send test answer responses back to master (Value of PI a digit at a time)
	 
    printf("slave: Get Answers Command: \r\n");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetAnswersCommand.MessageID);
   	
	CoProcessorMessage.GetAnswersCommand.NumberOfAnswers = 7;
	Answers[0] = 3.0;
	Answers[1] = 1.0;
	Answers[2] = 4.0;
	Answers[3] = 1.0;
	Answers[4] = 5.0;
	Answers[5] = 9.0;
	Answers[6] = 2.0;
	
/*	
	CoProcessorMessage.GetAnswersCommand.Answers[0] = 3.0;
	CoProcessorMessage.GetAnswersCommand.Answers[1] = 1.0;
	CoProcessorMessage.GetAnswersCommand.Answers[2] = 4.0;
	CoProcessorMessage.GetAnswersCommand.Answers[3] = 1.0;
	CoProcessorMessage.GetAnswersCommand.Answers[4] = 5.0;
	CoProcessorMessage.GetAnswersCommand.Answers[5] = 9.0;
	CoProcessorMessage.GetAnswersCommand.Answers[6] = 2.0;
*/	
#endif
	
 	//for (AnswerID = 0; AnswerID < CoProcessorMessage.GetAnswersCommand.NumberOfAnswers; AnswerID++)
 	for (AnswerID = 0; AnswerID < 7; AnswerID++)
	{	
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
		// Copy local answers to Answer message
		CoProcessorMessage.GetAnswersCommand.Answers[AnswerID] = Answers[AnswerID];

#ifdef TEST
   		printf("  Answer ID = ");
    	dec(AnswerID);
    	printf(", Answer = ");
    	//hex4(CoProcessorMessage.GetAnswersCommand.Answers[AnswerID]);  
    	hex4(Answers[AnswerID]);  
    	printf("\r\n");
#endif
 
	}
 
	return GetAnswersStatus;
}
			
//**********************************************************************************
//* ProcessClearMemoryCommand - Erases the contents of the Math co-processor's 
//* floating point storage registers. These same registers are used for storing and
//* retrieving temporary values and answers. It does this by setting floating point 
//* registers 0..MAX_FLOAT_REGISTERS to zero.  Registers 0..10 are reserved for answers.
//**********************************************************************************
byte ProcessClearMemoryCommand(void)
{
 	byte ClearMemoryCommandStatus = TRUE;							// Status return
    byte RegisterID;									 		    // Local Loop Index
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	// Read the Pot control duty value and send it to the corresponding DC motor 
 
 	for (RegisterID = 0; RegisterID < MAX_FLOAT_REGISTERS; RegisterID++)
	{
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		Registers[RegisterID] = 0.0;
	}
	
#ifdef TEST
    printf("save: Clear Memory Command Completed.");
   	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   printf("\r\n\r\n");
#endif

	return ClearMemoryCommandStatus;
}

//**********************************************************************************
//* ProcessStoreCommand - Stores a floating point value to a selected floating     *
//* point register.     The register ID range is 0..MAX_FLOAT_REGISTERS.           *
//**********************************************************************************
byte ProcessStoreCommand(void)
{
 	byte StoreCommandStatus = TRUE;										// Status return

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
			
 	// Store the value into the selected register.
  					
 	Registers[CoProcessorMessage.StoreCommand.RegisterID] = CoProcessorMessage.StoreCommand.RegisterValue;
  	 
#ifdef TEST
    printf("slave: Store Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   	printf("  Register ID = ");
   	dec(CoProcessorMessage.StoreCommand.RegisterID);
    printf("Register Value = ");
    hex4(Registers[CoProcessorMessage.StoreCommand.RegisterID]);
    printf("\r\n\r\n");
#endif 

	return StoreCommandStatus;
}
  					
//**********************************************************************************
//* ProcessRecallCommand - Retrieves a floating point value from the selected      *
//* floating point register. The register ID range is 0..MAX_FLOAT_REGISTERS.      *                                                               
//**********************************************************************************
byte ProcessRecallCommand(void)
{
 	byte RecallCommandStatus = TRUE;									// Status return

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
 	
    // Accept number of samples xxxx to be used for calibration and data
   	// collection used for reading the DC Motor Control Pots, where the 
   	// number of samples range 0000..FFFF.
	
  	CoProcessorMessage.RecallCommand.RegisterValue = Registers[CoProcessorMessage.RecallCommand.RegisterID];
  	
#ifdef TEST
    printf("slave: Recall Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.RecallCommand.MessageID);
   	printf("  Register ID = ");
   	dec(CoProcessorMessage.RecallCommand.RegisterID);
    printf("Register Value = ");
    hex4(CoProcessorMessage.RecallCommand.RegisterValue);
    printf("\r\n\r\n");
#endif 

	return RecallCommandStatus;
}


//**********************************************************************************
//* ProcessGetStatusCommand - Process the the PSP Slave Controller Get Status 
//* command and return the latest command execution status. 
//**********************************************************************************
void ProcessGetStatusCommand(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
		
    CoProcessorMessage.GetStatusCommand.ControllerState = ControllerState; 		 			// Store current co-processor State
    CoProcessorMessage.GetStatusCommand.ControllerErrorCode = ControllerErrorCode; 			// Store current co-processor Error Code
    CoProcessorMessage.GetStatusCommand.ControllerDiagnosticCode = ControllerDiagnosticCode;  // Store current co-processor Diagnostic Code

 
 #ifdef TEST
    CoProcessorMessage.GetStatusCommand.ControllerState = 3; 		 			// Store current co-processor State
    CoProcessorMessage.GetStatusCommand.ControllerErrorCode = 2; 			// Store current co-processor Error Code
    CoProcessorMessage.GetStatusCommand.ControllerDiagnosticCode = 1;  // Store current co-processor Diagnostic Code
     
    printf("Slave: Received Get Status Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   	printf("  ControllerState = ");
   	dec(ControllerState);
   	printf("  ControllerErrorCode = ");
   	dec(ControllerErrorCode);
   	printf("  ControllerDiagnosticCode = ");
   	dec(ControllerDiagnosticCode);
    printf("\r\n");
#endif

}
 			

//**********************************************************************************
//* ProcessResetCommand - Process the Reset the PSP Slave Math Co-processor.  This command 
//* resets the selected controller.  It is used to force a hard reset of the math
//* co-procesor when a fatal error is encountered, using the PIC's watchdog timer or 
//* via the "Z" command over the serial interface.
//**********************************************************************************
void ProcessResetCommand1(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
#ifdef TEST
    printf("Reset Command: ");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.ResetCommand.MessageID);
    printf("\r\n\r\n");
#endif

	// Reset the Controller by allowing the Watchdog Timer to timeout.
	
	while (1);
}
 			
//******************************************************************************
//* InitializeParallelPort - Initializes the 8-Bit Parallel Slave Port.
//******************************************************************************
void InitializeParallelPort(void)
{
	int i;
	OutData = 0;					// clrf	OUTDATA		;Clear data registers
	InData = 0;						// clrf	INDATA
									// bsf	STATUS,RP0	;Select register page 1
 


	// Set Port_B to all outputs
    PORTB = 0;
 	TRISB = 0x00;					// movwf	TRISB		;Set Port_B to all outputs
 
	BUSY_DIR = OUTPUT;				   						// Set Slave BUSY Status Pin Direction  to output

    BUSY = 1;												// Let the Master know the Slave is busy initializing


	// Configure Port D and an 8 Bit Slave Parallel Port 

	//TRISEbits.PSPMODE = 1;
	PSPCONbits.PSPMODE = 1;

	// Configure PORT E for 8 Bit Slave Parallel Port
/*
	TRISE = 	0x17;				// movlw	b'00010111'	;Set RD, WR, and CS as
 	 								// movwf	TRISE	    ; inputs, Enable Parallel Slave port
 									// movlw	0FFh
*/


 	RD_BAR_DIR	= INPUT;       // RD_BAR Control Pin Direction
 	WR_BAR_DIR 	= INPUT;       // WR_BAR Control Pin Direction 
 	CS_BAR_DIR 	= INPUT;       // CS_BAR Control Pin Direction 

 
  	// Configure the ADCON1 register for digital I/O (Port D and E) using the 
	// PCFG3:PCFG0: A/D Port Configuration Control bits TO 0x0F:

	ADCON1bits.PCFG3 =  1;
	ADCON1bits.PCFG2 =  1;
	ADCON1bits.PCFG1 =  1;
	ADCON1bits.PCFG0 =  1;


	PIE1bits.PSPIE = 1;        		// movlw	b'10000000'	;
									// movwf	PIE1		;Enable Parallel Slave Port interrupt
									// bcf	STATUS,RP0	;Select register page 0

	PORTD = OutData;				// movf	OUTDATA,W	;Set output Data in PORTD
									// movwf	PORTD

	// Initialize Message buffers

 
	// Initialization Logic for PSP Slave Buffer Interface goes here ...
	for (i=0; i<PSP_BUFFER_SIZE; i++)
	{
 		PSP_Rx_Buffer[i] = 0;                  	// Clear PSP Receive Buffer 
 		//PSP_Tx_Buffer[i] = 0;               	// Clear PSP Transmit Buffer   
 		PSP_Tx_Buffer[i] = i;                  	// Initialize PSP Transmit Buffer  to a ramp 0..PSP_BUFFER_SIZE 
	}
	
 
 	
 	
	for (i=0; i<PSP_BUFFER_SIZE; i++)
	{
 		CoProcessorMessage.Message.Data[i] = 0;            			// Clear the message  
 	}
   
 	
	INTCONbits.GIE  = 1;			// movlw	b'11000000'	;Set GIE, PEIE (enable interrupts)
	INTCONbits.PEIE = 1;			// movwf	INTCON
}



