//********************************************************************************
//*         Name:  coprocs.h                                                     *               
//*                                                                              *
//*       Author:  Daniel F. Ramirez                                             *
//*                                                                              *
//*         Date:  06/13/2003                                                    *
//*                                                                              *
//*   Description:                                                               *
//*                                                                              *
//*                                                                              *
//*     This is a Slave Math Co-Processor application that uses a PIC18F8720 to  * 
//* send expressions to the embedded Math Co-Processor using the SPI interface,  *
//* in order to compute floating point arithmetic expressions in RPN format.  It *
//* can be used for Sumo Robots that require floating point for navigation and   *
//* tracking of an oponent with minimal space, power and weight requirements are *
//* prime candidates for using this co-processor, since it is very compact.      *
//* The master can be a PIC or STAMP or any other micro-controller that requires *
//* a math co-processor and has at least 4 spare I/O pins. In addition to the    *
//* basic arithmetic functions, the math co-processor can also evaluate          *
//* expressions that contain trigonometric and scientific functions.             *
//*                                                                              *
//*      A communication interface is provided that allows the host to use high  *
//* speed asynchronous serial link or SPI or I2C Master to request floating point*
//* equation calculations or to pass it individual sensor readings, average      *
//* sensor readings, calibrating a sensor or to perform a sensor processing      *
//* algorithm using command message packets.  The commands available are         *
//* described below.  This header file provides the common Master and Slave      *
//* message formats and descriptors.                                             *
//*                                                                              *
//*                                                                              *
//* The high priority ISR will invoke the "Z;" reset command via the serial      *
//* interface. Both the serial interface and the SPI interface are supported by  *
//* defining the approprate compiler flag.                                       *
//*                                                                              *
//*                                                                              *
//*     This slave controller can be used to communicate with a PC host or any   *
//* USART, I2C or SPI master, in order to  give the microcontroller the          *
//* capability to serve as an SPI or I2C slave controller in order to provide it * 
//* the floating point coprocessor functions.                                    *  
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each I2C device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application will include implementing various    *
//* filters including FIR, FFR, Kalman and Median filters.  Both FFT and IFFT    *
//* functions and a Histogram function are also in the works. Advanced algorithms*
//* such as Vector Force Fields (VFF), Fuzzy Logic and Neural Networks are new   *
//* areas that may be explored using this application.   In addition, this       *
//* application may be able to take advantage of the new DSPic Signal Processors *
//* (DSP's) become available from Microchip.                                     *
//*                                                                              *
//* Usage:                                                                       *
//*	The following Math Co-Processor Controller command messages are available:   *
//*                                                                              * 
//* calculate    - Evalueates the RPN expressions that are stored in the         *
//* message expression buffer.  Multiple expressions can be sent to the          *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.      *
//* Large expressions can span multiple messages to the co-processor.            *
//*                                                                              * 
//* get_answers  - Get the results from the latest arithmetic operations         *
//* carried out during the last cycle. Up to 7 separate single precision floating*
//* point and 3 double precision  values may be returned during a single message *
//* transfer.                                                                    *
//*                                                                              *
//* clear_memory - Erases the contents of the Math co-processor's storage        * 
//* registers. These same registers are used for storing and retrieving temporary*
//* values. It clears floating point registers 0..MAX_FLOAT_REGISTERS.           *
//*                                                                              *
//* store   - Stores a floating point value to a selected floating point         *
//* register. The register ID range is 0..MAX_FLOAT_REGISTERS.                   *                                                            
//*                                                                              *
//* recall 	- Retrieves a floating point value from the selected floating        *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.             *                                                         
//*                                                                              *                                                                     
//*	GetStatus - Get the Math Co-processor status including current state, and    *  
//* any error conditions encountered i.e. division by 0, overflow, underflow.    *
//*                                                                              *
//* ResetCommand - Reset the DC motor Controller (brake the motor).  This        *
//* command can be used to implement an emergency stop function.                 *  
//*                                                                              *
//*                                                                              * 
//*                                                                              *
//********************************************************************************
    
 
#ifndef COPROCS_LOADED
#define COPROCS_LOADED

// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
//#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
//#include <p18c452.h>					// Register definitions for PIC18F8720
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
 
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <spi.h>													  // I2C library functions
#include <pwm.h>                                        // PWM library functions
#include <adc.h>                                        // ADC library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
#include "network.h"                                    // Include SPI Network Support
#include "dcmotor.h"												  // Include Common DC Motor Controller header
#include "timeutil.h"                                   // Timer utility functions
#include "stdio.h"                                      // I/O support functions
#include "sensors.h"												  // Include Sensor utilities


//********************************************************************************
// Function Prototypes
//********************************************************************************

// Math Co-Processor Message handling Functions

byte ProcessCalculateCommand(void);  
byte ProcessGetAnswers(void);
byte ProcessClearMemoryCommand(void); 
byte ProcessStoreCommand(void);
byte ProcessRecallCommand(void);
void ProcessResetCommand1(void);
void ProcessGetStatusCommand(void);

// Math Co-Processor Control Functions

 
#endif


