//********************************************************************************
//*         Name:  coprocs_spi.c                                                 *               
//*                                                                              *
//*       Author:  Daniel F. Ramirez                                             *
//*                                                                              *
//*          Date:  06/13/2003                                                   *
//*                                                                              *
//*   Description:                                                               *
//*                                                                              *
//*                                                                              *
//*     This is a Math Co-Processor application that utilizes a PIC18F8720 to    * 
//* send expressions to the embedded Math Co-Processor using the SPI interface,  *
//* in order to compute floating point arithmetic expressions in RPN format.  It *
//* can be used for Sumo Robots that require floating point for navigation and   *
//* tracking of an oponent with minimal space, power and weight requirements are *
//* prime candidates for using this co-processor, since it is very compact.      *
//* The master can be a PIC or STAMP or any other micro-controller that requires *
//* a math co-processor and has at least 4 spare I/O pins. In addition to the    *
//* basic arithmetic functions, the math co-processor can also evaluate          *
//* expressions that contain trigonometric and scientific functions.             *
//*                                                                              *
//*      A communication interface is provided that allows the host to a use high*
//* speed asynchronous serial link or SPI or I2C Master to request floating point*
//* equation calculations or to pass it individual sensor readings, average      *
//* sensor readings, calibrating a sensor or to perform a sensor processing      *
//* algorithm using command message packets.  The commands available are         *
//* described below.  This header file provides the common Master and Slave      *
//* message formats and descriptors.                                             *
//*                                                                              *
//*                                                                              *
//* The high priority ISR will invoke the "Z;" reset command via the serial      *
//* interface. Both the serial interface and the SPI interface are supported by  *
//* defining the approprate compiler flag.                                       *
//*                                                                              *
//*                                                                              *
//*     This slave controller can be used to communicate with a PC host or any   *
//* SPI, I2C or SPI master, in order to  give the microcontroller the            *
//* capability to serve as an SPI or I2C slave controller in order to provide it * 
//* the floating point coprocessor functions.                                    *  
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each I2C device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application will include implementing various    *
//* filters including FIR, FFR, Kalman and Median filters.  Both FFT and IFFT    *
//* functions and a Histogram function are also in the works. Advanced algorithms*
//* such as Vector Force Fields (VFF), Fuzzy Logic and Neural Networks are new   *
//* areas that may be explored using this application.   In addition, this       *
//* application may be able to take advantage of the new DSPIc Signal Processors *
//* (DSP's) become available from Microchip.                                     *
//*                                                                              *
//* Usage:                                                                       *
//*	The following Math Co-Processor Controller command messages are available:   *
//*                                                                              * 
//* calculate    - Evalueates the RPN expressions that are stored in the         *
//* message expression buffer.  Multiple expressions can be sent to the          *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.      *
//* Large expressions can span multiple messages to the co-processor.            *
//*                                                                              * 
//* get_answers  - Get the results from the latest arithmetic operations         *
//* carried out during the last cycle. Up to 7 separate single precision floating*
//* point and 3 double precision  values may be returned during a single message *
//* transfer.                                                                    *
//*                                                                              *
//* clear_memory - Erases the contents of the Math co-processor's storage        * 
//* registers. These same registers are used for storing and retrieving temporary*
//* values. It clears floating point registers 0..MAX_FLOAT_REGISTERS.           *
//*                                                                              *
//* store   - Stores a floating point value to a selected floating point         *
//* register. The register ID range is 0..MAX_FLOAT_REGISTERS.                   *                                                            
//*                                                                              *
//* recall 	- Retrieves a floating point value from the selected floating        *
//* point register. The register ID range is 0..MAX_FLOAT_REGISTERS.             *                                                         
//*                                                                              *                                                                     
//*	GetStatus - Get the Math Co-processor status including current state, and    *  
//* any error conditions encountered i.e. division by 0, overflow, underflow.    *
//*                                                                              *
//* ResetCommand - Reset the DC motor Controller (brake the motor).  This        *
//* command can be used to implement an emergency stop function.                 *  
//*                                                                              *
//*                                                                              * 
//*                                                                              *
//********************************************************************************
    
 
// Initialize Calculator compilation flags here

//#define SINGLE_PRECISION                               // Include Single Precision code

//#define TEST                                           // Test code

// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
//#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
//#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#ifdef PIC18F452     
//#include <p18f452.h>					// Register definitions for PIC18F452
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>					// Register definitions for PIC18F8720
#endif
 
#ifdef PIC18F6620     
//#include <p18f6620.h>
#endif
 
#include <ctype.h>

                        
#ifdef SERIAL_DRIVER
#include "stdio.h"                                      // Header file for stdio file
#endif

#include <spi.h>                                        // SPI library functions
#include "delays.h"
#include "network.h"                                    // Include SPI Network Support
//#include "spislave.h"                                   // Include SPI Slave Services
#include "coproc.h"										// Include Common Math Co-processor header
#include "coprocs.h"									// Include Math Co-processor Services header
#include "math.h"
#include "constant.h"									// Jmath constants 
#include "jmath.h"										// Jmath headers

// Constants

#define TRUE 1
#define FALSE 0


// Port A

//#define SS			PORTAbits.RA5						// SPI Slave Co-processor Select
//#define SS_DIR		TRISAbits.TRISA5					// SPI Slave Co-processor Select Direction

// Port B

//#define TRACE0		PORTBbits.RB0						// Trace bit 0 for logic analyzer
//#define TRACE0_DIR	TRISBbits.TRISB0
#define BUSY_FLAG		PORTBbits.RB0				   	    // SPI Slave Co-processor busy flag
#define BUSY_FLAG_DIR	TRISBbits.TRISB0			   	    // SPI Slave Co-Processor busy flag Direction

// Port C

#define SPI_SCL_DIR TRISCbits.TRISC3 					// SPI Clock Direction (Slave)
#define SPI_SI_DIR TRISCbits.TRISC4 					// SPI Input Direction (Slave)
#define SPI_SO_DIR TRISCbits.TRISC5 					// SPI Output Direction (Slave)

// PORT D definitions

#define LeftEmitter PORTDbits.RD0							  // Left IR emitter (used for strobe)
#define RightEmitter PORTDbits.RD1							  // Right IR emitter (used for strobe)

// Port E

//#define BUSY_FLAG		PORTEbits.RE2				   	    // SPI Slave Co-processor busy flag
//#define BUSY_FLAG_DIR	TRISEbits.TRISE2			   	    // SPI Slave Co-Processor busy flag Direction

//#define SPI_CS		PORTEbits.RE2				   	    // SPI Slave Co-processor Select
//#define SPI_CS_DIR	TRISEbits.TRISE2			   	    // SPI Slave Co-Processor Select Direction

// Port F

#define SPI_CS		PORTFbits.RF7				   	    // SPI Slave Co-processor Slave Select
#define SPI_CS_DIR	TRISFbits.TRISF7			   	    // SPI Slave Co-Processor Slave Select Direction

#define SS			PORTFbits.RF7						// SPI Slave Co-processor Slave Select 
#define SS_DIR		TRISFbits.TRISF7					// SPI Slave Co-processor Select Select Direction 

 
// Declare global variables here

#define BufferSize		20			// Floating point buffer size
#define StackSize		 5			// HP-45 stack (5 levels)
#define RegisterSize	10			// HP-45 Registers (1-9)

#define X_LOCATION 0				// X Location in stack
#define Y_LOCATION 1				// Y Location in stack
#define Z_LOCATION 2				// Z Location in stack
#define T_LOCATION 3				// T Location in stack (top of stack)

//************************************************************************************************

 
// General Math Coprocessor functions

#define GOLD    'a'					// Gold function key    
#define ENTER	'^'                 // Enter key  
#define ROLL    '|'					// Roll key   
#define SWAP    '~'					// Swap X with Y key  
#define STORE   '>'					// Store X into register N, N=1..9   
#define RECALL  '<'					// Recall register N, N=1..9 into X  
#define LASTX   '='                 // Recall register Last X into X

// Math Operators 

#define PLUS	'+'					// "+" Operator
#define MINUS	'-'					// "-" Operator
#define TIMES	'*'					// "*" Operator
#define DIVIDE	'/'					// "/" Operator

#define PERIOD	'.'					// "." Symbol
#define SEMICOLON	';'				// ";" Symbol

// Math functions


// Trigonometric Functions

#define SIN		'A'					// Sine function
#define ASIN  	'B'					// Tangent function
#define COS		'C'					// Cosine function
#define ACOS  	'D'					// Arc Cosine function
#define TAN		'E'					// Tangent function
#define ATAN	'F'					// Arc Tangent function
#define ATAN2	'G'					// Arc Tangent Two function
 
// Scientific Functions

#define SQRT	'H'					// Square root function
#define SQR		'I'					// Square function
#define EXP		'J'					// Exponential function E**X
#define LOG		'K' 				// Log function
#define Log10	'L' 				// Log base 10 function
#define POWER	'M' 				// X**N  function
#define MAX		'N'                 // Maximum of x and y
#define MIN		'O'                 // Minimum of x and y
#define ABS		'P'                 // Absolute value of x
#define SIGN	'Q'                 // Sign of x and y

// Special functions

#define RAD 	'R'					// Convert to radians
#define DEG 	'S'					// Convert to degrees
#define FIX 	'T'					// Rounds x towards 0
#define FLOOR 	'U'					// Rounds x towards - Infinity
#define CEIL 	'V'					// Rounds x towards + Infinity
#define ROUND 	'W'					// Round x to nearest integer
#define PRECISION 	'X'				// Specify maximum number of digits precision up to machine precision used for display purposes
#define MOD		'Y'					// Mod Function identical to C mod function 


//************************************************************************************************

typedef union message_union_type
{
    CALCULATE_TYPE CalculateCommand;							// Calculate command message
	GET_ANSWERS_TYPE GetAnswersCommand;							// Get answers command message
	CLEAR_MEMORY_TYPE CLearMemoryCommand;						// Clear memory command message
	STORE_TYPE StoreCommand;									// Store command message
	RECALL_TYPE RecallCommand;                          		// Recall command message	
 	GET_STATUS_COMMAND_TYPE GetStatusCommand;					// Get status message
	RESET_COMMAND_TYPE ResetCommand;					 		// Reset command message

	COMMAND_MESSAGE_TYPE SelectedCommand;						// Selected command	
	MESSAGE_TYPE Message;										// Unformatted message
} MESSAGE_UNION_TYPE;

// Define the Math Co-processor Commands message
MESSAGE_UNION_TYPE CoProcessorMessage;
  
#ifdef SINGLE_PRECISION
float Registers[MAX_FLOAT_REGISTERS];
float Answers[MAX_ANSWERS];
#else
double Registers[MAX_FLOAT_REGISTERS];
double Answers[MAX_ANSWERS];
#endif

char af1[BufferSize];    // Holds ascii representation of an IEEE floating point number for testing
/*
char Period[] = ".";
char Plus[] = "+";
char Minus[] = "-";
char Blank[] = " ";
char E[] = "E";

char Operators[] = {'+','-','*','/','.','=',0};          // Math operators 
//extern char Operators[];                               // Math operators declared in keypad.c

char NumericChars[] = {'.','E',0};                       // Numeric characters
*/
unsigned char DisableLift = FALSE;								 // Disable the lift flag

static int Index = 0;                            		// SPI Ring Buffer index
static byte CurrentState = 0;                    		// Current SPI state variable

static byte SyncFlag = FALSE;							 //  SPI Master/Slave Sync flag
static byte GetStatusFlag = FALSE;						 //  SPI Master/Slave Get Status flag
static byte AnswerIndex = 0;								// Index into co-processor answers buffer
extern byte ReceivedDataFlag ;
int NumChars;

extern byte Ch;  
 
// Declare function headers here

void main(void);
void isrhandler(void);                           // This function directs execution to the
void ServiceHighInterrupt(void);
void InitializeMathCoprocessor(void);
void SyncWithMaster(void);
void SendByte(byte TheData);
byte GetByte(void);
void EnableSPIInterrupts(void);
 
void InitializeSPISlave(void);
unsigned char ReadSPIByte( void );
unsigned char WriteSPIByte( unsigned char data_out );
void ReadSPIString( unsigned char *rdptr, unsigned char length );
void WriteSPIString( unsigned char *wrptr );
void EnableSPIInterrupts(void);
byte SPI_Get_Byte(void);
void DisableSPIInterrupts(void);
 

// HP-45 Functions

void EvaluateRPN(void);
double Calculator(double Argument1, char Operator, double Argument2);
void Lift(void);
void Drop(void);
void Roll(void);
void Swap(void);
 

// Floating point conversion functions

void Normalize(double *Value, int *Exponent);
unsigned long NormalizeFraction(double FractionalPart, int NumberOfDigits);

#ifdef SINGLE_PRECISION
void FloatToAscii(float Number, char * s);
void AsciiToFloat(float Number, char * s);
#endif
void DoubleToAscii(double Number, char * s);
void AsciiToDouble(double *Number, char * s);
int Member(char Ch, char * Set);

float GetFloat(void);
double GetDouble(void);
//double log10(double x);
double pow10(int x);
void ConfigureInterrupts(void);
char GetKey(void);

//char TheOperator;        // Current operator function key: "+","-","*","/"
//char TheEqualSign;       // Equal sign, "="

double Argument1;
double Argument2;
double Result;

// Calculator specific variables

extern char Buffer[BufferSize];				// Floating point Buffer
char StackLabel[]={'X','Y','Z','T'};		// Stack register labels
double Stack[StackSize];					// HP-45 Stack 
double Registers[RegisterSize];				// HP-45 Registers 1-9
double X = 0.0;								// X register (Display)
double Y = 0.0;								// Y register
double LastX = 0.0;							// Last X register
byte RegisterNumber = 0;					// Register number 1..9
//int CharacterCount = 0;								// Character count
int BufferCount = 0;								// Buffer count
int MessageBufferIndex = 0;							// Message buffer index
 
byte KeyValue; 
 
double my_x;
byte TheCharacterRead;
byte TheCharacterSent = 0xAB;


byte ControllerState; 		// State of selected slave controller, range (0..255) 
											// 		0 => NO_STATES		No States    
											//		1 => INITIALIZED	The selected slave contoller is Initialzed 
											//		2 => DONE 	 	 	The selected slave contoller is done processing current command
											// 		3 => BUSY 	 		The selected slave contoller is busy processing current command 
											// 		4 => ERROR 	 		The selected slave contoller encountered an error processing the current command 

byte ControllerErrorCode; 	// Error code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_ERRORS 		No error codes  
											// 		1 => FAILURE 	 	Failed to complete command
											//  	2 => SUCCESS  	 	Completed command successfully 
 
byte ControllerDiagnosticCode; 	// Diagnostic message code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_DIAGNOSTICS 	No Diagnostic messages
											//  	1 => DIAGNOSTIC 1  	Diagnostic message 2 
											// 		2 => DIAGNOSTIC 2 	Diagnostic message 1
 
// Declare SPI Master Buffer variables here

byte SPI_Rx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Receive Buffer
byte SPI_Tx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Transmit Buffer
byte SPI_Start_Buffer=0;                         		// Start pointer for ring buffer
byte SPI_End_Buffer = 0;								// SPI End Buffer Index

byte SPI_Rx_Flag = FALSE;                        		// Indicates that data was received into the ring buffer
word SPI_Timeout = 0;                            		// SPI I/O Timeout counter
byte SPI_Timeout_Flag;                           		// SPI Timeout flag
word SPI_Number_Characters = 0;						 	// SPI Number of characters in buffer
int SPI_Communications_Error;                    		// SPI I/O error flag
                                                 		// -1 = SPI Input Bufer overrun error
                                                 		// -2 = SPI Communications timeout error
                                                 		// -3 = SPI buffer empty error

int SPI_Interrupt_Count = 0;                   	// Number of MSSP interrupts
int SPI_Data_In = 0;								// Data byte read from the SPI Master
int SPI_Data_Out = 0;							// Data byte sent to the SPI Master
int SPI_Status;									// SPI Status returned from routines
 
int MessageIndex = 0;							// Message ID number

//********************************************************************************
// Interrupt Code
//********************************************************************************

// Designate ServiceInterrupt as a high priority interrupt function and save key registers

//#pragma code interrupt ServiceInterrupt save = PRODL,PRODH 

// Locate ISR handler code at interrupt vector

#pragma code isrhandler=0x08

void  isrhandler(void)                           // This function directs execution to the
{                                                          // actual interrupt code
 
	_asm   goto ServiceHighInterrupt  _endasm
 
}

#pragma code

#pragma interrupt ServiceHighInterrupt save=section(".tmpdata")

//********************************************************************************
//* ServiceHighInterrupt - Process each type of High Priority interrupt  
//* (MSSP, USART, INT0..INT2, and TMR0IF..TMR3IF).
//********************************************************************************
void    ServiceHighInterrupt(void)
{

 
	int i;													// Local loop index
	
 	//SPISIntISR();			// Maestro SPI Driver

	
#ifdef SPI_BUFFER_INTERFACE
		
	// Interrupt Logic for SPI Slave Buffer Interface goes here ...

    if (PIR1bits.SSPIF)                          			// Check for incomming characters from SPI
    {
     
    	// Turn on trace 0
                             
      	//TRACE0 = ON;
 	
		// Don't accept any data if busy processing

 
        // Read a single byte from the SPI Bus (SSPBUF) into the Rx buffer.
	
		SPI_Rx_Buffer[SPI_End_Buffer] = ReadSPIByte();
 		
		//Delay1TCY(); 					// 1 Clock cycle delay same as a Nop() 
 
		// Write response from Tx buffer to the SPI Bus

 		//SPI_Status = WriteSPIByte(SPI_Tx_Buffer[SPI_End_Buffer]);
 		SPI_Status = WriteSPIByte(0xAB);

		// Check for buffer full indicating the end of message

       	//if (SPI_End_Buffer ==  (SPI_BUFFER_SIZE - 2))
       	if (SPI_End_Buffer == SPI_BUFFER_SIZE)
        {
 
 	
       		//SPI_Number_Characters = 0;
       		
   			// Copy Rx buffer to local message buffer for processing 

			for (i=0; i<DATA_BUFFER_SIZE; i++)
			{
				CoProcessorMessage.Message.Data[i]= SPI_Rx_Buffer[i];
			}
   			//memcpy(&CoProcessorMessage.Message.Data, SPI_Rx_Buffer+1, DATA_BUFFER_SIZE);  
  			       	
       		SPI_End_Buffer = 0;
         	SPI_Rx_Flag = TRUE;				// Set the SPI message Received Data flag
      				   			
       	}
        
		// Increment buffer index 
 			
		SPI_End_Buffer++;										 			// Increment the SPI Ring Buffer pointer
        //SPI_End_Buffer = SPI_End_Buffer % SPI_BUFFER_SIZE;        		// Wrap the EndBuffer pointer if necessary
      
      	// Increment SPI Interrupt counter
      	SPI_Interrupt_Count++;
      	
       	// Turn OFF trace 0

      	//TRACE0 = OFF;
   	
  	   	// Clear the SSPIF flag

  		PIR1bits.SSPIF = 0;
 		 
   	}   
#endif

#ifdef SPI_NETWORK_INTERFACE
		// Interrupt Logic for SPI Slave Network Interface goes here ...
		
        // Read the message data sent from the SPI Slave
   	   	getsSPI(SPI_Rx_Buffer, sizeof(SPI_Rx_Buffer));
   	   	
   	   	// Copy the buffer and add 10 to each element
   	   	
		for (i=0; i<SPI_BUFFER_SIZE; i++)
		{
 			SPI_Tx_Buffer[i] = SPI_Rx_Buffer[i] + 10;
 		}
 		
 		// Send message back to SPI Master
        putsSPI(SPI_Tx_Buffer);					// Send the message stored in the SPI_Tx_Buffer to the 
                                  				// SPI Slave Co-Processor
 
        //pause(5); 								// Wait some time to allow it to be processed.
         										// We may want handshaking here...
 
#ifdef TEST   	   	
   	   	// Display the data sent and received 
/*   	   	
		for (i=0; i<SPI_BUFFER_SIZE; i++)
		{
			printf("Slave: i = ");
			dec(i);
			printf("  SPI_Tx_Buffer = ");
			dec(SPI_Tx_Buffer[i]);
			
			printf("  SPI_Rx_Buffer = ");
			dec(SPI_Rx_Buffer[i]);
			
			printf("\r\n");
			
			pause(1000);
		}
*/ 
#endif   	   	
#endif       
        
/*
    // Use serial I/O interrupt
    
     else
    if (PIR1bits.RCIF)                                     // Check for incomming characters from USART  
    {
		ProcessUSART();                                    // Process USART interrupt here
    } 
*/

}
 
//******************************************************************************
//* main - Slave Co-processor application accepts expressions and commands from the 
//* master controller, evaluates them and sends back the answers. 
//******************************************************************************
#ifdef SLAVE_COPROCESSOR

 
void main (void)
{

	// local variables for all test cases and their initialisation
	unsigned char RxBuffer[34];
	unsigned char TxBuffer[34];
	unsigned char chData;
	int i;                                   // Index 
	byte Ch;

	
#ifdef STDIO_INTERRUPTS	
	ReceivedDataFlag =  FALSE;
#endif

 	NumChars = 0;									// Initialize the number of characters read from the message buffer
	BufferCount = 0;								// Buffer count
	AnswerIndex = 0;								// points to current answer in Answer[] array				

	//SetupTimers();                                // Setup timers and software
                                                    // variables.
	// Initialize math test package 

	InitializeMath();

 	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
	InitializeUsart(); 

	// Initialize the Math Co-processor 
	
	InitializeMathCoprocessor();

	printf("SPI Co-Processor Application ...\r\n");

	// Check timer code by delaying 5 seconds ...

	//printf("Start of 5 second delay...\r\n");
	//pause(5000);
	//printf("End of 5 second delay. \r\n");


#ifdef SPI_BUFFER_INTERFACE
	// Initialization Logic for SPI Slave Buffer Interface goes here ...

	// Flush the Rx, Tx and message buffers
	for (i=0; i<SPI_BUFFER_SIZE; i++)
	{
 		SPI_Rx_Buffer[i] = 0;                  			// Clear SPI Receive Buffer 
 		//SPI_Tx_Buffer[i] = SPI_BUFFER_SIZE - i;        	// Set SPI Transmit Buffer to a test ramp  
 		SPI_Tx_Buffer[i] = 0;        	// Clear SPI Transmit Buffer   
 		CoProcessorMessage.Message.Data[i] = 0;
	}

	for (i=0; i<DATA_BUFFER_SIZE; i++)
	{
 		CoProcessorMessage.Message.Data[i] = 0;
	}	
#endif	

 	// Enable the SPI interrupts after all necessary initialization is complete
	EnableSPIInterrupts();

#ifdef TEST_0

	// Simple SPI Ramp test

   	while(1)
   	{
 		// Clear the watchdog timer each time through the major menu loop

		//ClrWdt();
 		
   		if (SPI_Rx_Flag == TRUE)
   		{
   			SPI_Rx_Flag = FALSE;

#ifdef TEST   			
			printf("Slave: Mainline, SPI_Interrupt_Count = ");
			dec(SPI_Interrupt_Count);
			printf("\r\n");
			
			// Display contents of SPI Buffers 	
			
			for (i=0; i<SPI_BUFFER_SIZE; i++)
			{
				printf("Slave: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);
			}
#endif
		}			
 
 	}
    
#endif

//****************************************************************************************************
//****************************************************************************************************  
#ifdef NEW_CODE
     
  	// Test the scientific functions

/*
    // log - Natural log test

	my_x = log(1.0);
	my_x = log(2.0);
   	my_x = log(2.78182818);
 	my_x = log(3.14159263);


    // log10  - Log base 10 test

	my_x = log10(1.0);
	my_x = log10(2.0);
	my_x = log10(10.0);
	my_x = log10(100.0);
	my_x = log10(1000.0);
	my_x = log10(3.14159263);    

	// exp - exponential function test

*/
  
    // The my_sqrt funtion works !!!
    
 	my_x = sqrt(4.0);
	my_x = sqrt(16.0);
	my_x = sqrt(144.0);

	
    my_x = sqrt(25.0);
    my_x = sqrt(1024);
    my_x = sqrt(144.0);
    
 
	my_x = sqrt((double)25.0);
 	
	// Test J. Crenshaw's sqrt function
	
	my_x = sqrt(2.0);
	my_x = sqrt(4.0);
	my_x = sqrt(16.0);
	my_x = sqrt(144.0);
	
	// Test Trig functions

    // Sin and Asin test

	my_x =   sin(radians(30.0));
    my_x =   degrees(asin(my_x));

	my_x =   sin(radians(45.0));
    my_x =   degrees(asin(my_x));

	my_x =   sin(radians(60.0));
    my_x =   degrees(asin(my_x));

    // Cos and Acos test

	my_x =   cos(radians(30.0));
    my_x =   degrees(acos(my_x));

	my_x =   cos(radians(45.0));
    my_x =   degrees(acos(my_x));

	my_x =   cos(radians(60.0));
    my_x =   degrees(acos(my_x));

    // Tan and Atan test

	my_x =   tan(radians(30.0));
    my_x =   degrees(atan(my_x));

	my_x =   tan(radians(45.0));
    my_x =   degrees(atan(my_x));

	my_x =   tan(radians(60.0));
    my_x =   degrees(atan(my_x));

    // Atan2(y,x) test

	my_x = degrees(atan2(0.0, 1.0));  	// 0.0 Degrees
	my_x = degrees(atan2(1.0, 0.0));  	// 90.0 Degrees
	my_x = degrees(atan2(0.0, -1.0));	// 180.0 Degrees
	my_x = degrees(atan2(-1.0, 0.0));  	// 270.0 Degrees
	my_x = degrees(atan2(0.5, 0.5));	// 45.0 Degrees
	my_x = degrees(atan2(0.5, -0.5));	// 135.0 Degrees
	my_x = degrees(atan2(-0.5, -0.5));	// 225.0 Degrees
	my_x = degrees(atan2(-0.5, 0.5));	// 315.0 Degrees
#endif


//****************************************************************************************************
//****************************************************************************************************



 
// Initialize test data messages

#ifdef DEBUG
	SPI_Rx_Flag = TRUE;													// Flag indicating that a message was received
  	CoProcessorMessage.CalculateCommand.MessageID = 1;    				// Message ID (0000 - FFFF) Hex

  	CoProcessorMessage.CalculateCommand.Command = CALCULATE_COMMAND;           // Send Calculate Command to math coprocessor   (see enumeration
      
                              								// declaration in network.h)

	// Copy test buffer which is initialized in mpsim.inc with Coprocessor test data

    for (i=0; i<strlen(InputBuffer); i++)
	{
    	CoProcessorMessage.CalculateCommand.Buffer[i] = InputBuffer[i];					// RPN expression to be evaluated: "12.0E2.0/...";
		//Ch = (char) CoProcessorMessage.CalculateCommand.Buffer[i];
 	}
 	 
#endif

  	
   while(1)
   {
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
   		if (SPI_Rx_Flag == TRUE)
   		{
  			BUSY_FLAG = 1;											// Let the Master know the Slave is Busy processing
 
 			SPI_Rx_Flag = FALSE;
   			



	//******************************************************************************
	// Process Math Co-processor Commands here
	//******************************************************************************

 
   		
    		// Change controller state to BUSY
			ControllerState = BUSY; 
				  	
       		// Process the selected command
			        	
 			switch (CoProcessorMessage.SelectedCommand.Command)
 			{
				case CALCULATE_COMMAND:
				{
 					// Process the Calculate command message that evaluates RPN expression
 					ProcessCalculateCommand();

 					// Process Get Answers command message in order to return the floating point answers to the master
  					//ProcessGetAnswers();
 				
					break;
				} 
			
				case GET_ANSWERS_COMMAND:
				{
		
 					// Process Get Answers command message in order to return the floating point answers to the master
  					ProcessGetAnswers();
  					
  					break;
				}
			
				case CLEAR_MEMORY_COMMAND:
				{
  					// Process the Clear Memory Command message that erases the floating point registers
					ProcessClearMemoryCommand();
					
 					break;
				}
			
				case STORE_COMMAND:
				{

 					// Process the Store Command message that saves a floating point value to a selected float register
					ProcessStoreCommand();
					
   					break;
				}
			
				case RECALL_COMMAND:
				{

  					// Process the Recall Command message that  Retrieves a floating point value from a 
  					// selected float register.	
  					ProcessRecallCommand();
   					break;
				}
#ifdef NEW_CODE
				case FILTER_COMMAND:
				{
  					// Process the Filter command message that performs selected filter 
  					// operation (FIR, IFR)

					ProcessFilterCommand();
    				break;
				}
				
				case FFT_COMMAND:
				{
  					// Process the FFT command message that performs the selected 
  					//FFT operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case IFFT_COMMAND:
				{
  					// Process the Inverse Filter command message that performs an inverse
  					// FFT operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case KALMAN_FILTER_COMMAND:
				{
  					// Process the Kalman Filter command message that performs a Kalman
  					// Filter on the specified data

					ProcessKalmanFilterCommand();
    				break;
				}
				
				case MEDIAN_FILTER_COMMAND:
				{
  					// Process the Median Filter command message that performs a Median filter 
  					//operation on the specified data.

					ProcessFilterCommand();
    				break;
				}
				
				case HISTOGRAM_COMMAND:
				{
  					// Process the Histogram command messagePerforms a Histogram operation

					ProcessHistogramCommand();
    				break;
				}
				
				case SORT_COMMAND:
				{
  					// Process the Sort command message that performs the selected Sort operation 
  					//on the specified data

					ProcessSortCommand();
    				break;
				}
#endif			
				case GET_STATUS_COMMAND:
				{
  					// Process the Get Status Command message

					ProcessGetStatusCommand();
					
   					break;
				}
				
				case RESET_COMMAND:
				{
  					// Process the Reset controller command message

					ProcessResetCommand1();
					
   					break;
				}

			}
		 	  
        	// Copy the entire Co-Processor Response to the SPI_Tx_Buffer for transmission to the SPI Master
        	// during the next message sent from the Master.  This 1 cycle delay allows the Math CO-processor 
        	// complete its task and provide the requested results.
          		 			
  			//memcpy(SPI_Tx_Buffer+2, &CoProcessorMessage.Message.Data, DATA_BUFFER_SIZE);  
  			
			// Copy local message buffer back to Tx buffer to send results back to Master
	
			for (i=0; i<DATA_BUFFER_SIZE; i++)
			{
				SPI_Tx_Buffer[i] = CoProcessorMessage.Message.Data[i];
			}
 
	  		// Change controller state to DONE

			ControllerState = DONE;

       		SPI_End_Buffer = 0;

			
			// Reset MSSP to send SPI Message response

			//CloseSPI();

    		//OpenSPI(SLV_SSON, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           			// SPI Mode 0,0, and Sample on falling edge. Almost works!

     		//OpenSPI(SLV_SSON, MODE_11, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           			// SPI Mode 1,1, and Sample on falling edge..
 
			BUSY_FLAG = 0;											// Let the Master know the Slave is done processing and
                                                                    // can accept another message while transmitting back the results
		
			// Send results back to Master

			for (i=0; i<SPI_BUFFER_SIZE; i++)
			{

        		// Read a single byte from the SPI Bus (SSPBUF) into the Rx buffer.
	
				SPI_Data_In  = ReadSPIByte();
 		
				//Delay1TCY(); 					// 1 Clock cycle delay same as a Nop() 
 
				// Write response from Tx buffer to the SPI Bus

 				SPI_Status = WriteSPIByte(SPI_Tx_Buffer[i]);

#ifdef TEST
				printf("Coprocessor: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				hex4(SPI_Tx_Buffer[i]);	
				printf("\r\n");
			
				//pause(1000);
#endif
  			}

#ifdef TEST  
/*
//??????????????????????????????????????????????? DEBUG ?????????????????????????????????
			for (i=0; i<DATA_BUFFER_SIZE; i++)
			{
				SPI_Tx_Buffer[i] = i;
			}	
		
			for (i=0; i< SPI_BUFFER_SIZE; i++)
   			{
				SPI_Tx_Buffer[i] = SPI_BUFFER_SIZE-i;
			}
*/			 			
			printf("Slave Co-processor application, SPI_Interrupt_Count = ");
			dec(SPI_Interrupt_Count);
			printf("\r\n");
			
			// Display contents of SPI Send and Receive Buffers 	
			
			for (i=0; i<SPI_BUFFER_SIZE; i++)
			{
				printf("Math Co-processor: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
                printf("  ");
                putch((char)SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);
			}		

       		printf("Slave: Math Co-processor received the command:.");
#endif       
			
 /*        				
			// Display the current contents of the buffer
			for (i=0; i<SPI_Number_Characters; i++)
			{
				printf("Slave: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);

			}

*/

//*******************************************************************************
//#endif

 
 		}
 		
//#endif 		
 		
  		// Check to see if the SPI Co-Processor Slave Select is asserted (=0).
      	// Disable the SPI interrupts if it is not.
/*
      	if (!SS)
      	{
      		// Enable SPI MSSP interrupts

			EnableInterrupts();
      	}
      	else
      	{
 		 	// Disable SPI MSSP interrupts

			DisableInterrupts();
     	}
*/

		}
    
 
}
//******************************************************************************
//* EvaluateRPN - Evaluates an RPN Expression in order to handle more complicated
//* arithmetic expressions. It does not currently support trigonometric or scientific 
//* functions.
//******************************************************************************
void EvaluateRPN(void)
{
		char Ch;					// Local character 

 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
				
		// Get a token character (operator or numeric) from the message buffer 
 
   		while ((NumChars <  MESSAGE_BUFFER_SIZE) && (NumChars < strlen(CoProcessorMessage.CalculateCommand.Buffer)))
   		{
 
	 
			Ch = CoProcessorMessage.CalculateCommand.Buffer[NumChars++];					// Get next token from message buffer  
 
			// Check for end of statement semicolon character, ";"

 			if (Ch == ';')
			{
				// Save last answer and continue parsing the buffer until all characters in it have been processed.
                // If answers exceed the maximum number then wrap the index to 0 using the mod function.

				Answers[AnswerIndex++]= LastX;
 				AnswerIndex = AnswerIndex % MAX_ANSWERS;
				
			}

			if (isdigit(Ch) || (Ch == '.'))
			{
				// Save the key in the input buffer

				Buffer[BufferCount++] = Ch;
 			}
			else
			{
												

				if (BufferCount > 0)
				{
					// Automatically lift the stack by one unless lift has been disabled for the current operation
 				
 					Lift();
 					
					// This logic needs to be modified or moved ?????
				
					Buffer[BufferCount++] = 0;			// Terminate the string 
				
					// Convert the value stored in the buffer from ascii to float and save it in register X
 
					AsciiToDouble(&X, Buffer);
				
					// Copy number from the display register X to the Stack register X 
				
 					Stack[X_LOCATION] = X;
 				
					LastX = X;							// Save value of X into LAST_X register
					BufferCount = 0;					// Buffer count
				}	
 										
				// Interpret the HP-45 RPN command here
				
				switch (Ch)
				{
					// Stack operations

					case ENTER:						// "Enter ^" key 
					{
						// Lift stack up one level and disable automatic lift for the next operation

						Lift();
						DisableLift = TRUE;
						break;
					}

					case ROLL:						// "R !" key 
					{
						// Rotate the values in the stack while preserving all values.    

						Roll();
						break;
					}

					case SWAP:						// "X <> Y" key 
					{
						// Swap X with Y

						Swap();
						break;
					}

					case STORE:						// "STO" key 
					{
						// Store X into register N, where N=1..9

						RegisterNumber = CoProcessorMessage.CalculateCommand.Buffer[NumChars++];
						Registers[RegisterNumber] = X;
						break;
					}

					case RECALL:					// "RCL" key 
					{
						// Recall register N, where N=1..9 into X

						RegisterNumber = CoProcessorMessage.CalculateCommand.Buffer[NumChars++];
						X = Registers[RegisterNumber];
						break;
					}

					case LASTX:					// "LAST X" key 
					{
						// Recall Last X register into X

						X = LastX;
 						Stack[X_LOCATION] = X;
 						
						break;
					}

					// Basic math operations

					case PLUS:						// "+" key
					{
						// Compute the "+" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],PLUS, Stack[X_LOCATION]);
			
						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case MINUS:						// "-" key
					{
						// Compute the "-" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], MINUS, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case TIMES:						// "*" key
					{
						// Compute the "*" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION], TIMES, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
					case DIVIDE:					// "/" key
					{
						// Compute the "/" operation and place result in the X register

						Stack[X_LOCATION] = Calculator(Stack[Y_LOCATION],DIVIDE, Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}					

					// Trigonometric functions

					case SIN:					// sin(x) function  
					{
						// Compute the sin(x) and place result in the X register

						Stack[X_LOCATION] = sin(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 					
					case ASIN:					// asin(x) function  
					{
						// Compute the asin(x) and place result in the X register

						Stack[X_LOCATION] = asin(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case COS:					// cos(x) function  
					{
						// Compute the cos(x) and place result in the X register

						Stack[X_LOCATION] = cos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case ACOS:					// acos(x) function  
					{
						// Compute the acos(x) and place result in the X register

						Stack[X_LOCATION] = acos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case TAN:					// tan(x) function  
					{
						// Compute the cos(x) and place result in the X register

						Stack[X_LOCATION] = cos(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case ATAN:					// atan(x) function  
					{
						// Compute the atan(x) and place result in the X register

						Stack[X_LOCATION] = atan(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					// Scientific functions ...

					case SQRT:					// sqrt(x) function  
					{
						// Compute the sqrt(x) and place result in the X register

						Stack[X_LOCATION] = sqrt(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case SQR:					// sqr(x) function  
					{
						// Compute the sqrt(x) and place result in the X register

						Stack[X_LOCATION] = sqr(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
#ifdef NEW_CODE
					case EXP:					// sqr(x) function  
					{
						// Compute the exp(x) and place result in the X register

						Stack[X_LOCATION] = exp(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case LOG:					// log(x) function  
					{
						// Compute the log(x) and place result in the X register

						Stack[X_LOCATION] = log(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}					

					case log10:					// log10(x) function  
					{
						// Compute the log10(x) and place result in the X register

						Stack[X_LOCATION] = log10(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case POWER:					// power(x,n) function  
					{
						// Compute the power(x,n) and place result in the X register

						Stack[X_LOCATION] = power(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					} 					
#endif

					case MAX:					// max(x,y) function  
					{
						// Compute the max(x,y) and place result in the X register

						Stack[X_LOCATION] = max(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

					case MIN:					// min(x,y) function  
					{
						// Compute the min(x,y) and place result in the X register

						Stack[X_LOCATION] = min(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case ABS:					// abs(x) function  
					{
						// Compute the abs(x) and place result in the X register

						Stack[X_LOCATION] = abs(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case SIGN:					// sign(x,y) function  
					{
						// Compute the sign(x,y) and place result in the X register

						Stack[X_LOCATION] = sign(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case MOD:					// mod(x,y) function  identical to the C mod  function
					{
						// Compute the mod(x,y) and place result in the X register

						Stack[X_LOCATION] = mod(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
					


					// Special math coprocessor functions

 					case RAD:					// rad(x) function  
					{
						// Convert degrees to radians and place result in the X register

						Stack[X_LOCATION] = radians(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

 					case DEG:					// deg(x) function  
					{
						// convert radians to degrees and place result in the X register

						Stack[X_LOCATION] = degrees(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
 
#ifdef NEW_CODE

  					case FIX:					// fix(x) - function rounds x towards 0 
					{
						// Rounds x towards 0 and places result in the X register

						Stack[X_LOCATION] = fix(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

   					case FLOOR:					// floor(x) - Rounds x towards - Infinity 
					{
						// Rounds x towards - Infinity and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}

    				case FLOOR:					// floor(x) - Rounds x towards + Infinity 
					{
						// Rounds x towards + Infinity and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
     				case ROUND:					// round(x) - Round x to nearest integer
					{
						// Rounds x to nearest integer and places result in the X register

						Stack[X_LOCATION] = floor(Stack[X_LOCATION], Stack[Y_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
     				case PRECISION:					// precision(x) - Specify maximum number of digits precision up to machine precision used for display purposes
					{
						// Specify maximum number of digits precision up to machine precision used for display purposes

						Stack[X_LOCATION] = precison(Stack[X_LOCATION]);

						// Drop the stack to fill the gap left by the previous operation.  The contents of the Y register are lost.
				
						Drop();

						break;
					}
 
#define PRE 	'X'					// 

#endif
				}
							
							
				// Copy number from the Stack register X to the display register X and to the Last X register  
				
 				X = Stack[X_LOCATION];
				LastX = X;
				
				// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

				DoubleToAscii(X, af1);

 

			}
		
 
		}
 		

	
 
}

//******************************************************************************
//* Calculator - Simple calculator that performs double precision floating point
//* addition, subtraction, multiplication and division.  It does not currently
//* support levels of nesting parentheses for complicated mathematical expressions.
//******************************************************************************

double Calculator(double Argument1, char Operator, double Argument2)
{
	double TheResult;
 
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	switch(Operator)
	{
		case '+':
		{
			TheResult = Argument1 + Argument2;
			break;
		}

		case '-':
		{
			TheResult = Argument1 - Argument2;
			break;
		}

		case '*':
		{
			TheResult = Argument1 * Argument2;
			break;
		}

		case '/':
		{
			TheResult = Argument1 / Argument2;
			break;
		}

		default:
		{
			TheResult = 0;
			break;
		}		

	}

	return (TheResult);

}



// HP-45 Functions

//******************************************************************************
//* Lift - Pushes the value in register X onto the simulated HP-45 Stack.  The 
//* contents of register T, at the top of the stack is lost, being replaced by
//* the contents register Z. The next number keyed into X overwrites the old X.
//******************************************************************************
void Lift(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
	// Check to make sure that the Lift operation has not been disabled 
	
	if (DisableLift)
	{
		DisableLift = FALSE;								// Enable Lift for the next operation
	}
	else
	{
 		Stack[T_LOCATION] = Stack[Z_LOCATION]; 				// Contents of T are lost
 		Stack[Z_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register Z   
 		Stack[Y_LOCATION] = Stack[X_LOCATION]; 				// Copy contents of register X to register Y  
 		//Stack[Y_LOCATION] = X; 							// Copy contents of register X to register Y
 	}  
}

//******************************************************************************
//* Drop - The rest of the stack is dropped to fill the gap left by the 
//* previous operation.  The value of the Y register is lost. 
//******************************************************************************
void Drop(void)
{
	int i;												// Local loop variable

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	//Stack[X_LOCATION] = Stack[Y_LOCATION]; 			// Contents of X are lost
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Y to register Z   
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register X to register Y  
	//X = Stack[X_LOCATION]; 							// Copy contents of register X to X  

 		
 		
	// Convert the value in register X from float to ascii for display to the monitor, LED or LCD.

	DoubleToAscii(Stack[X_LOCATION], af1);
		
	// Send result to Serial Port
	//printf(" = ");
	//puts(af1);

/*	
	// Display the contents of the Stack
		
	printf("\r\nStack...\r\n");
	for (i=3; i>=0; i--)
	{
 		putch(StackLabel[i]);
 		printf(": ");
		DoubleToAscii(Stack[i], af1);
		string(af1);
	}
	printf("\r\n> ");					// Calculator prompt
*/

 				
#ifdef KEYPAD_DRIVER
	// Hit any  key to continue
	Ch = GetKey();                      // Keypad Input    
#endif
	
 	//lcd_clear();						// clear LCD, home cursor
 	//printf("> ");					// Calculator prompt
	
	
}

//******************************************************************************
//* Roll - 	 Rotate the values in the stack while preserving them.    
//******************************************************************************
void Roll(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	
 	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X
 	Stack[Y_LOCATION] = Stack[Z_LOCATION]; 				// Copy contents of register Z to register Y
 	Stack[Z_LOCATION] = Stack[T_LOCATION]; 				// Copy contents of register T to register Z    
 	Stack[T_LOCATION] = X; 								// Copy contents of X to register T  
    X = Stack[X_LOCATION];								// Copy contents of register X to X 
 
}

//******************************************************************************
//* Swap - Swaps the contents of register X with the contents of register Y.
//******************************************************************************
void Swap(void)
{
	double TempValue;									// Temporary storage for swap
	
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
	TempValue = Stack[X_LOCATION];						// Save contents of register X
	Stack[X_LOCATION] = Stack[Y_LOCATION]; 				// Copy contents of register Y to register X   
  	Stack[Y_LOCATION] = TempValue; 						// Copy contents of TempValue to register Y     	
	X = Stack[X_LOCATION]; 								// Copy contents of register X to X  
 
}
//********************************************************************************
//* ConfigureInterrupts - Configure interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void ConfigureInterrupts(void)
{
     // Configure SPI interrupts (1 = High Priority, 0 = Low Priority).

    IPR1bits.RCIP = 1;

    // Enable SPI asynchronous interrupt

    PIE1bits.RCIE = 1; 

  // Enable peripheral interrupts
 
   INTCONbits.PEIE = 1;                                    

    // Enable general interrupts

    INTCONbits.GIE = 1;                                    // Enable all interrupts	   
}


//********************************************************************************
//* EnableSPIInterrupts - Enable interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void EnableSPIInterrupts(void)
{

	// Enable MSSP interrupts
 	PIE1bits.SSPIE = 1;

    // Enable USART asynchronous interrupt

    //PIE1bits.RCIE = 1;

  	// Enable peripheral interrupts
   INTCONbits.PEIE = 1;

    // Enable general interrupts
    INTCONbits.GIE = 1;                                    // Enable all interrupts
}

//********************************************************************************
//* DisableSPIInterrupts - Disable interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void DisableSPIInterrupts(void)
{


    // Disable general interrupts

    INTCONbits.GIE = 0;                                    // Disable all interrupts
}

//********************************************************************************
//********************************************************************************
//********************************************************************************
//********************************************************************************
 

//********************************************************************************
//* InitializeMathCoprocessor - Initializes Timers, program variables and peripheral registers
//* for the math co-processor.
//********************************************************************************
void            InitializeMathCoprocessor(void)
{
 	int i; 									// Local loop index
 
  	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();
       
    // Initialize SPI device address pointer

    //AddressPointer = 0;
    SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts                           

    // Initialize Rx, Tx and local message buffer to zero  
 
    for (i=0; i< SPI_BUFFER_SIZE; i++)
    {

		SPI_Rx_Buffer[i] = 0;
		SPI_Tx_Buffer[i] = 0;
 		CoProcessorMessage.Message.Data[i] = 0;
   	} 
 		
	for (i=0; i<DATA_BUFFER_SIZE; i++)
	{
 		CoProcessorMessage.Message.Data[i] = 0;
	}	

   	for (i = 0; i < MAX_ANSWERS; i++)
	{	
		Answers[i] = 0.0;
 	}

     // configure port A for all digital inputs
 
    OpenADC(ADC_0ANA, ADC_INT_OFF);
   
    // Initialize ports  

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                        // Clear PORTC (Some Port C bits are reserved for USART)
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE
    PORTF = 0x00;                                          // Clear PORTF

    // Initialize port I/O direction for output

 
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
    //TRISC = 0x00;                                        // PORTC all outputs (Some Port C bits are reserved for USART).
    TRISD = 0x00;                                          // PORTD all outputs.
    TRISE = 0x00;                                          // PORTE all outputs.
    TRISF = 0x00;                                          // PORTF all outputs.

	// Initialize the SPI Slave Pin directions
	//SPI_SCL_DIR = INPUT; 									// Set SPI Clock Direction to Input
 	SPI_SI_DIR = INPUT;										// Set SPI Input Direction  to Input
    SPI_SO_DIR = OUTPUT; 									// Set SPI Output Direction to Output

    //SS_DIR = INPUT;									  		// Set Slave Select line to Input

 	BUSY_FLAG_DIR = OUTPUT;										// Set port B RB0 to output
    

    //TRACE0_DIR   = 0;									    // Set TRACE 0 line to output

    // Turn OFF trace 0

    //TRACE0 = OFF;

    // Configure the PIC SPI module as a slave
        
    //OpenSPI(SLV_SSOFF, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                           // SPI Mode 0,0, and Sample on falling edge.

    //OpenSPI(SLV_SSOFF, MODE_00, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                           // SPI Mode 0,0, and Sample the middle.

    OpenSPI(SLV_SSON, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 0,0, and Sample on falling edge. Almost works!
   
	//OpenSPI(SLV_SSON, MODE_01, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 0,1, and Sample the middle.

 	//OpenSPI(SLV_SSOFF, MODE_01, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 0,1, and Sample on falling edge.
														   // Select this mode if not using the Slave Select line. It almost works !!

    //OpenSPI(SLV_SSOFF, MODE_11, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                           // SPI Mode 1,1, and Sample the middle.
    //OpenSPI(SLV_SSON, MODE_11, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 1,1, and Sample the middle.
    //OpenSPI(SLV_SSON, MODE_11, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 1,1, and Sample on falling edge. Almost works!
    //OpenSPI(SLV_SSOFF, MODE_11, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 1,1, and Sample on falling edge..


    PIR1 = 0;

    SSPSTAT = 0;

	SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts
	Index = 0;                               // SPI Ring Buffer index
	SPI_Rx_Flag =  FALSE;
	NumberCharacters = 0;

	CurrentState = 0;                        // Current SPI state variable
 
   // Configure USART interrupts (1 = High Priority, 0 = Low Priority).

   //IPR1bits.RCIP = 1;

	// Configure interrupts (1 = High Priority, 0 = Low Priority).  Interrupt 0 is always
	// set to high priority

	//INTCON3bits.INT1IP = 1;                                // Set interrupt 1 priority to high
	//INTCON3bits.INT2IP = 1;                                // Set interrupt 2 priority to high

	// Configure MSSP interrupts (1 = High Priority, 0 = Low Priority).

   IPR1bits.SSPIP = 1;

   // Clear the BF and SSPIF flag bits by reading the SSPBUF register

   SSPBUF = 0;
    
   PIR1bits.SSPIF = 0;
    
 

/*
   // Clear the BF and SSPIF flag bits by reading the SSPBUF register

    i = SSPBUF;  
    
    PIR1bits.SSPIF = 0;    
*/
 
	SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts
	Index = 0;                               // SPI Ring Buffer index
	SPI_Rx_Flag =  FALSE;
	NumChars = 0;

	CurrentState = 0;                        // Current SPI state variable
 

      // Wait 5 seconds 
/*      
      printf("5 Second Test ...\r\n");
      pause(5000);
      printf("End of 5 Second Test\n");
*/

	// Initialize Serial LCD Display here
	
	//printf("\r8-N-1");  
	 
	//printf("DIY Calculator \r\n");	
	
	// Test the Ascii to double conversion function
/*	
	af1[0] = '9';
	af1[1] = '8';
	af1[2] = '7';
	af1[3] = '.';
	af1[4] = '6';
	af1[5] = '5';
	af1[6] = '4';
	af1[7] = 0;
	
	AsciiToDouble(&X, af1);
	

   Enter();
*/	
/* 
	Buffer[0] = '0';
	Buffer[1] = '0';
	Buffer[2] = '0';
	Buffer[3] = '0';
	Buffer[4] = '0';
	Buffer[5] = '0';
	Buffer[6] = '0';
	Buffer[7] = 0;

	Buffer[0] = '3';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
	
	Buffer[0] = '4';
	Buffer[1] = '.';
	Buffer[2] = '0';
	Buffer[3] = 0;
	
	AsciiToDouble(&X, Buffer);
*/	

	// Initialize stack 
	
   Stack[X_LOCATION] = 0.0;
   Stack[Y_LOCATION] = 0.0;
   Stack[Z_LOCATION] = 0.0;
   Stack[T_LOCATION] = 0.0;


	// Test stack operations 
	
   Stack[X_LOCATION] = 1.0;
   Stack[Y_LOCATION] = 2.0;
   Stack[Z_LOCATION] = 3.0;
   Stack[T_LOCATION] = 4.0;
    
	// Make sure we can perform a stack lift initially
	
    DisableLift = FALSE;
    AnswerIndex = 0;
   	MessageBufferIndex = 0;
   	   
	// Initialize Math Co-processor Controller State, Error codes and Diagnostic codes
	
	ControllerState = INITIALIZED; 	  
   	ControllerErrorCode =  NO_ERRORS;	   
  	ControllerDiagnosticCode = NO_DIAGNOSTICS;  
 	BUSY_FLAG = 0;											// Let the Master know the Slave is ready  	

}

//****************************************************************************************
//****************************************************************************************
//****************************************************************************************
//****************************************************************************************


//**********************************************************************************
//* ProcessCalculateCommand - Evalueates the RPN expressions that are stored in    *
//* the message expression buffer.  Multiple expressions can be sent to the        *
//* co-processor for evaluation, if they are separated by a ";" semi-colon.  Large *
//* expressions can span multiple messages to the co-processor.                    *
//**********************************************************************************
byte ProcessCalculateCommand(void)  
{
 	byte CalculateCommandStatus = TRUE;							// Status return
 
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	// Build the Calculate Response message  
	
 	CoProcessorMessage.CalculateCommand.MessageID = MessageIndex++;		// Message ID range (0000 - FFFF) Hex
	CoProcessorMessage.CalculateCommand.Command = CALCULATE_COMMAND;	// Calculate command evaluates RPN expression
		 	
 	// Parse and Evalueate the RPN expression contained in the co-processor message buffer ...
 	
 	
 	
#ifndef TEST
    printf("slave: Calculate Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.CalculateCommand.MessageID);
   	printf("\r\n");
    printf("Buffer = !");
    string(CoProcessorMessage.CalculateCommand.Buffer);
    printf("!");
   	printf("\r\n\r\n");
#endif

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

 	// Evaluate the RPN  expression and store the answers in the Answer buffer
 	EvaluateRPN();
				
	return CalculateCommandStatus;
}

//**********************************************************************************
//* ProcessGetAnswers - Get the results from the latest arithmetic operations      *
//* carried out during the last cycle. Up to 7 separate single precision floating  *
//* point or 3 double precision  values may be returned during a single message    *
//* transfer.                                                                      *
//**********************************************************************************
byte ProcessGetAnswers(void)
{
 	byte GetAnswersStatus = TRUE;								// Status return
   	byte AnswerID;									 		    // Local Answer Loop Index 
 		
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	// Build the Get Answers Response message 
			
	CoProcessorMessage.GetAnswersCommand.MessageID = MessageIndex++;				// Message ID range (0000 - FFFF) Hex 
  	CoProcessorMessage.GetAnswersCommand.Command = GET_ANSWERS_COMMAND;  			// Gets the floating point answers (see enumeration
               			                 									        // declaration above.)                                 	 
 		    				
  	// Send  latest answers from requested RPN expressions back to the master.
	CoProcessorMessage.GetAnswersCommand.NumberOfAnswers = AnswerIndex;

#ifndef TEST

	// Send test answer responses back to master (Value of PI a digit at a time)
	 
    printf("slave: Get Answers Command: \r\n");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetAnswersCommand.MessageID);
#endif   	

#ifndef TEST

	CoProcessorMessage.GetAnswersCommand.NumberOfAnswers = 7;

	Answers[0] = 3.0;
	Answers[1] = 1.0;
	Answers[2] = 4.0;
	Answers[3] = 1.0;
	Answers[4] = 5.0;
	Answers[5] = 9.0;
	Answers[6] = 2.0;
	
/*	
	CoProcessorMessage.GetAnswersCommand.Answers[0] = 3.0;
	CoProcessorMessage.GetAnswersCommand.Answers[1] = 1.0;
	CoProcessorMessage.GetAnswersCommand.Answers[2] = 4.0;
	CoProcessorMessage.GetAnswersCommand.Answers[3] = 1.0;
	CoProcessorMessage.GetAnswersCommand.Answers[4] = 5.0;
	CoProcessorMessage.GetAnswersCommand.Answers[5] = 9.0;
	CoProcessorMessage.GetAnswersCommand.Answers[6] = 2.0;
*/	
#endif
	
 	//for (AnswerID = 0; AnswerID < CoProcessorMessage.GetAnswersCommand.NumberOfAnswers; AnswerID++)
 	for (AnswerID = 0; AnswerID < MAX_ANSWERS; AnswerID++)
	{	
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
		// Copy local answers to Answer message
		CoProcessorMessage.GetAnswersCommand.Answers[AnswerID] = Answers[AnswerID];

#ifdef TEST
   		printf("  Answer ID = ");
    	dec(AnswerID);
    	printf(", Answer = ");
    	//hex4(CoProcessorMessage.GetAnswersCommand.Answers[AnswerID]);  
    	hex4(Answers[AnswerID]);  
    	printf("\r\n");
#endif
 
	}

#ifdef TEST
    	printf("\r\n\r\n");
#endif

	return GetAnswersStatus;
}
			
//**********************************************************************************
//* ProcessClearMemoryCommand - Erases the contents of the Math co-processor's 
//* floating point storage registers. These same registers are used for storing and
//* retrieving temporary values and answers. It does this by setting floating point 
//* registers 0..MAX_FLOAT_REGISTERS to zero.  Registers 0..10 are reserved for answers.
//**********************************************************************************
byte ProcessClearMemoryCommand(void)
{
 	byte ClearMemoryCommandStatus = TRUE;							// Status return
    byte RegisterID;									 		    // Local Loop Index
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

  	// Build the Clear Memory Response message
 	
	CoProcessorMessage.CLearMemoryCommand.MessageID = MessageIndex++; 			// Message ID (0000 - FFFF) Hex
	CoProcessorMessage.CLearMemoryCommand.Command = CLEAR_MEMORY_COMMAND;		// Clears the floating point scratch registers (see enumeration
	
 	// Read the Pot control duty value and send it to the corresponding DC motor 
 
 	for (RegisterID = 0; RegisterID < MAX_FLOAT_REGISTERS; RegisterID++)
	{
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		Registers[RegisterID] = 0.0;
	}
	
#ifndef TEST
    printf("save: Clear Memory Command Completed.");
   	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   printf("\r\n\r\n");
#endif

	return ClearMemoryCommandStatus;
}

//**********************************************************************************
//* ProcessStoreCommand - Stores a floating point value to a selected floating     *
//* point register.     The register ID range is 0..MAX_FLOAT_REGISTERS.           *
//**********************************************************************************
byte ProcessStoreCommand(void)
{
 	byte StoreCommandStatus = TRUE;										// Status return

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	// Build Store Response message
 		
	CoProcessorMessage.StoreCommand.MessageID = MessageIndex++; 	// Message ID (0000 - FFFF) Hex

  	CoProcessorMessage.StoreCommand.Command = STORE_COMMAND;		// Stores a floating point value to a selected float register
  
  	     			                           						// (see enumeration declaration above.)  
			
 	// Store the value into the selected register.
  					
 	Registers[CoProcessorMessage.StoreCommand.RegisterID] = CoProcessorMessage.StoreCommand.RegisterValue;
  	 
#ifdef TEST
    printf("slave: Store Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   	printf("  Register ID = ");
   	dec(CoProcessorMessage.StoreCommand.RegisterID);
    printf("Register Value = ");
    hex4(Registers[CoProcessorMessage.StoreCommand.RegisterID]);
    printf("\r\n\r\n");
#endif 

	return StoreCommandStatus;
}
  					
//**********************************************************************************
//* ProcessRecallCommand - Retrieves a floating point value from the selected      *
//* floating point register. The register ID range is 0..MAX_FLOAT_REGISTERS.      *                                                               
//**********************************************************************************
byte ProcessRecallCommand(void)
{
 	byte RecallCommandStatus = TRUE;									// Status return

 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

   	// Build Recall Response message
 
 	CoProcessorMessage.RecallCommand.MessageID = MessageIndex++;   		// Message ID (0000 - FFFF) Hex

  	CoProcessorMessage.RecallCommand.Command = 	RECALL_COMMAND;  		// Retrieves a floating point value from a selected float register	
   	                                							    	// Command (see enumeration declaration above.)  
	
    // Accept number of samples xxxx to be used for calibration and data
   	// collection used for reading the DC Motor Control Pots, where the 
   	// number of samples range 0000..FFFF.
	
  	CoProcessorMessage.RecallCommand.RegisterValue = Registers[CoProcessorMessage.RecallCommand.RegisterID];
  	
#ifndef TEST
    printf("slave: Recall Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.RecallCommand.MessageID);
   	printf("  Register ID = ");
   	dec(CoProcessorMessage.RecallCommand.RegisterID);
    printf("Register Value = ");
    hex4(CoProcessorMessage.RecallCommand.RegisterValue);
    printf("\r\n\r\n");
#endif 

	return RecallCommandStatus;
}


//**********************************************************************************
//* ProcessGetStatusCommand - Process the the SPI Slave Controller Get Status 
//* command and return the latest command execution status. 
//**********************************************************************************
void ProcessGetStatusCommand(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	// Build Get Status Response Message
	
	CoProcessorMessage.GetStatusCommand.MessageID = MessageIndex++;	// Message ID range (0000 - FFFF) Hex
	CoProcessorMessage.GetStatusCommand.Command = GET_STATUS_COMMAND;		// Send Get Status Command
		
    CoProcessorMessage.GetStatusCommand.ControllerState = ControllerState; 		 			// Store current co-processor State
    CoProcessorMessage.GetStatusCommand.ControllerErrorCode = ControllerErrorCode; 			// Store current co-processor Error Code
    CoProcessorMessage.GetStatusCommand.ControllerDiagnosticCode = ControllerDiagnosticCode;  // Store current co-processor Diagnostic Code

 
 #ifndef TEST
    CoProcessorMessage.GetStatusCommand.ControllerState = 3; 		 			// Store current co-processor State
    CoProcessorMessage.GetStatusCommand.ControllerErrorCode = 2; 			// Store current co-processor Error Code
    CoProcessorMessage.GetStatusCommand.ControllerDiagnosticCode = 1;  // Store current co-processor Diagnostic Code
     
    printf("Slave: Received Get Status Command:");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.GetStatusCommand.MessageID);
   	printf("  ControllerState = ");
   	dec(ControllerState);
   	printf("  ControllerErrorCode = ");
   	dec(ControllerErrorCode);
   	printf("  ControllerDiagnosticCode = ");
   	dec(ControllerDiagnosticCode);
    printf("\r\n");
#endif

}
 			

//**********************************************************************************
//* ProcessResetCommand - Process the Reset the SPI Slave Math Co-processor.  This command 
//* resets the selected controller.  It is used to force a hard reset of the math
//* co-procesor when a fatal error is encountered, using the PIC's watchdog timer or 
//* via the "Z" command over the serial interface.
//**********************************************************************************
void ProcessResetCommand1(void)
{
 	// Clear the watchdog timer each time through the major menu loop

	ClrWdt();

	// Build Reset Response Message
	
	CoProcessorMessage.ResetCommand.MessageID = MessageIndex++;	// Message ID range (0000 - FFFF) Hex
	CoProcessorMessage.ResetCommand.Command = RESET_COMMAND;		// Send Reset Command
	
#ifndef TEST
    printf("Reset Command: ");
  	printf("  MessageID = ");
   	dec(CoProcessorMessage.ResetCommand.MessageID);
    printf("\r\n\r\n");
#endif

	// Reset the Controller by allowing the Watchdog Timer to timeout.
	
	while (1);
}
 			
 
//**********************************************************************************
//* SyncWithMaster - Synchronizes with Master by waiting for the SYNC byte and responding with
//* an ACK byte.  This function is called before sending or receiving any messages.
//**********************************************************************************
void SyncWithMaster(void)
{
	byte Ch;

	// Wait for the SYNC byte and Issue ACK byte bac to Master when it is finally received
 
 	Ch = 0;
	while (Ch != SYNC_BYTE)
	{
		Ch = GetByte();
	}

	SendByte(ACK_BYTE);
}

//**********************************************************************************
//* SendByte - First get the coprocessor's attention by synchronizing with it. 
//**********************************************************************************

void SendByte(byte TheData)
{
/*
//**************************   Missing Logic for SPI *****************************

	if(!vUARTIntStatus.UARTIntTxBufferFull)
	{
		UARTIntPutChar(TheData);		
	}
*/
}
//**********************************************************************************
//* GetByte - Read a byte from the serial port
//**********************************************************************************

byte GetByte(void)
{
	byte TheData = 0;
/*
//**************************   Missing Logic for SPI *****************************

	while(!vUARTIntStatus.UARTIntRxBufferEmpty)
	{
		UARTIntGetChar(&TheData);
	}
*/
	return TheData;
}



// SPI input with interrupts and SPI buffer

//******************************************************************************
//* SPI_Get_Byte - Get next character from the SPI Rx Buffer.  The function
//* returns a -1 if the routine times out. 
//******************************************************************************
 
byte	SPI_Get_Byte(void)
{

	char	Data[2];                                     // Data read from USART
    	
	// Disable general interrupts

   //INTCONbits.GIE = 0;                                    // Disable all interrupts

	Data[1] = 0;
    
   // Get next valid data character from the serial input ring buffer
       
	// Check for and filter out any CR characters that indicate the end of serial input

   while (SPI_Rx_Buffer[SPI_Start_Buffer] == CR)
	{
		SPI_Start_Buffer++;
      SPI_Start_Buffer = SPI_Start_Buffer & (SPI_BUFFER_SIZE - 1);    // Wrap the StartBuffer pointer if necessar

 	}

   //Data = SPI_Rx_Buffer[SPI_Start_Buffer];
   Data[0] = SPI_Rx_Buffer[SPI_Start_Buffer];

   SPI_Start_Buffer++;
   SPI_Start_Buffer = SPI_Start_Buffer & (SPI_BUFFER_SIZE - 1);    // Wrap the StartBuffer pointer if necessary

 

   // Enable general interrupts

   //INTCONbits.GIE = 1;                                    // Enable all interrupts
 

   return (Data[0]);
}
 
/********************************************************************
*     Function Name:    ReadSPIByte                                 *
*     Return Value:     contents of SSPBUF register                 *
*     Parameters:       void                                        *
*     Description:      Read single byte from SPI bus (Slave).      *
********************************************************************/
unsigned char ReadSPIByte( void )
{
         
  if ( SSPCON1bits.SSPOV )        		// test if new byte received before previously 
                                    	// received byte has been read.
  {
  	SSPCON1bits.SSPOV = 0;				// Clear the SSPOV flag
   	return ( -1 );              		// if SSPOV bit was set return negative #
  }
 
  return ( SSPBUF );              		// return with byte read 
}

/********************************************************************
*     Function Name:    WriteSPIByte                                *
*     Return Value:     Status byte for WCOL detection.             *
*     Parameters:       Single data byte for SPI bus.               *
*     Description:      This routine writes a single byte to the    * 
*                       SPI bus (Slave).                            *
********************************************************************/
unsigned char WriteSPIByte( unsigned char data_out )
{
  SSPCON1bits.WCOL = 0;	   			// Clear the WCOL flag.

  SSPBUF = data_out;           		// write byte to SSPBUF register
 
  if ( SSPCON1bits.WCOL )        	// test if write collision occurred
  {
   	return ( -1 );              	// if WCOL bit is set return negative #
  }
/*
  else
  {
    while( !SSPSTATbits.BF );  // wait until bus cycle complete 
  }
*/ 
  return ( 0 );                // if WCOL bit is not set return non-negative#
}

/********************************************************************
*     Function Name:    ReadSPIString                               *
*     Return Value:     void                                        *
*     Parameters:       address of read string storage location and *
*                       length of string bytes to read              *
*     Description:      This routine reads a string from the SPI    *
*                       bus (Slave).  The number of bytes to read   *
*                       is determined by parameter 'length'.        *
********************************************************************/
void ReadSPIString( unsigned char *rdptr, unsigned char length )
{
  while ( length )                		// stay in loop until length = 0
  {
    *rdptr++ = ReadSPIByte();         	// read a single byte
    length--;                     		// reduce string length count by 1
  }
}

/********************************************************************
*     Function Name:    WriteSPIString                              *
*     Return Value:     void                                        *
*     Parameters:       address of write string storage location    *
*     Description:      This routine writes a string to the SPI bus.*  
********************************************************************/
void WriteSPIString( unsigned char *wrptr )
{
  while ( *wrptr )                		// test for string null character
  {
     SSPBUF = *wrptr++;           		// initiate SPI bus cycle
     while( !SSPSTATbits.BF );    		// wait until 'BF' bit is set
  }
}
 
