//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*     Name:  dcmotor.h                                                         *
//*   Author:  Daniel F. Ramirez                                                 *
//*     Date:  01/27/2003                                                        *
//*                                                                              *
//*     This is an embedded DC motor controller that uses Pulse Width Modulation * 
//* (PWM), that is used to control DC motors. The DC motor controller allows     *
//* the operator to send motion control commands from a Master Controller, PC or * 
//* laptop host.  It receives DC motor commands from the host (PC or laptop)     *
//* using the serial port, it then computes the latest PWM command to send to    *
//* the motor and proceeds in turn to send the PWM motor commands to the         *
//* selected DC motor. The controller will also report encoder counts to the     *
//* host for position, speed and direction information using the                 *
//* tachometer/encoder hardware.  Other functions performed by this application  *
//* include sending velocity profiles (ramp, trapezoid, sine) to the motors for  *
//* motion trajectory control.                                                   * 
//*                                                                              *
//*      A communication interface is provided that allows the host or SPI or    *
//* I2C Master to request individual sensor readings, average sensor readings,   *
//* calibrating a sensor or perform a sensor processing algorithm using command  *
//* message packets.  The commands available are described below.  This header   *
//* file provides the common Master and Slave message formats and descriptors.   *                                                             *   
//*                                                                              *
//* 	  Calibration of motor speeds can be accomplished by setting the         *
//* Calibration flag.  The calibration process includes using two potentiometers *
//* connected to PIC analog inputs AN0 and AN1, so that the speed of each DC     *
//* motor may be varied accordingly.  The actual motor speed may be determined   *
//* using optical encoders.  The PIC has the capability to capture the encoder   *
//* counts using the CCP1 and CCP2 capture registers and also by using the PortB *
//* interrupt source.  These counts may then be converted to RPM(s) and displayed*
//* as the motor's true current speed for calibration.                           *
//*                                                                              *
//*     Future upgrades to this application will include implementing a Fuzzy    *  
//* logic DC motor control algorithm for use in advanced robots.  Providing for  *
//* control of up to 16 DC motors, by using 2 additional multiplexers in order   *
//* to save I/O bits or by using available I/O bits available to the PIC18C452.  *  
//* Another important upgrade to this application includes using the PIC Port B  *
//* interrupts for an emergency stop function.  The high priority ISR will       *
//* invoke the "Z;" reset motor command. Both the serial interface and the SPI   *
//* interface are supported by defining the approprate compiler flag.            *                                                 
//*                                                                              *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements an SPI master and slave message formats used *
//* by spimast.c, spicli.c, spislave.c, and spiserv.c applications.  This file is*
//* included by each of these applications so that they may communicate over the *
//* SPI bus using common message formats.                                        *                                                        *
//*                                                                              *
//*     This SPI slave controller can be used with a PC host or any SPI master,  *
//* in order to  give any PC the capability to serve as an SPI controller.       * 
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each SPI device has its own unique address and only listens   *
//* to commands associated with it.                                              *
//*                                                                              *
//*     Future upgrades to this application include: Implementing large          *
//* CRC and Checksum capabilities along with large message definitions for       *
//* various applications.                                                        *
//*                                                                              *
//* Usage:                                                                       *
//*	The following DC motor Controller command messages are available:            *
//*                                                                              * 
//* 	Cmm       - Calibrate motor speed for motor mm range (00..01). This      *
//*                 command is used to vary the motor speed and direction using  *
//*                 two potentiometers with the blades connected to AN0 and AN1. *
//*                 The true motor speed (RPM) is determined from the counts     *
//*                 received from the optical encoders (if used) and displayed.  *
//*                                                                              * 
//* 	Dmm,xxxx  - Send PWM duty command range xxxx (0000..FFFF) to motor       *
//*                 mm range (00..01). This command is used to vary              *
//*                 the motor speed and direction.                               *
//*                                                                              *
//* 	Emm       - Read DC motor encoders for position, speed and               *
//*                 acceleration and display the information for motor           *
//*                 mm range (00..01).                                           *
//*                                                                              *
//*     Fxx       - Select filter xx, where the filter ID                        *
//*                 range (00..FF) is in a 2 digit hex byte format,              *
//*                 where 00 => No Filter, 01 => Average Filter,                 *
//*                 02 => Kalman Filter2, 03 => Filter3,...                      *
//*                                                                              *
//* 	Gmm       - Get new motor commands from the current POT readings and     *
//*                 display them for the specific motor mm range (00..01).  This *
//*                 mode will remain in effect until the Hold command is issued. *
//*                                                                              *
//* 	Hmm       - Hold current POT reading and use it for the next PWM duty    *
//*                 command range (0000..FFFF) to motor                          *
//*                 mm range (00..01). This command is used to vary              * 
//*                 the motor speed and direction.                               *
//*                                                                              *
//*     Nxxxx     - Accept total number of samples, XXXX to be read from         *
//*                 selected Potentiometer, where xxxx is in 4 digit             *
//*                 hex byte format  range (00..FFFF)                            *
//*                                                                              *
//* 	Pmm,xx    - Send PWM period command range xx (00..FF) to motor           *
//*                 mm range (00..01). This command is used to vary the          *
//*                 PWM update frequency.                                        *
//*                                                                              *
//* 	Smm       - Read DC motor status including temperature, direction,       *
//*                 speed (RPM read from optical encoders and power state        * 
//*                 (ON/OFF) from motor mm range (00..01).                       *
//*                                                                              *
//* 	Tmm,xx    - Send trajectory profile xx range (00..FF) to motor mm        *
//*                 range (00..01).                                              *
//*                                                                              *
//* 	Vmm,xxxx  - Send velocity command range xxxx (0000..FFFF) to motor       *
//*                 mm range (00..01). This command is used to vary              * 
//*                 the motor speed and direction.                               *
//*                                                                              *
//* 	Z;        - Reset the DC motor Controller (brake the motor).  This       *
//*                 command can be used to implement an emergency stop           *
//*                 function.                                                    *
//*                                                                              *
//*                                                                              * 
//*   In order to use PWM2, the CCP2/RB3 pin fuse must be selected when          *
//*   programming the PIC.                                                       *                                                                                                       
//*                                                                              *
//* 	Example:                                                                 *
//*   		Send the following command string to the DC motor controller         *
//* 	to set PWM period to 0xFF using the string:                              *
//*       "P01,FF;"                                                              *
//* 	Where:                                                                   *
//*      	P - Is the motor command to change the PWM period.                   *
//*     	01 - Is the motor ID.                                                *
//*     	FF - Is the 2 character hex PWM period command.                      *
//*                                                                              *
//********************************************************************************
   
#ifndef DCMOTOR_LOADED  /* prevent multiple loading */
#define DCMOTOR_LOADED


#include "flags.h"                                    // Global flags used for MPSIM debugging 
#include "stdio.h"                                    // I/O support functions  
#include "network.h"                                  	// Include support for networking
 
 
// Common Constants

#define INPUT 1											// Pin input 
#define OUTPUT 0										// Pin output

#define ENABLE 0										// Enable the Slave device
#define DISABLE 1										// Disable the Slave device
#define TRUE  1                                         // Boolean TRUE value
#define FALSE 0                                         // Boolean FALSE value

#define ON  1                                         	//  read operation
#define OFF 0                                         	// SPI write operation

//#define READ  1                                       // SPI read operation
//#define WRITE 0                                       // SPI write operation

#define MAX_MOTORS 28                                   // Maximum number of DC motors connected to the controller

#define MOTOR_0 0x00                           			  // Motor #0 ID
#define MOTOR_1 0x01 											  // Motor #1 ID

#define MOTOR_0 0x00                           			  // Motor #0 ID
#define MOTOR_1 0x01 											  // Motor #1 ID

#define LEFT_MOTOR      0x00                            // Left Motor ID
#define RIGHT_MOTOR     0x01 									  // Right Motor ID

// motor enumeration types

typedef enum   {CW, CCW} Direction_T;                   // Motor Direction (CW, CCW)
 

//**********************************************************************************
// Define messages for each type of command that is sent by the SPI Master and 
// recognized by the selected SPI Slave.
//**********************************************************************************

//**********************************************************************************
//* 	Cmm       - Calibrate motor speed message for motor mm range (00..01). This*
//*                 command is used to vary the motor speed and direction using    *
//*                 two potentiometers with the blades connected to AN0 and AN1.   *
//*                 The true motor speed (RPM) is determined from the counts       *
//*                 received from the optical encoders (if used) and displayed.    *
//**********************************************************************************
typedef struct calibrate_motor_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;         	// Calibrate Command to SPI Slave (see enumeration
                                	// declaration above.)  
 
 	byte MotorID; 					// Selected motor range (0..MAX_MOTORS)  
    byte Filler[28];				// Padding 
    
} CALIBRATE_MOTOR_TYPE;

//**********************************************************************************
//* 	Dmm,xxxx  - Send PWM duty command message range xxxx (0000..FFFF) to motor *
//*                 mm range (00..01). This command is used to vary                *
//*                 the motor speed and direction.                                 *
//**********************************************************************************
typedef struct duty_command_type
{
  	word MessageID;    				// Message ID range (0000 - FFFF) Hex 

  	COMMAND_TYPE Command;         	// Duty Command to SPI Slave (see enumeration
                                	// declaration above.)  
 
 	byte NumberOfMotors; 			// Total number of motors range (0..MAX_MOTORS)  
 	word Duty[MAX_MOTORS]; 			// Duty Cycle range (0000 - FFFF) Hex              	
    //byte Filler[26];				// Padding 

} DUTY_COMMAND_TYPE;

//**********************************************************************************
//* 	Gmm       - Get new motor commands message.  Used to get the current POT   *
//*                 readings and display them for the specific motor mm range      *
//*                 (00..01).  This mode will remain in effect until the Hold      *
//*                 command is issued.                                             *
//**********************************************************************************
typedef struct get_pot_command_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;         	// Get new motor commands from the current POT readings
  	                                // Command (see enumeration declaration above.)  
 
 	byte MotorID; 					// Selected motor range (0..MAX_MOTORS)  
    byte Filler[28];				// Padding 
    
} GET_POT_COMMAND_TYPE;

//**********************************************************************************
//*                                                                                *
//* 	Hmm       - Hold current POT reading message. It is used for the next PWM  *
//*                 duty command range (0000..FFFF) to motor                       *
//*                 mm range (00..01). This command is used to vary                * 
//*                 the motor speed and direction.                                 *
//**********************************************************************************
typedef struct hold_pot_type
{
  	word MessageID;    				// Message ID (0000 - FFFF) Hex

  	COMMAND_TYPE Command;         	// Get new motor commands from the current POT readings
  	                                // Command (see enumeration declaration above.)  
  
 	byte MotorID; 					// Selected motor range (0..MAX_MOTORS)  
    byte Filler[28];				// Padding 
    
} HOLD_POT_TYPE;


/******************************************************************************
 * Define union of unformatted message and command message structures in order to 
 * overlay the memory. 
 ******************************************************************************/

typedef union message_union_type
{
    CALIBRATE_MOTOR_TYPE CalibrateCommand;						// Calibrate motor command message
	DUTY_COMMAND_TYPE DutyCommand;								// Duty command message
	GET_POT_COMMAND_TYPE GetPotCommand;							// Get Pot command message
	HOLD_POT_TYPE HoldPotCommand;								// Hold pot command message
	SAMPLE_COMMAND_TYPE SampleCommand;                          // Sample command message	
	PERIOD_COMMAND_TYPE PeriodCommand;					 		// Period command message
	GET_STATUS_COMMAND_TYPE GetStatusCommand;					// Get status message
	RESET_COMMAND_TYPE ResetCommand;					 		// Reset command message

	COMMAND_MESSAGE_TYPE SelectedCommand;						// Selected command	
	MESSAGE_TYPE Message;										// Unformatted message
} MESSAGE_UNION_TYPE;

#endif

