//********************************************************************************                                                                                      ;
//*        Name:  sensors.h                                                      *
//*      Author:  Daniel F. Ramirez                                              *  
//*        Date:  02/20/2003                                                     *     
//*                                                                              *
//*                                                                              *  
//* Description:                                                                 *  
//*     This application gathers and processes sensor data collected from up to  *
//* 8 A/D channels and sends the data back to the Host, I2C or SPI Master        *
//* Controller using the Microchip PIC18C452 microcontroller.  It can be used    *
//* to collect and display data from any light sensor (phototransistor or CDS    *
//*  cell), IR proximity sensor or temperature sensor for robotic applications.  *
//* It also provides access to the digital I/O ports (A-E), which is used for    *
//* reading switches, status words and binary data.  Port B interrupts may be    *
//* utilized for interfacing external devices such as A/D's, keypads and         *
//* switches.  The device can get the PIC's attention using a Level or edge      *
//* triggered interrupt.                                                         *
//*                                                                              *
//*      A communication interface is provided that allows the host or SPI or    *
//* I2C Master to request individual sensor readings, average sensor readings,   *
//* calibrating a sensor or perform a sensor processing algorithm using command  *
//* message packets.  The commands available are described below.  This header   *
//* file provides the common Master and Slave message formats and descriptors.   *                                                             *   
//*                                                                              *
//*      Possible sensor algorithms include: Fuzzy Logic, Sensor Fusion, AI      *  
//* algorithms, Neural Networks, Mean, RMS, STD, Variance, Median, Filters,      *
//* Least Squares Fit, and Gaussian Polynomial Curve Fitting.                    * 
//*                                                                              *
//*      Possible filter algorithms include: IIR, FIR, digital filters,          *
//* averaging filters, median filters and Kalman Filters.                        *         
//*                                                                              *
//*                                                                              *
//*		The On-Chip calibration process includes finding the minimum, maximum    *
//* and average light sensor readings so that the sensor data can be             *
//* characterized and scaled.  These values are later stored to EEPROM or        *
//* scratch pad RAM available on the PIC, so that they may be accessed by other  *
//* applications which need them.   This calibration process may also be         *
//* performed from a laptop or PC Workstation in order to save EEPROM/FLASH      *
//* program space on the PIC.                                                    *
//*                                                                              *
//* 		The Controller allows Each port (A-E) on the PIC processor to be     *
//* be read (digitial I/O) and count Port B interrupts which indicate that a     *
//* digital switch has fired off will also be reported.  New sensor history      *
//* functions that compute sensor statistics  using previously stored data in a  *
//* ring buffer will also be used for data logging and data analysis.            *
//* The switch states for Port B are considered to be a pseudo sensor with       *
//* ID = 0 or 8 ???.  No calibration or scaling is required for this sensor.     *
//*                                                                              *
//*     The sensor calibration data is collected by HYPERTERMINAL, PCPLUS or VB  *
//* or VC++ application on the host, and then may be plotted using Matlab, Excel *
//* or the Stamp Plot Lite program which also runs on the host PC. Plot commands *
//* are sent from the PIC to the host using specific debug commands (printf).    *
//*                                                                              *
//*                                                                              *
//********************************************************************************                                                                                      ;
#ifndef SENSORS_LOADED  /* prevent multiple loading */
#define SENSORS_LOADED

//#include <stdlib.h>
//#include <conio.h>
#include <math.h>
#include <string.h>
#include "stdio.h"
 
#include "sensor.h"
 
// Sensor class

// Define global constants here

#define TRUE  1                                            	// Boolean TRUE value
#define FALSE 0                                            	// Boolean FALSE value
 
#define NumberOfSensors 8                                  	// Total number of sensors (A/D channels)
 
#define NumberOfReadings 20                                	// Total number of samples read while averaging                     

const int NumberCoefficients  = 4;                // Number of polynomial coefficients used to calibrate
															     // temperature sensor.

 
/* declare global types here */

// These data structures are used by the various sensor drivers.
/*
rom enum SensorTypes {POSITION_SENSOR, ENCODER, OPTICAL_ENCODER, CCD, MICROPHONE, IR_SENSOR,
						PHOTOTRANSISTOR,  ANALOG_TO_DIGITAL_CONVERTER,
						CDS, PROXIMITY_SENSOR, SONAR,MAGNETIC_PICKUP_COIL, CAPACITANCE_SENSOR,
						WFS_SENSOR, LVDT_SENSOR, PIZZEO_ELECTRIC_TRANSDUCER, PRESSURE_SENSOR,
						SONIC_TRANSDUCER, TEMPERATURE_SENSOR, THERMOCOUPLE, THERMISTOR, VOLTAGE_SENSOR,
						CURRENT_SENSOR, VELOCITY_SENSOR, ACCELERATION_SENSOR, FORCE_SENSOR,
						STRAIN_GAUGE, CONTACT_SWITCH, STATUS_BIT, DATA_COLLECTION };

rom enum StatusTypes { OFF_LINE, ON_LINE, START, READING, FINISHED };


const int TotalNumberOfSensorTypes = 29;
const int TotalNumberStatusTypes = 5;

 

typedef float CoefficientType[NumberCoefficients];
   

//******************************************************************************
//* POLYVAL Polynomial evaluation.
//*        If p is a vector of length d+1 whose elements are the coefficients
//*        of a polynomial, then y = POLYVAL(p,x) is the value of the
//*        polynomial evaluated at x.
//*            y = p(1)*x^d + p(2)*x^(d-1) + ... + p(d)*x + p(d+1)
//*  For Y = C0 + C1*X.. Cn*X^n, n =  3  coeffs C(0) to Cn:
//******************************************************************************

float PolyVal(float x, float C[]);

//******************************************************************************
//* ConvertCelsiusToFarenheight  - Convert Degrees Celsius to Degrees Farenheight
//******************************************************************************
float ConvertCelsiusToFarenheight(float C);

*/


//******************************************************************************
//* Define structure for Sensor type here
//******************************************************************************


// This record type contains the sensor packet with scaled sensor values to be
// used by the Record and PLay functions.

typedef struct 
{
   //byte PortBStatus;													 	 // Port B status
	float CurrentSensorValue[NumberOfSensors];                // Current sensor reading (scaled)
 
} SensorRecord_T;

// This record type contains information on each sensor, including its Channel ID,
// mininmum, maximum nominal and average value.  It also includes the window
// parameters used to scale each sensor.

typedef struct  
{

// Define Sensor attributes here

	int Channel;                                 // Sensor Channel ID   

  //SensorTypes SensorType;                     // Type of Sensor  
  //StatusTypes Status;                         // Sensor status
   
  word RawSensorValue;                          // Raw Sensor reading  
  //float AverageSensorValue;                     // Average sensor reading
  //float SensorGain;                             // Sensor gain  
  //float SensorOffset;                           // Sensor offset
  word MinSensorValue;                          // Minimum value for scaling sensor reading
  word MaxSensorValue;                          // Maximum value for scaling sensor reading
  word WindowMin;                               // Window minimum used for scaling sensor reading
  word WindowMax;                               // Window maximum used for scaling sensor reading
 
  // Declare other sensor attributes here
  
  //byte Resolution;                            // Resolution (in bits)
  //float NominalSensorValue;                   // Nominal value for sensor reading
  //int MaxRawSensorValue;                      // Maximum raw sensor reading (in bits)
  //float SensorRange;                          // Sensor range
  //float Sensitivity;                          // Sensor sensitivity
  //float Precision;	                           // Sensor precision
  //float ConversionTime;                       // Time for one conversion
  //float ConversionRate;                       // Conversion rate in conversions per second
  //int LogFlag;                                // Enables logging of sensor data to selected																// file in MATLAB format.
  //char LogFileName[80];                       // Name of sensor log data file
  //FILE * LogFile;                             // File variable used for logging sensor data

  // Define Generic Sensor object's attributes here

  //int Delay;                                 // Delay between consecutive sensor reads

  // Declare array to contain polynomial coefficients generated by the gausfit program
  // for each temperature sensor.

  // CoefficientType C;

} Sensor_T;     


// Define function protypes here     

void	InitializeSensor(void);						        // Initialize sensor controller variables
void	Collect(Sensor_T SensorTable[NumberOfSensors]);  	// Collect data from sensors
void   	SendData(Sensor_T SensorTable[NumberOfSensors]);	// Send calibration results back to host
int     ReadSensor(byte SensorID);						   	// Read a sensor value from the A/D converter
void	CalibrateSensor(byte SensorID);                    	// Calibrate a sensor
float   AverageSensor(byte SensorID);                      	// Average a sensor 
void 	DisplayIRSensors(void);								// Display the IR proximity sensor readings 

//char   keypad(void);                                      // Scan the keypad
//int    button(int Pin);                                   // Debounce the selected switch connected to Pin
void     ReadSwitchStates(byte *SwitchStatus);              // Read the debounced Port B change on interrupt bits
void     ScaleSensorReading(byte SensorID);                 // Scale the sensor readings
void     AvoidBehavior(void);                               // Avoid behavior algorithm
void     Record(byte SensorID);                             // Log sensor data using specified sample period     
                                                            // period or number of samples
void     Play(byte SensorID);                               // Dump sensor data to Serial RS-232 using specified sample period or
                                                            // number of samples 
void     StoreSensorReadings(byte SensorID, word EEAddress); // Store sensor readings to EEPROM      

// Sensor Message Processing

//**********************************************************************************
//* ProcessRecordCommand - Process the Sensor Controller Record Command message used 
//* record sensor data to the serial EEPROM for later playback.
//**********************************************************************************
byte ProcessRecordCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessPlayCommand - Process the Sensor Controller Play Command message used
//* to play back data previously recorded to serial EEPROM.
//**********************************************************************************
byte ProcessPlayCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessCalibrateSensorCommand -  Process the Sensor Controller Calibrate Command 
//* message used to calibrate selected sensors.
//********************************************************************************** 
byte ProcessCalibrateSensorCommand(byte TheSensorID);
 
//**********************************************************************************
//* ProcessGetInterruptCountsCommand -  Process the Get Interrupts Count Command  
//* message by returning the Port B interrupt counts for RB4..RB7.  
//**********************************************************************************
byte ProcessGetInterruptCountsCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessSelectAlgorithmCommand -  Process the Select Algorithm Command message. 
//* Return a status byte indicating the outcome of the sensor calibration process.
//**********************************************************************************
byte ProcessSelectAlgorithmCommand(byte TheSensorID);  

 //**********************************************************************************
//* ProcessSelectFilterCommand -  Process the Select Filter Command message.
//* Return a status byte indicating the outcome of the Filter select processing.
//**********************************************************************************
byte ProcessSelectFilterCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessReadPortCommand -  Process the Read Port Command message.  This function 
//* returns the 8-Bit values read from the PIC's digital I/O Ports.  Return a status byte
//* indicating the outcome of the Read Port command.
//**********************************************************************************
byte ProcessReadPortCommand(byte TheSensorID);  

//**********************************************************************************
//* ProcessWritePortCommand -  Process the Write Port Command message.  This function 
//* writes an 8-Bit value to the selected digital I/O port.  Return a status byte 
//* indicating the outcome of the Write Port Command.
//**********************************************************************************
byte ProcessWritePortCommand(byte TheSensorID);
  
//**********************************************************************************
//* ProcessNumberSamplesCommand - Process Get number of samples command message. 
//* Return a status byte indicating that the Number of Samples was accepted.
//**********************************************************************************
byte ProcessNumberSamplesCommand(word TheNumberOfSamples);

//**********************************************************************************
//* ProcessGetSensorCalibrationCommand -  Process the Get Sensor Calibration Command 
//* message.  This function returns important sensor calibration back to the host.
//**********************************************************************************
byte ProcessGetSensorCalibrationCommand(byte TheSensorID);
 
//**********************************************************************************
//* ProcessSetSensorWindowCommand -  Process the Set Sensor Window Command message. 
//* This function sets the sensor scaling window parameters that are used to convert raw
//* sensor values to floating point scaled values.  
//**********************************************************************************
byte ProcessSetSensorWindowCommand(byte TheSensorID); 

//**********************************************************************************
//* ProcessGetRawSensorDataCommand -  Process the Get Raw Sensor Data command message. 
//* This function gets the selected raw sensor data value and returns it to the host. 
//**********************************************************************************
byte ProcessGetRawSensorDataCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessGetScaledSensorDataCommand -  Process the Get Scaled Sensor Data command message. 
//* This function gets the selected scaled sensor data value and returns it to the host. 
//**********************************************************************************
byte ProcessGetScaledSensorDataCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessTestCommand -  Process the Test Command message. This function gets the 
//* selected scaled sensor data value and returns it to the host. 
//**********************************************************************************
byte ProcessTestCommand(byte TheSensorID);

//**********************************************************************************
//* ProcessGetStatusCommand - Process the the SPI Slave Controller Get Status 
//* command.  This command returns the latest command execution status from the 
//* ISR in order not to interfere with the current command being executed in the 
//* mainline.
//**********************************************************************************
void ProcessGetStatusCommand(void); 

//**********************************************************************************
//* ProcessResetCommand - Process the Reset the SPI Slave Controller.  This command 
//* resets the selected controller.  It may be used to implement an emergency stop 
//* function using the PIC's watchdog timer. 
//**********************************************************************************
void ProcessResetCommand(byte TheSensorID);
 
#endif


