//********************************************************************************
//*                                                                              *
//* Description:                                                                 *
//*                                                                              *
//*        Name:  spislave.c                                                     *
//*      Author:  Daniel F. Ramirez                                              *
//*        Date:  12/06/2002                                                     *
//*                                                                              *
//* Description:                                                                 *
//*     This application implements an SPI Slave Co-Processor controller that    *
//* emulates Microchip's 24LCXX SPI Memory chip interface and acts as a smart    *
//* peripheral, by using the SPI hardware module contained in the PIC18C452      *  
//* micro-controller.  In addition it can act as an SPI network server to an     *
//* SPI Client or function as an SPI based RPN Floating Point co-processor.  In  *
//* the simplest mode of operation, it simulates writing to the EEPROM and       *
//* reading it back under control from the Master SPI controller.  Included is a *
//* diagnostic function that will write a ramp to the EEPROM using SPI and read  *
//* it back to the Host PC via the USART when used with the spimast.c            *
//* application. A more sophisticated SPI Networking mode is also available to   *
//* applications that require message passing with CRC or checksum.  The common  *
//* message format between the SPI Master and SPI Slave is defined in the        *
//* file spimsg.h.  SPI transmit and receive buffers are available to process    *
//* messages to/from the Master controller.                                      *                         
//*                                                                              *
//*     This SPI slave controller can be used with a PC host or any SPI master,  *
//* in order to  give any PC the capability to serve as an SPI controller.       * 
//* It can also be used for robotic applications where multiple controllers and  *
//* SPI devices are used, with a minimum of data and control lines needed to     *
//* connect them.  Each SPI device has its own unique slave select enable line   *
//* (SS_BAR) and only listens to the SPI Master commands when SS_BAR is cleared. *  
//*                                                                              *
//*    This application emulates a Microchip 24LCXX EEPROM that has 8 registers  *
//* accessed as sub-addresses 1:8 modulo 7.  Sub-address 0 is used as a data     *
//* channel.  All the read and write data modes available for this device        *
//* including: Byte Write, Page Write, Current Address Read, Random Read and     *
//* Sequential Read are emulated by this application.  As far as the Master SPI  * 
//* controller knows, it is talking to a real EEPROM instead of another PIC.     *
//*                                                                              *
//*                                                                              * 
//*     The SPI message format used by this application is for the master to     *
//* send commands to the Digital Pot SPI slave device.  This translates to the   *
//* following sequence of operations for the Master:                             *                    
//*                                                                              *
//*  1) The SPI Co-Processors Command Bytes: X|X|C1|C0|X|X|P1|P0                 *
//*                                                                              *
//*                                                                              *
//*  2) Data is latched in on the rising edge of the SCK.                        *
//*                                                                              *
//*  3) Data is clocked out the SO pin after the falling edge of SCK.            *
//*                                                                              *
//*  4) The Clock Iddle is high.                                                 *
//*                                                                              *
//*  5) The SPI Clock speed is FOSC/16                                           *
//*                                                                              *
//*  6) The Reset Pin, RS_BAR will automatically set all potentiometer data      *
//*     latches to mid scale (Code 80h).                                         *
//*                                                                              *
//*  7) The Shutdown Pin, SHDN_BAR when held low will (provided that the pin is  *
//*     held low for at least 150 ns and CS_BAR is high, will cause the          *
//*     application circuit to into a power saver mode.                          *
//*                                                                              *
//*  8) The device when powered up for the first time, will automatically set    *
//*     reset all potentiometer data latches to mid scale (Code 80h).  A power   *
//*     on will insure that the device powers up in this known state             *
//*                                                                              *
//*  9) This device can operate in SPI Mode 0,0, in which the clock idles in the *
//*     low state.                                                               *
//*                                                                              *
//*  10) The following SPI data transfers are available:                         *
//*                                                                              *
//*    		Master sends data byte  -  Slave sends dummy data byte               *                                                                        
//*    		Master sends data byte  -  Slave sends data byte                     *                                                                  
//*    		Master sends dummy data byte  -  Slave sends data byte               *                                                                         
//*                                                                              *
//*                                                                              *
//* Note:                                                                        *
//*		The SPI networking is handled using a simple data packet messaging       *
//* protocol.  Each message includes an SPI source address field (Master), an SPI*
//* destination address field, the actual data and a CRC or checksum field.  Data*
//* packets that are determined to be corrupted will be retransmitted up to a    *
//* specified number of times.                                                   *
//*                                                                              * 
//*                                                                              *
//*      Future upgrades to this application include: the ability to communicate * 
//* with other SPI slave devices such as Microchip MPC3002 A/D converter, the    *
//* Maxim MAX7219 LED Driver, the Microchip SPI Serial EEPROMS such as the       *
//* 93XXXX Series and the ISD 33270 Voice Recorder IC.  It may also be used to   *
//* communicate as a master or a slave for applications requiring that dual      *
//* capability.  Other upgrades include using SPI to implement the C printf      *
//* function to be used for a "cheap" embedded debugger similar to JTAG used by  *
//* high end micro-processors. Explore using communication protocols and error   *
//* detection/correction algorithms such as parity, CRC and various other codes  *
//* for robust operation and fault tolerance. TMR and N-Module redundancy could  *
//* also be investigated for fault tolerant systems that work in hazardous       *
//* environments. The same SPI networking functions may also be used with the    *
//* asynchronous serial RS-232 interface and the I2C interface.                  *                         
//*                                                                              *
//*                                                                              *
//********************************************************************************
  
   
// Include file headers here

#include "flags.h"                                      // Global flags used for MPSIM debugging 
  
#ifdef PIC18C252     
#include <p18c252.h>					// Register definitions for PIC18C252
#endif

#ifdef PIC18C452     
#include <p18c452.h>					// Register definitions for PIC18C452
#endif

#ifdef PIC18F452     
#include <p18f452.h>					// Register definitions for PIC18C452
#endif
  
#ifdef PIC18F8720     
#include <p18f8720.h>
#endif
 
#ifdef PIC18F6620     
#include <p18f6620.h>
#endif
 
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <spi.h>                                        // SPI library functions
#include <pwm.h>                                        // PWM library functions
#include <timers.h>                                     // Timer library functions
#include <portb.h>                                      // PORTB library functions                                 
#include <usart.h>                                      // USART library functions
#include <delays.h>                                     // High resolution delay functions
//#include "portutil.h"                                 // Port utility functions
#include "timeutil.h"
#include "stdio.h"                                      // Serial I/O support functions
#include "network.h"                                   	// SPI Networking Messages
#include "spislave.h"                                  	// SPI Slave services

//********************************************************************************
// Constant Definitions
//********************************************************************************

#define TEST                                          // Flag used for debug I/O
//#define USART_TEST                                    // Test serial I/O with this application

//#define SPI_TEST_0                                 	// Select the SPI Master Controller test #0
//#define SPI_TEST_1                                 	// Select the SPI Master Controller test #1
//#define SPI_TEST_2                                 	// Select the SPI Master Controller test #2
//#define SPI_TEST_3                                 	// Select the SPI Master Controller test #3


 
//#ifdef SPI_BUFFER_INTERFACE  							// SPI Buffered I/O (using SPI interrupt)

// Constants

// Port A

#define SS			PORTAbits.RA5						// SPI Slave Co-processor Select
#define SS_DIR		TRISAbits.TRISA5					// SPI Slave Co-processor Select Direction

// Port B

#define TRACE0		PORTBbits.RB0						// Trace bit 0 for logic analyzer
#define TRACE0_DIR	TRISBbits.TRISB0

// Port C

#define SPI_SCL_DIR TRISCbits.TRISC3 					// SPI Clock Direction (Slave)
#define SPI_SI_DIR TRISCbits.TRISC4 					// SPI Input Direction (Slave)
#define SPI_SO_DIR TRISCbits.TRISC5 					// SPI Output Direction (Slave)
 



// Variables

int Status = 0;											// Status return code

byte ControllerState; 		// State of selected slave controller, range (0..255) 
											// 		0 => NO_STATES		No States    
											//		1 => INITIALIZED	The selected slave contoller is Initialzed 
											//		2 => DONE 	 	 	The selected slave contoller is done processing current command
											// 		3 => BUSY 	 		The selected slave contoller is busy processing current command 
											// 		4 => ERROR 	 		The selected slave contoller encountered an error processing the current command 

byte ControllerErrorCode; 	// Error code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_ERRORS 		No error codes  
											// 		1 => FAILURE 	 	Failed to complete command
											//  	2 => SUCCESS  	 	Completed command successfully 
 
byte ControllerDiagnosticCode; 	// Diagnostic message code returned by the selected slave controller, range (0..255) 
											// 		0 => NO_DIAGNOSTICS 	No Diagnostic messages
											//  	1 => DIAGNOSTIC 1  	Diagnostic message 2 
											// 		2 => DIAGNOSTIC 2 	Diagnostic message 1
 
// Pointers and arrays

// Declare SPI Slave Buffer variables here

byte SPI_Rx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Receive Buffer
byte SPI_Tx_Buffer[SPI_BUFFER_SIZE];                  	// SPI Transmit Buffer
byte SPI_Start_Buffer=0;                       			// Start pointer for ring buffer
byte SPI_End_Buffer = 0;								// SPI End Buffer Index
byte SPI_Rx_Flag = FALSE;                        		// Indicates that data was received into the ring buffer
word SPI_Timeout = 0;                            		// SPI I/O Timeout counter
byte SPI_Timeout_Flag;                           		// SPI Timeout flag
word SPI_Number_Characters = 0;						 	// SPI Number of characters in buffer
int SPI_Communications_Error;                    		// SPI I/O error flag
                                                 		// -1 = SPI Input Bufer overrun error
                                                 		// -2 = SPI Communications timeout error
                                                 		// -3 = SPI buffer empty error

int SPI_Interrupt_Count = 0;                   			// Number of MSSP interrupts
int SPI_Data_In = 0;									// Data byte read from the SPI Master
int SPI_Data_Out = 0;									// Data byte sent to the SPI Master
int SPI_Status;											// SPI Status returned from routines

int Dummy;												// Dummy Data byte read from the SPI bus

 
 
//#endif

static char TheCommand;									// The Command byte read from the SPI bus

byte AddressPointer;                         	 		// SPI Device Address pointer used to simulate serial
                                                 		// EEPROM reads and writes
byte EEPromPointer;                          	 		// Pointer to EEPROM array

static int Index = 0;                            		// SPI Ring Buffer index

static byte CurrentState = 0;                    		// Current SPI state variable



// FUNCTION Prototypes

void main(void);
 
#ifdef SPI_SLAVE_TEST

// Interrupt service routines for spislave.c

//********************************************************************************
// Interrupt Code
//********************************************************************************

  
// Designate ServiceInterrupt as an interrupt function and save key registers

#pragma interrupt ServiceInterrupt save = PRODL,PRODH

// Locate ISR handler code at interrupt vector

#pragma code isrcode=0x0008

void            isrhandler(void)                           // This function directs
 // execution to the
{                                                          // actual interrupt code
 
	_asm

    	goto ServiceInterrupt

    	_endasm

}

#pragma code


//********************************************************************************
//* ServiceInterrupt - Process each type of interrupt  (MSSP, USART, INT0..INT2, 
//* and TMR0IF..TMR3IF).
//********************************************************************************
void            ServiceInterrupt(void)
{
	int i;										// Local loop index
	
#ifdef SPI_BYTE_INTERFACE
 	// Interrupt Logic for SPI Slave Byte Interface goes here ...
  
   // Transmit and Receive single bytes over the SPI Bus
   
    if (PIR1bits.SSPIF)                                   // Check for incomming characters from SPI
    {
     
    	// Turn on trace 0
                             
      	TRACE0 = ON;

       	// Read a single byte from the SPI Bus (SSPBUF).
		SPI_Data_In = ReadSPIByte();
		
      	// Convert the byte (character) to upper case      	
      	SPI_Data_Out =  SPI_Data_In + 32;

		// Display value read on on the serial 
		
		printf("Slave: SPI_Data_In = ");
		dec(SPI_Data_In);
 		
		printf("  SPI_Data_Out = ");
		dec(SPI_Data_Out);
		
      	// Write the character to the SPI Bus
	  	SPI_Status = WriteSPIByte(SPI_Data_Out);

		printf(" SPI_Status = ");
		dec(SPI_Status);
 		printf("\r\n");

      	// Read the dummy data that is automatically sent by the SPI Master
      	//Dummy = getcSPI();


  	   	// Clear the BF flag by reading the SSPBUF register

  		//Data = SSPBUF;

      	// Increment SPI Interrupt counter
      	SPI_Interrupt_Count++;

     	// Turn OFF trace 0

      	TRACE0 = OFF;
      	
      	// Clear the SSPIF flag

  		PIR1bits.SSPIF = 0;

    }
/*
    // Use serial I/O interrupt
    
     else
    if (PIR1bits.RCIF)                                     // Check for incomming characters from USART  
    {
		ProcessUSART();                                    // Process USART interrupt here
    } 
*/

#endif
	
#ifdef SPI_BUFFER_INTERFACE
		
	// Interrupt Logic for SPI Slave Buffer Interface goes here ...

    if (PIR1bits.SSPIF)                          // Check for incomming characters from SPI
    {
     
    	// Turn on trace 0
                             
      	TRACE0 = ON;

       	// Read a single byte from the SPI Bus (SSPBUF) into the serial 
       	// input ring buffer.
	
		SPI_Rx_Buffer[SPI_End_Buffer] = ReadSPIByte();
		
        // Send sensor data, telemetry data or status information back to the SPI Master Controller 
       	// message at the same time that a message is received from the SPI Master.
 
	  	SPI_Status = WriteSPIByte(SPI_Rx_Buffer[SPI_End_Buffer]);
 	  	
		// Check for buffer full indicating the end of message

       	if (SPI_End_Buffer ==  (SPI_BUFFER_SIZE - 2))
        {
        	SPI_Rx_Flag = TRUE;				// Set the SPI message Received Data flag
        	
       		//SPI_Number_Characters = 0;
       		
         	// Copy the entire DC Motor Controller message sent from the SPI Master
   			// to local storage for processing.
   			
  			memcpy(&MotorMessage.Message.Data, SPI_Rx_Buffer, sizeof(SPI_Rx_Buffer));  
  			  
/*  			
   			for (i=0; i<SPI_BUFFER_SIZE; i++)
			{
  			
				MotorMessage.Message.Data[i] = SPI_Rx_Buffer[i];
			}
*/     	
        				   			
        }
        
		// Increment buffer index 
 			
		SPI_End_Buffer++;										 			// Increment the SPI Ring Buffer pointer
        SPI_End_Buffer = SPI_End_Buffer % SPI_BUFFER_SIZE;        	 	// Wrap the EndBuffer pointer if necessary
        
      	// Increment SPI Interrupt counter
      	SPI_Interrupt_Count++;
      	
       	// Turn OFF trace 0

      	TRACE0 = OFF;
   	
  	   	// Clear the SSPIF flag

  		PIR1bits.SSPIF = 0;
  
   }   
#endif

#ifdef SPI_NETWORK_INTERFACE
		// Interrupt Logic for SPI Slave Network Interface goes here ...
		
        // Read the message data sent from the SPI Slave
   	   	getsSPI(SPI_Rx_Buffer, sizeof(SPI_Rx_Buffer));
   	   	
   	   	// Copy the buffer and add 10 to each element
   	   	
		for (i=0; i<SPI_BUFFER_SIZE; i++)
		{
 			SPI_Tx_Buffer[i] = SPI_Rx_Buffer[i] + 10;
 		}
 		
 		// Send message back to SPI Master
        putsSPI(SPI_Tx_Buffer);					// Send the message stored in the SPI_Tx_Buffer to the 
                                  				// SPI Slave Co-Processor
 
        //pause(5); 								// Wait some time to allow it to be processed.
         										// We may want handshaking here...
 
   	   	
   	   	// Display the data sent and received 
/*   	   	
		for (i=0; i<SPI_BUFFER_SIZE; i++)
		{
			printf("Slave: i = ");
			dec(i);
			printf("  SPI_Tx_Buffer = ");
			dec(SPI_Tx_Buffer[i]);
			
			printf("  SPI_Rx_Buffer = ");
			dec(SPI_Rx_Buffer[i]);
			
			printf("\r\n");
			
			pause(1000);
		}
*/ 
   	   	
#endif       
        


}





//******************************************************************************
//* main - Main application writes 8-Bit word to 24CXXX SPI serial EEPROM(s).  
//******************************************************************************

void main(void)
{
	int i;								// Local loop index



 	// Configure USART for 20 MHz, 9600 Baud, 8-Bits, No parity, Asynchronous
   
	InitializeUsart(); 

	//printf("SPI Co-Processor Application ...\r\n");


	SetupTimers();                                    // Setup timers and software
                                                      // variables.

	// Check timer code by delaying 5 seconds ...
/*
	printf("Start of 5 second delay...\r\n");

	pause(5000);

	printf("End of 5 second delay. \r\n");
*/

#ifdef SPI_BUFFER_INTERFACE
	// Initialization Logic for SPI Slave Buffer Interface goes here ...
	for (i=0; i<SPI_BUFFER_SIZE; i++)
	{
 		SPI_Rx_Buffer[i] = 0;                  	// Clear SPI Receive Buffer 
 		SPI_Tx_Buffer[i] = 0;                  	// Clear SPI Transmit Buffer   
	}
	
#endif

 #ifdef SPI_NETWORK_INTERFACE
	// Initialization Logic for SPI Slave Network Interface goes here ...
	for (i=0; i<SPI_BUFFER_SIZE; i++)
	{
 		SPI_Rx_Buffer[i] = 0;                  	// Clear SPI Receive Buffer 
 		SPI_Tx_Buffer[i] = 0;                  	// Clear SPI Transmit Buffer   
	}
	
#endif
    
   // Initialize SPI slave variables and hardware

   InitializeSPISlave();
   
   // Enable SPI MSSP interrupts

   EnableSPIInterrupts();

   while(1)
   {
 		// Clear the watchdog timer each time through the major menu loop

		ClrWdt();
		
#ifdef SPI_BUFFER_INTERFACE
		
   		if (SPI_Rx_Flag == TRUE)
   		{
   			SPI_Rx_Flag = FALSE;

#ifdef TEST   			
			printf("Slave: Mainline, SPI_Interrupt_Count = ");
			dec(SPI_Interrupt_Count);
			printf("\r\n");
			
			// Display contents of SPI Buffers 	
			
			for (i=0; i<SPI_BUFFER_SIZE; i++)
			{
				printf("Slave: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);
			}
			
      		printf("Slave: Mainline, Received DC Motor ");
#endif        	
        	// Process the selected command
			        	
 			switch (MotorMessage.SelectedCommand.Command)
 			{
				case CALIBRATE_MOTOR_COMMAND:
				{
					//**********************************************************************************
					// Build the DC Motor Controller Calibrate Command
					//**********************************************************************************
/*	
					MotorMessage.CalibrateCommand.MessageID = MessageIndex++;	// Message ID range (0000 - FFFF) Hex
					MotorMessage.CalibrateCommand.Command = CALIBRATE_MOTOR_COMMAND;
					MotorMessage.CalibrateCommand.MotorID = 1;
*/
#ifdef TEST
      				printf("Calibration Command...\r\n");
  					printf("MessageID = ");
        			dec(MotorMessage.CalibrateCommand.MessageID);
        			printf("  Command = CALIBRATE_MOTOR_COMMAND");
         			printf(",  Motor ID = ");
        			dec(MotorMessage.CalibrateCommand.MotorID);
        			printf("\r\n\r\n");
#endif
				
					break;
				}
			
				case DUTY_COMMAND:
				{
		
					//**********************************************************************************
					// Build the Send PWM duty command message 
					//**********************************************************************************
/*	
					MotorMessage.DutyCommand.MessageID = MessageIndex++;		// Message ID range (0000 - FFFF) Hex 
  					MotorMessage.DutyCommand.Command = SEND_DUTY_COMMAND;  		// Duty Command to SPI Slave (see enumeration
               			                 									// declaration above.)                                 	 
 					MotorMessage.DutyCommand.MotorID = 0; 						// Selected motor range (0..1) Hex
 					MotorMessage.DutyCommand.Duty = 1000 - MessageIndex; 		// Duty Cycle range (0000 - FFFF) Hex              	
*/
#ifdef TEST
      				printf("PWM Duty Command...\r\n");
  					printf("MessageID = ");
        			dec(MotorMessage.DutyCommand.MessageID);
        			printf("  Command = SEND_DUTY_COMMAND");
        			printf(",  Motor ID = ");
        			dec(MotorMessage.DutyCommand.MotorID);
        			printf(", Duty = ");
        			dec(MotorMessage.DutyCommand.Duty);  
        			printf("\r\n\r\n");
#endif
 					break;
				}
			
				case GET_POT_COMMAND:
				{
	
					//**********************************************************************************
 					// Build the Get Pot command message
					//**********************************************************************************
/*	
					MotorMessage.GetPotCommand.MessageID = MessageIndex++; 		// Message ID (0000 - FFFF) Hex
					MotorMessage.GetPotCommand.Command = GET_POT_COMMAND;		// Get Pot Command to SPI Slave (see enumeration
        		                        									// declaration above.)  
 
					MotorMessage.GetPotCommand.MotorID = 0;						// Selected motor range (0..1)
*/
#ifdef TEST
      				printf("Get Pot Command...\r\n");
  					printf("MessageID = ");
        			dec(MotorMessage.GetPotCommand.MessageID);
        			printf("  Command = GET_POT_COMMAND");
         			printf(",  Motor ID = ");
        			dec(MotorMessage.GetPotCommand.MotorID);
        			printf("\r\n\r\n");
#endif

 					break;
				}
			
				case HOLD_POT_COMMAND:
				{

					//**********************************************************************************
					// Build Hold Pot Command message
					//**********************************************************************************
/*
		
					MotorMessage.HoldPotCommand.MessageID = MessageIndex++; 	// Message ID (0000 - FFFF) Hex

  					MotorMessage.HoldPotCommand.Command = HOLD_POT_COMMAND;     // Hold current Pot command  
  	     			                           								// (see enumeration declaration above.)  
 		
 					MotorMessage.HoldPotCommand.MotorID = 1; 					// Selected motor range (0..1)  
*/
#ifdef TEST
      				printf("Hold Pot Command...\r\n");
  					printf("MessageID = ");
        			dec(MotorMessage.HoldPotCommand.MessageID );
        			printf("  Command = HOLD_POT_COMMAND");
         			printf(",  Motor ID = ");
        			dec(MotorMessage.HoldPotCommand.MotorID);
        			printf("\r\n\r\n");
#endif 
  					break;
				}
			
				case NUMBER_SAMPLES_COMMAND:
				{

					//**********************************************************************************
 					// Build Get number of samples command message
					//**********************************************************************************
/*
 					MotorMessage.SampleCommand.MessageID = MessageIndex++;   	// Message ID (0000 - FFFF) Hex

  					MotorMessage.SampleCommand.Command = 
  						NUMBER_SAMPLES_COMMAND; 								// Get number of samples  
  	                                							 			// Command (see enumeration declaration above.)  
  
 					MotorMessage.SampleCommand.MotorID = 1; 					// Selected motor range (0..1)  
					MotorMessage.SampleCommand.NumberOfSamples = 
						1000 - MessageIndex;									// Number of samples 
 
*/ 
#ifdef TEST
      				printf("Number of Samples Command...\r\n");
  					printf("MessageID = ");
        			dec(MotorMessage.SampleCommand.MessageID);
        			printf("  Command = NUMBER_SAMPLES_COMMAND");
        			printf(",  Motor ID = ");
        			dec(MotorMessage.SampleCommand.MotorID);
        			printf(", Number of Samples = ");
        			dec(MotorMessage.SampleCommand.NumberOfSamples);  
        			printf("\r\n\r\n");
	
#endif 
 
  					break;
				}

				case PERIOD_COMMAND:
				{
					//**********************************************************************************
 					// Build the Send period command message
					//**********************************************************************************
/*
  					MotorMessage.PeriodCommand.MessageID = MessageIndex++;   	// Message ID range (0000 - FFFF) Hex 

  					MotorMessage.PeriodCommand.Command = SEND_PERIOD_COMMAND;   // Send period command to selected motor (see enumeration
                                									// declaration above.)  
 
 					MotorMessage.PeriodCommand.MotorID = 0; 					// Selected motor range (0..1) Hex
 					MotorMessage.PeriodCommand.Period = 1000 - MessageIndex; 	// Period range xx (00..FF) Hex              	
*/
#ifdef TEST
      				printf("PWM Period Command...\r\n");
  					printf("MessageID = ");
        			dec(MotorMessage.PeriodCommand.MessageID);
        			printf("  Command = SEND_PERIOD_COMMAND");
        			printf(",  Motor ID = ");
        			dec(MotorMessage.PeriodCommand.MotorID);
        			printf(", Period = ");
        			dec(MotorMessage.PeriodCommand.Period);  
        			printf("\r\n\r\n");
#endif

   					break;
				}
			}
								
 			
 			
 			
 			
//#ifdef TEST
//*******************************************************************************
// Test code
/*        				
			// Display the current contents of the buffer
			for (i=0; i<SPI_Number_Characters; i++)
			{
				printf("Slave: i = ");
				dec(i);
				printf("  SPI_Tx_Buffer = ");
				dec(SPI_Tx_Buffer[i]);
			
				printf("  SPI_Rx_Buffer = ");
				dec(SPI_Rx_Buffer[i]);
			
				printf("\r\n");
			
				//pause(1000);

			}

*/

//*******************************************************************************
//#endif

 		}
#endif 		
 		
  		// Check to see if the SPI Co-Processor Slave Select is asserted (=0).
      	// Disable the SPI interrupts if it is not.
/*
      	if (!SS)
      	{
      		// Enable SPI MSSP interrupts

			EnableInterrupts();
      	}
      	else
      	{
 		 	// Disable SPI MSSP interrupts

			DisableInterrupts();
     	}
*/

   }
    
/*        
   while (1)
   {
   	// Clear the BF and SSPIF flag bits by reading the SSPBUF register

   	Data = SSPBUF;  
    
   	PIR1bits.SSPIF = 0;  
   	
   	// Wait for receive to end
      
      while (SSPSTATbits.BF);
      
      // Check if last byte received was a data byte
      
      if (SSPSTATbits.D_A)
      {
      	Data = SSPBUF;
      }
      printf("Data = ");
      dec(Data);
      printf("\r\n");
          
 		//PORTB = Data;                                // Write value to PORTB LED(s)
 	}
   
 
   while (1)
   {
 		PORTB = CurrentState;                                // Write value to PORTB LED(s)
      
      printf("Slave: SPI_Interrupt_Count = ");
      dec(SPI_Interrupt_Count);
      printf("\r\n");
 		
      printf("Slave: CurrentState = ");
      dec(CurrentState);
      printf("\r\n");
     
   }

 

   for (i=0; i<255; i++)
   {
       printf("This is a test ");
       dec(i);
       printf(" times ! \r\n");
       PORTB = i;
       pause(500);
   }
*/
    
/*   
   printf("PIC18C452 SPI Slave Controller Application... \r\n");
   
   // Test LED(s)
   
   for (i=0; i<256; i++)
   {
    	PORTB = i;
   	pause(200);
   	
      printf("Slave: i = ");
      dec(i);
      printf("\r\n");
      
   }
*/   
  

 

	// close the spi port

	CloseSPI();

	// Done.

	while(1);
}
 

#endif
 
//******************************************************************************
//* InitializeSPISlave - Initialize variables and hardware used by the SPI 
//* slave device.
//******************************************************************************

void InitializeSPISlave(void)
{
    int i;                    // Loop index


    // Initialize SPI device address pointer

    AddressPointer = 0;
    SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts                           

    // Initialize ring buffer to zero  
 
    for (i=0; i< SPI_BUFFER_SIZE; i++)
    {

       SPI_Rx_Buffer[i] = 0;
       SPI_Tx_Buffer[i] = 0;
    }    
     // configure port A for all digital inputs
 
    OpenADC(ADC_0ANA, ADC_INT_OFF);
   
    // Initialize ports  

    PORTA = 0x00;                                          // Clear PORTA
    PORTB = 0x00;                                          // Clear PORTB
    //PORTC = 0x00;                                        // Clear PORTC (Some Port C bits are reserved for USART)
    PORTD = 0x00;                                          // Clear PORTD
    PORTE = 0x00;                                          // Clear PORTE

    // Initialize port I/O direction for output

 
    TRISA = 0x00;                                          // PORT A all outputs.
    TRISB = 0x00;                                          // PORT B all outputs.
    //TRISC = 0x00;                                        // PORTC all outputs (Some Port C bits are reserved for USART).
    TRISD = 0x00;                                          // PORTD all outputs.
    TRISE = 0x00;                                          // PORTE all outputs.

	// Initialize the SPI Slave Pin directions
/*	
	SPI_SCL_DIR = INPUT; 									// Set SPI Clock Direction to Input
 	SPI_SI_DIR = INPUT;										// Set SPI Input Direction  to Input
    SPI_SO_DIR = OUTPUT; 									// Set SPI Output Direction to Output

    SS_DIR = INPUT;									  		// Set Slave Select line to Input
*/

    //TRACE0_DIR   = 0;									    // Set TRACE 0 line to output

    // Turn OFF trace 0

    //TRACE0 = OFF;

    // Configure the PIC SPI module as a slave
        
    OpenSPI(SLV_SSOFF, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                           // SPI Mode 0,0, and Sample on falling edge.

    //OpenSPI(SLV_SSON, MODE_00, SMPEND );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 0,0, and Sample on falling edge.

    //OpenSPI(SLV_SSOFF, MODE_11, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit disabled and use SPI
                                                           // SPI Mode 1,1, and Sample the middle.

    //OpenSPI(SLV_SSON, MODE_11, SMPMID );	               // Setup MSSP for slave SPI with Slave Select bit enabled and use SPI
                                                           // SPI Mode 1,1, and Sample the middle.


    PIR1 = 0;

    SSPSTAT = 0;

	SPI_Interrupt_Count = 0;                      // Number of MSSP interrupts
	Index = 0;                               // SPI Ring Buffer index
	SPI_Rx_Flag =  FALSE;
	//NumberCharacters = 0;

	CurrentState = 0;                        // Current SPI state variable
 
   // Configure USART interrupts (1 = High Priority, 0 = Low Priority).

   //IPR1bits.RCIP = 1;

	// Configure interrupts (1 = High Priority, 0 = Low Priority).  Interrupt 0 is always
	// set to high priority

	//INTCON3bits.INT1IP = 1;                                // Set interrupt 1 priority to high
	//INTCON3bits.INT2IP = 1;                                // Set interrupt 2 priority to high

	// Configure MSSP interrupts (1 = High Priority, 0 = Low Priority).

   IPR1bits.SSPIP = 1;

   // Clear the BF and SSPIF flag bits by reading the SSPBUF register

   SSPBUF = 0;
    
   PIR1bits.SSPIF = 0;
    
 

/*
   // Clear the BF and SSPIF flag bits by reading the SSPBUF register

    i = SSPBUF;  
    
    PIR1bits.SSPIF = 0;    
*/
 

 }

/********************************************************************
*     Function Name:    ReadSPIByte                                 *
*     Return Value:     contents of SSPBUF register                 *
*     Parameters:       void                                        *
*     Description:      Read single byte from SPI bus (Slave).      *
********************************************************************/
unsigned char ReadSPIByte( void )
{
         
  if ( SSPCON1bits.SSPOV )        		// test if new byte received before previously 
                                    	// received byte has been read.
  {
  	SSPCON1bits.SSPOV = 0;				// Clear the SSPOV flag
   	return ( -1 );              		// if SSPOV bit was set return negative #
  }
 
  return ( SSPBUF );              		// return with byte read 
}

/********************************************************************
*     Function Name:    WriteSPIByte                                *
*     Return Value:     Status byte for WCOL detection.             *
*     Parameters:       Single data byte for SPI bus.               *
*     Description:      This routine writes a single byte to the    * 
*                       SPI bus (Slave).                            *
********************************************************************/
unsigned char WriteSPIByte( unsigned char data_out )
{
  SSPCON1bits.WCOL = 0;	   			// Clear the WCOL flag.

  SSPBUF = data_out;           		// write byte to SSPBUF register
 
  if ( SSPCON1bits.WCOL )        	// test if write collision occurred
  {
   	return ( -1 );              	// if WCOL bit is set return negative #
  }
/*
  else
  {
    while( !SSPSTATbits.BF );  // wait until bus cycle complete 
  }
*/ 
  return ( 0 );                // if WCOL bit is not set return non-negative#
}

/********************************************************************
*     Function Name:    ReadSPIString                               *
*     Return Value:     void                                        *
*     Parameters:       address of read string storage location and *
*                       length of string bytes to read              *
*     Description:      This routine reads a string from the SPI    *
*                       bus (Slave).  The number of bytes to read   *
*                       is determined by parameter 'length'.        *
********************************************************************/
void ReadSPIString( unsigned char *rdptr, unsigned char length )
{
  while ( length )                		// stay in loop until length = 0
  {
    *rdptr++ = ReadSPIByte();         	// read a single byte
    length--;                     		// reduce string length count by 1
  }
}

/********************************************************************
*     Function Name:    WriteSPIString                              *
*     Return Value:     void                                        *
*     Parameters:       address of write string storage location    *
*     Description:      This routine writes a string to the SPI bus.*  
********************************************************************/
void WriteSPIString( unsigned char *wrptr )
{
  while ( *wrptr )                		// test for string null character
  {
     SSPBUF = *wrptr++;           		// initiate SPI bus cycle
     while( !SSPSTATbits.BF );    		// wait until 'BF' bit is set
  }
}
 
//********************************************************************************
//* EnableSPIInterrupts - Enable interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void EnableSPIInterrupts(void)
{

	// Enable MSSP interrupts
 	PIE1bits.SSPIE = 1;

    // Enable USART asynchronous interrupt

    //PIE1bits.RCIE = 1;

  	// Enable peripheral interrupts
   INTCONbits.PEIE = 1;

    // Enable general interrupts
    INTCONbits.GIE = 1;                                    // Enable all interrupts
}

// SPI input with interrupts and SPI buffer

//******************************************************************************
//* SPI_Get_Byte - Get next character from the SPI Rx Buffer.  The function
//* returns a -1 if the routine times out. 
//******************************************************************************
 
byte	SPI_Get_Byte(void)
{

	char	Data[2];                                     // Data read from USART
    	
	// Disable general interrupts

   //INTCONbits.GIE = 0;                                    // Disable all interrupts

	Data[1] = 0;
    
   // Get next valid data character from the serial input ring buffer
       
	// Check for and filter out any CR characters that indicate the end of serial input

   while (SPI_Rx_Buffer[SPI_Start_Buffer] == CR)
	{
		SPI_Start_Buffer++;
      SPI_Start_Buffer = SPI_Start_Buffer & (SPI_BUFFER_SIZE - 1);    // Wrap the StartBuffer pointer if necessar

 	}

   //Data = SPI_Rx_Buffer[SPI_Start_Buffer];
   Data[0] = SPI_Rx_Buffer[SPI_Start_Buffer];

   SPI_Start_Buffer++;
   SPI_Start_Buffer = SPI_Start_Buffer & (SPI_BUFFER_SIZE - 1);    // Wrap the StartBuffer pointer if necessary

#ifdef TEST
/*     
   printf("trace 7:  SPI_Rx_Flag = ");
   dec(SPI_Rx_Flag);
   printf(" SPI_Number_Characters = ");
   dec(SPI_Number_Characters);
   printf(" Data = !");
   putchar(Data[0]);
   printf("!\r\n");
*/
#endif

   // Enable general interrupts

   //INTCONbits.GIE = 1;                                    // Enable all interrupts
 

   return (Data[0]);
}
//********************************************************************************
//* DisableSPIInterrupts - Disable interrupts for serial I/O and for other peripherals
//* that require them.
//********************************************************************************

void DisableSPIInterrupts(void)
{


    // Disable general interrupts

    INTCONbits.GIE = 0;                                    // Disable all interrupts
}
